@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2019 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -19,8 +19,6 @@
 **/
 
 
-require_once dirname(__FILE__).'/../../include/hostgroups.inc.php';
-
 class CControllerWidgetTrigOverView extends CControllerWidget {
 
 	public function __construct() {
@@ -50,7 +48,8 @@ class CControllerWidgetTrigOverView extends CControllerWidget {
 		];
 
 		list($data['hosts'], $data['triggers']) = getTriggersOverviewData(getSubGroups($fields['groupids']),
-			$fields['application'], $fields['style'], [], $trigger_options, $fields['show_suppressed']
+			$fields['application'], $fields['style'], [], $trigger_options,
+			['show_suppressed' => $fields['show_suppressed']]
 		);
 
 		$this->setResponse(new CControllerResponseData($data));
