@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2019 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -18,13 +18,14 @@
 ** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 **/
 
+
 $user_navigation = (new CList())
 	->addClass(ZBX_STYLE_TOP_NAV_ICONS)
 	->addItem(
 		(new CForm('get', 'search.php'))
 			->cleanItems()
 			->addItem([
-				(new CTextBox('search', '', false, 255))
+				(new CTextBox('search', getRequest('search', ''), false, 255))
 					->setAttribute('autocomplete', 'off')
 					->addClass(ZBX_STYLE_SEARCH)
 					->setAttribute('aria-label', _('type here to search')),
@@ -41,7 +42,7 @@ $user_menu = (new CList())
 	->addItem(CBrandHelper::isRebranded()
 		? null
 		: (new CListItem(
-			(new CLink('Support', 'https://www.zabbix.com/support/'))
+			(new CLink(_('Support'), $data['support_url']))
 				->addClass(ZBX_STYLE_TOP_NAV_SUPPORT)
 				->setAttribute('target', '_blank')
 				->setTitle(_('Zabbix Technical Support'))
