@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2019 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -29,25 +29,31 @@ require_once dirname(__FILE__).'/include/page_header.php';
 
 // VAR	TYPE	OPTIONAL	FLAGS	VALIDATION	EXCEPTION
 $fields = [
-	'period' =>			[T_ZBX_INT, O_OPT, P_NZERO,	BETWEEN(ZBX_MIN_PERIOD, ZBX_MAX_PERIOD), null],
-	'stime' =>			[T_ZBX_INT, O_OPT, P_NZERO,	null,				null],
-	'isNow' =>			[T_ZBX_INT, O_OPT, null,	IN('0,1'),			null],
-	'profileIdx' =>		[T_ZBX_STR, O_OPT, null,	null,				null],
-	'profileIdx2' =>	[T_ZBX_STR, O_OPT, null,	null,				null],
-	'updateProfile' =>	[T_ZBX_STR, O_OPT, null,	null,				null],
-	'name' =>			[T_ZBX_STR, O_OPT, null,	null,				null],
-	'width' =>			[T_ZBX_INT, O_OPT, null,	BETWEEN(20, 65535),	null],
-	'height' =>			[T_ZBX_INT, O_OPT, null,	BETWEEN(0, 65535),	null],
-	'graphtype' =>		[T_ZBX_INT, O_OPT, null,	IN('2,3'),			null],
-	'graph3d' =>		[T_ZBX_INT, O_OPT, P_NZERO,	IN('0,1'),			null],
-	'legend' =>			[T_ZBX_INT, O_OPT, P_NZERO,	IN('0,1'),			null],
-	'items' =>			[T_ZBX_STR, O_OPT, null,	null,				null]
+	'from' =>			[T_ZBX_RANGE_TIME,	O_OPT, P_SYS,	null,				null],
+	'to' =>				[T_ZBX_RANGE_TIME,	O_OPT, P_SYS,	null,				null],
+	'profileIdx' =>		[T_ZBX_STR,			O_OPT, null,	null,				null],
+	'profileIdx2' =>	[T_ZBX_STR,			O_OPT, null,	null,				null],
+	'name' =>			[T_ZBX_STR,			O_OPT, null,	null,				null],
+	'width' =>			[T_ZBX_INT,			O_OPT, null,	BETWEEN(CPieGraphDraw::GRAPH_WIDTH_MIN, 65535),		null],
+	'height' =>			[T_ZBX_INT,			O_OPT, null,	BETWEEN(CPieGraphDraw::GRAPH_HEIGHT_MIN, 65535),	null],
+	'graphtype' =>		[T_ZBX_INT,			O_OPT, null,	IN('2,3'),			null],
+	'graph3d' =>		[T_ZBX_INT,			O_OPT, P_NZERO,	IN('0,1'),			null],
+	'legend' =>			[T_ZBX_INT,			O_OPT, null,	IN('0,1'),			null],
+	'items' =>			[T_ZBX_STR,			O_OPT, null,	null,				null],
+	'widget_view' =>	[T_ZBX_INT,			O_OPT, null,	IN('0,1'),			null]
 ];
 if (!check_fields($fields)) {
 	exit();
 }
+validateTimeSelectorPeriod(getRequest('from'), getRequest('to'));
 
 $items = getRequest('items', []);
+
+if (!$items) {
+	show_error_message(_('No items defined.'));
+	exit();
+}
+
 CArrayHelper::sort($items, ['sortorder']);
 
 /*
@@ -88,23 +94,27 @@ foreach ($items as $item) {
 /*
  * Display
  */
-$timeline = calculateTime([
-	'profileIdx' => getRequest('profileIdx', 'web.screens'),
+$timeline = getTimeSelectorPeriod([
+	'profileIdx' => getRequest('profileIdx'),
 	'profileIdx2' => getRequest('profileIdx2'),
-	'updateProfile' => (getRequest('updateProfile', '0') === '1'),
-	'period' => getRequest('period'),
-	'stime' => getRequest('stime'),
-	'isNow' => getRequest('isNow')
+	'from' => getRequest('from'),
+	'to' => getRequest('to')
 ]);
 
 $graph = new CPieGraphDraw(getRequest('graphtype', GRAPH_TYPE_NORMAL));
 $graph->setHeader(getRequest('name', ''));
-$graph->setPeriod($timeline['period']);
-$graph->setSTime($timeline['stime']);
+$graph->setPeriod($timeline['to_ts'] - $timeline['from_ts']);
+$graph->setSTime($timeline['from_ts']);
 
 if (!empty($_REQUEST['graph3d'])) {
 	$graph->switchPie3D();
 }
+
+if (getRequest('widget_view') === '1') {
+	$graph->draw_header = false;
+	$graph->with_vertical_padding = false;
+}
+
 $graph->showLegend(getRequest('legend', 0));
 $graph->setWidth(getRequest('width', 400));
 $graph->setHeight(getRequest('height', 300));
