@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2016 Zabbix SIA
+** Copyright (C) 2001-2019 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -44,10 +44,13 @@ $fields = [
 	'key' =>						[T_ZBX_STR, O_OPT, null,	NOT_EMPTY,	'isset({add}) || isset({update})',
 		_('Key')
 	],
-	'delay' =>						[T_ZBX_INT, O_OPT, null,	BETWEEN(0, SEC_PER_DAY),
+	'master_itemid' =>				[T_ZBX_STR, O_OPT, null,	null,
+		'(isset({add}) || isset({update})) && isset({type}) && {type}=='.ITEM_TYPE_DEPENDENT, _('Master item')],
+	'delay' =>						[T_ZBX_TU, O_OPT, P_ALLOW_USER_MACRO | P_ALLOW_LLD_MACRO, null,
 		'(isset({add}) || isset({update}))'.
-			' && (isset({type}) && ({type} != '.ITEM_TYPE_TRAPPER.' && {type} != '.ITEM_TYPE_SNMPTRAP.'))',
-		_('Update interval (in sec)')
+			' && (isset({type}) && ({type} != '.ITEM_TYPE_TRAPPER.' && {type} != '.ITEM_TYPE_SNMPTRAP.')'.
+			' && {type}!='.ITEM_TYPE_DEPENDENT.')',
+		_('Update interval')
 	],
 	'delay_flex' =>					[T_ZBX_STR, O_OPT, null,	null,			null],
 	'status' =>						[T_ZBX_INT, O_OPT, null,	IN(ITEM_STATUS_ACTIVE), null],
@@ -55,15 +58,11 @@ $fields = [
 		IN([-1, ITEM_TYPE_ZABBIX, ITEM_TYPE_SNMPV1, ITEM_TYPE_TRAPPER, ITEM_TYPE_SIMPLE, ITEM_TYPE_SNMPV2C,
 			ITEM_TYPE_INTERNAL, ITEM_TYPE_SNMPV3, ITEM_TYPE_ZABBIX_ACTIVE, ITEM_TYPE_AGGREGATE, ITEM_TYPE_EXTERNAL,
 			ITEM_TYPE_DB_MONITOR, ITEM_TYPE_IPMI, ITEM_TYPE_SSH, ITEM_TYPE_TELNET, ITEM_TYPE_JMX, ITEM_TYPE_CALCULATED,
-			ITEM_TYPE_SNMPTRAP]
-		),
+			ITEM_TYPE_SNMPTRAP, ITEM_TYPE_DEPENDENT, ITEM_TYPE_HTTPAGENT
+		]),
 		'isset({add}) || isset({update})'
 	],
 	'value_type' =>					[T_ZBX_INT, O_OPT, null,	IN('0,1,2,3,4'), 'isset({add}) || isset({update})'],
-	'data_type' =>					[T_ZBX_INT, O_OPT, null,
-		IN(ITEM_DATA_TYPE_DECIMAL.','.ITEM_DATA_TYPE_OCTAL.','.ITEM_DATA_TYPE_HEXADECIMAL.','.ITEM_DATA_TYPE_BOOLEAN),
-		'(isset({add}) || isset({update})) && (isset({value_type}) && ({value_type} == '.ITEM_VALUE_TYPE_UINT64.'))'
-	],
 	'valuemapid' =>					[T_ZBX_INT, O_OPT, null,	DB_ID,
 		'(isset({add}) || isset({update})) && isset({value_type})'.
 			' && '.IN(ITEM_VALUE_TYPE_FLOAT.','.ITEM_VALUE_TYPE_UINT64, 'value_type')
@@ -139,21 +138,13 @@ $fields = [
 		'(isset({add}) || isset({update})) && isset({type}) && ({type} == 2)'
 	],
 	'units' =>						[T_ZBX_STR, O_OPT, null,	null,
-		'(isset({add}) || isset({update})) && isset({value_type})'.
-			' && '.IN('0,3', 'value_type').' (isset({data_type}) && ({data_type} != '.ITEM_DATA_TYPE_BOOLEAN.'))'
-	],
-	'multiplier' =>					[T_ZBX_INT, O_OPT, null,	null,		null],
-	'delta' =>						[T_ZBX_INT, O_OPT, null,	IN('0,1,2'),
-		'(isset({add}) || isset({update})) && isset({value_type})'.
-			' && '.IN('0,3', 'value_type').' (isset({data_type}) && ({data_type} != '.ITEM_DATA_TYPE_BOOLEAN.'))'
-	],
-	'formula' =>					[T_ZBX_DBL_STR, O_OPT, null,
-		'({value_type} == 0 && {} != 0) || ({value_type} == 3 && {} > 0)',
-		'(isset({add}) || isset({update})) && isset({multiplier}) && {multiplier} == 1', _('Custom multiplier')
+		'(isset({add}) || isset({update})) && isset({value_type}) && '.
+		IN(ITEM_VALUE_TYPE_FLOAT.','.ITEM_VALUE_TYPE_UINT64, 'value_type')
 	],
 	'logtimefmt' =>					[T_ZBX_STR, O_OPT, null,	null,
 		'(isset({add}) || isset({update})) && (isset({value_type}) && ({value_type} == 2))'
 	],
+	'preprocessing' =>				[T_ZBX_STR, O_OPT, P_NO_TRIM,	null,	null],
 	'group_itemid' =>				[T_ZBX_INT, O_OPT, null,	DB_ID,		null],
 	'new_application' =>			[T_ZBX_STR, O_OPT, null,	null,		'isset({add}) || isset({update})'],
 	'applications' =>				[T_ZBX_INT, O_OPT, null,	DB_ID,		null],
@@ -161,14 +152,80 @@ $fields = [
 		'isset({parent_discoveryid}) && (isset({add}) || isset({update}))'
 	],
 	'application_prototypes' =>		[T_ZBX_STR, O_OPT, null,	null,		null],
-	'history' =>					[T_ZBX_INT, O_OPT, null,	BETWEEN(0, 65535), 'isset({add}) || isset({update})',
+	'history_mode' =>				[T_ZBX_INT, O_OPT, null,	IN([ITEM_STORAGE_OFF, ITEM_STORAGE_CUSTOM]), null],
+	'history' =>					[T_ZBX_STR, O_OPT, null,	null,
+		'(isset({add}) || isset({update})) && isset({history_mode}) && {history_mode}=='.ITEM_STORAGE_CUSTOM,
 		_('History storage period')
 	],
-	'trends' =>						[T_ZBX_INT, O_OPT, null,	BETWEEN(0, 65535),
-		'(isset({add}) || isset({update})) && isset({value_type})'.
-			' && '.IN(ITEM_VALUE_TYPE_FLOAT.','.ITEM_VALUE_TYPE_UINT64, 'value_type'),
+	'trends_mode' =>				[T_ZBX_INT, O_OPT, null,	IN([ITEM_STORAGE_OFF, ITEM_STORAGE_CUSTOM]), null],
+	'trends' =>						[T_ZBX_STR, O_OPT, null,	null,
+		'(isset({add}) || isset({update})) && isset({trends_mode}) && {trends_mode}=='.ITEM_STORAGE_CUSTOM.
+			' && isset({value_type}) && '.IN(ITEM_VALUE_TYPE_FLOAT.','.ITEM_VALUE_TYPE_UINT64, 'value_type'),
 		_('Trend storage period')
 	],
+	'jmx_endpoint' =>				[T_ZBX_STR, O_OPT, null,	NOT_EMPTY,
+		'(isset({add}) || isset({update})) && isset({type}) && {type} == '.ITEM_TYPE_JMX
+	],
+	'timeout' =>					[T_ZBX_STR, O_OPT, null,	null,		null],
+	'url' =>						[T_ZBX_STR, O_OPT, null,	NOT_EMPTY,
+		'(isset({add}) || isset({update})) && isset({type}) && {type} == '.ITEM_TYPE_HTTPAGENT, _('URL')],
+	'query_fields' =>				[T_ZBX_STR, O_OPT, null,	null,		null],
+	'posts' =>						[T_ZBX_STR, O_OPT, null,	null,		null],
+	'status_codes' =>				[T_ZBX_STR, O_OPT, null,	null,		null],
+	'follow_redirects' =>			[T_ZBX_INT, O_OPT, null,
+										IN([HTTPTEST_STEP_FOLLOW_REDIRECTS_OFF, HTTPTEST_STEP_FOLLOW_REDIRECTS_ON]),
+										null
+									],
+	'post_type' =>					[T_ZBX_INT, O_OPT, null,
+										IN([ZBX_POSTTYPE_RAW, ZBX_POSTTYPE_JSON, ZBX_POSTTYPE_XML]),
+										null
+									],
+	'http_proxy' =>					[T_ZBX_STR, O_OPT, null,	null,		null],
+	'headers' =>					[T_ZBX_STR, O_OPT, null,	null,		null],
+	'retrieve_mode' =>				[T_ZBX_INT, O_OPT, null,
+										IN([HTTPTEST_STEP_RETRIEVE_MODE_CONTENT, HTTPTEST_STEP_RETRIEVE_MODE_HEADERS,
+											HTTPTEST_STEP_RETRIEVE_MODE_BOTH
+										]),
+										null
+									],
+	'request_method' =>				[T_ZBX_INT, O_OPT, null,
+										IN([HTTPCHECK_REQUEST_GET, HTTPCHECK_REQUEST_POST, HTTPCHECK_REQUEST_PUT,
+											HTTPCHECK_REQUEST_HEAD
+										]),
+										null
+									],
+	'output_format' =>				[T_ZBX_INT, O_OPT, null,	IN([HTTPCHECK_STORE_RAW, HTTPCHECK_STORE_JSON]), null],
+	'allow_traps' =>				[T_ZBX_INT, O_OPT, null,
+										IN([HTTPCHECK_ALLOW_TRAPS_OFF, HTTPCHECK_ALLOW_TRAPS_ON]),
+										null
+									],
+	'ssl_cert_file' =>				[T_ZBX_STR, O_OPT, null,	null,		null],
+	'ssl_key_file' =>				[T_ZBX_STR, O_OPT, null,	null,		null],
+	'ssl_key_password' =>			[T_ZBX_STR, O_OPT, null,	null,		null],
+	'verify_peer' =>				[T_ZBX_INT, O_OPT, null,
+										IN([HTTPTEST_VERIFY_PEER_OFF, HTTPTEST_VERIFY_PEER_ON]),
+										null
+									],
+	'verify_host' =>				[T_ZBX_INT, O_OPT, null,
+										IN([HTTPTEST_VERIFY_HOST_OFF, HTTPTEST_VERIFY_HOST_ON]),
+										null
+									],
+	'http_authtype' =>				[T_ZBX_INT, O_OPT, null,
+										IN([HTTPTEST_AUTH_NONE, HTTPTEST_AUTH_BASIC, HTTPTEST_AUTH_NTLM]),
+										null
+									],
+	'http_username' =>				[T_ZBX_STR, O_OPT, null,	null,
+										'(isset({add}) || isset({update})) && isset({http_authtype})'.
+											' && ({http_authtype} == '.HTTPTEST_AUTH_BASIC.
+												' || {http_authtype} == '.HTTPTEST_AUTH_NTLM.')',
+										_('Username')
+									],
+	'http_password' =>				[T_ZBX_STR, O_OPT, null,	null,
+										'(isset({add}) || isset({update})) && isset({http_authtype})'.
+											' && ({http_authtype} == '.HTTPTEST_AUTH_BASIC.
+												' || {http_authtype} == '.HTTPTEST_AUTH_NTLM.')',
+										_('Password')
+									],
 	// actions
 	'action' =>						[T_ZBX_STR, O_OPT, P_SYS|P_ACT,
 		IN('"itemprototype.massdelete","itemprototype.massdisable","itemprototype.massenable"'), null
@@ -205,8 +262,16 @@ if (!$discoveryRule) {
 }
 
 $itemPrototypeId = getRequest('itemid');
-if ($itemPrototypeId && !API::ItemPrototype()->isWritable([$itemPrototypeId])) {
-	access_deny();
+if ($itemPrototypeId) {
+	$item_prorotypes = API::ItemPrototype()->get([
+		'output' => [],
+		'itemids' => $itemPrototypeId,
+		'editable' => true
+	]);
+
+	if (!$item_prorotypes) {
+		access_deny();
+	}
 }
 
 /*
@@ -252,26 +317,40 @@ elseif (hasRequest('add') || hasRequest('update')) {
 		}
 	}
 
+	$delay = getRequest('delay', DB::getDefault('items', 'delay'));
+	$type = getRequest('type', ITEM_TYPE_ZABBIX);
+
 	/*
-	 * Intially validate "delay_flex" field one by one to make sure it does not have interval separator ";".
-	 * Skip empty fields and convert "delay_flex" array to string glued with ";" which is later validated through API.
+	 * "delay_flex" is a temporary field that collects flexible and scheduling intervals separated by a semicolon.
+	 * In the end, custom intervals together with "delay" are stored in the "delay" variable.
 	 */
-	$delay_flex = '';
-	$intervals = [];
+	if (!in_array($type, [ITEM_TYPE_ZABBIX_ACTIVE, ITEM_TYPE_TRAPPER, ITEM_TYPE_SNMPTRAP]) && hasRequest('delay_flex')) {
+		$intervals = [];
+		$simple_interval_parser = new CSimpleIntervalParser([
+			'usermacros' => true,
+			'lldmacros' => true
+		]);
+		$time_period_parser = new CTimePeriodParser([
+			'usermacros' => true,
+			'lldmacros' => true
+		]);
+		$scheduling_interval_parser = new CSchedulingIntervalParser([
+			'usermacros' => true,
+			'lldmacros' => true
+		]);
 
-	if (getRequest('delay_flex')) {
 		foreach (getRequest('delay_flex') as $interval) {
-			if ($interval['type'] == ITEM_DELAY_FLEX_TYPE_FLEXIBLE) {
+			if ($interval['type'] == ITEM_DELAY_FLEXIBLE) {
 				if ($interval['delay'] === '' && $interval['period'] === '') {
 					continue;
 				}
 
-				if (strpos($interval['delay'], ';') !== false) {
+				if ($simple_interval_parser->parse($interval['delay']) != CParser::PARSE_SUCCESS) {
 					$result = false;
 					info(_s('Invalid interval "%1$s".', $interval['delay']));
 					break;
 				}
-				elseif (strpos($interval['period'], ';') !== false) {
+				elseif ($time_period_parser->parse($interval['period']) != CParser::PARSE_SUCCESS) {
 					$result = false;
 					info(_s('Invalid interval "%1$s".', $interval['period']));
 					break;
@@ -284,7 +363,7 @@ elseif (hasRequest('add') || hasRequest('update')) {
 					continue;
 				}
 
-				if (strpos($interval['schedule'], ';') !== false) {
+				if ($scheduling_interval_parser->parse($interval['schedule']) != CParser::PARSE_SUCCESS) {
 					$result = false;
 					info(_s('Invalid interval "%1$s".', $interval['schedule']));
 					break;
@@ -295,7 +374,7 @@ elseif (hasRequest('add') || hasRequest('update')) {
 		}
 
 		if ($intervals) {
-			$delay_flex = join(';', $intervals);
+			$delay .= ';'.implode(';', $intervals);
 		}
 	}
 
@@ -319,13 +398,39 @@ elseif (hasRequest('add') || hasRequest('update')) {
 			$application_prototypes[] = ['name' => $new_application_prototype];
 		}
 
+		$preprocessing = getRequest('preprocessing', []);
+
+		foreach ($preprocessing as &$step) {
+			switch ($step['type']) {
+				case ZBX_PREPROC_MULTIPLIER:
+					$step['params'] = trim($step['params'][0]);
+					break;
+
+				case ZBX_PREPROC_RTRIM:
+				case ZBX_PREPROC_LTRIM:
+				case ZBX_PREPROC_TRIM:
+				case ZBX_PREPROC_XPATH:
+				case ZBX_PREPROC_JSONPATH:
+					$step['params'] = $step['params'][0];
+					break;
+
+				case ZBX_PREPROC_REGSUB:
+					$step['params'] = implode("\n", $step['params']);
+					break;
+
+				default:
+					$step['params'] = '';
+			}
+		}
+		unset($step);
+
 		$item = [
 			'name'			=> getRequest('name'),
 			'description'	=> getRequest('description'),
 			'key_'			=> getRequest('key'),
 			'hostid'		=> $discoveryRule['hostid'],
 			'interfaceid'	=> getRequest('interfaceid'),
-			'delay'			=> getRequest('delay'),
+			'delay'			=> $delay,
 			'status'		=> getRequest('status', ITEM_STATUS_DISABLED),
 			'type'			=> getRequest('type'),
 			'snmp_community' => getRequest('snmp_community'),
@@ -333,11 +438,10 @@ elseif (hasRequest('add') || hasRequest('update')) {
 			'value_type'	=> getRequest('value_type'),
 			'trapper_hosts'	=> getRequest('trapper_hosts'),
 			'port'			=> getRequest('port'),
-			'history'		=> getRequest('history'),
-			'trends'		=> getRequest('trends'),
+			'history'		=> (getRequest('history_mode', ITEM_STORAGE_CUSTOM) == ITEM_STORAGE_OFF)
+				? ITEM_NO_STORAGE_VALUE
+				: getRequest('history'),
 			'units'			=> getRequest('units'),
-			'multiplier'	=> getRequest('multiplier', 0),
-			'delta'			=> getRequest('delta'),
 			'snmpv3_contextname' => getRequest('snmpv3_contextname'),
 			'snmpv3_securityname' => getRequest('snmpv3_securityname'),
 			'snmpv3_securitylevel' => getRequest('snmpv3_securitylevel'),
@@ -345,7 +449,6 @@ elseif (hasRequest('add') || hasRequest('update')) {
 			'snmpv3_authpassphrase' => getRequest('snmpv3_authpassphrase'),
 			'snmpv3_privprotocol' => getRequest('snmpv3_privprotocol'),
 			'snmpv3_privpassphrase' => getRequest('snmpv3_privpassphrase'),
-			'formula'		=> getRequest('formula', '1'),
 			'logtimefmt'	=> getRequest('logtimefmt'),
 			'valuemapid'	=> getRequest('valuemapid'),
 			'authtype'		=> getRequest('authtype'),
@@ -355,24 +458,40 @@ elseif (hasRequest('add') || hasRequest('update')) {
 			'privatekey'	=> getRequest('privatekey'),
 			'params'		=> getRequest('params'),
 			'ipmi_sensor'	=> getRequest('ipmi_sensor'),
-			'data_type'		=> getRequest('data_type'),
-			'ruleid'		=> getRequest('parent_discoveryid'),
-			'delay_flex'	=> $delay_flex
+			'ruleid'		=> getRequest('parent_discoveryid')
 		];
 
+		if ($item['type'] == ITEM_TYPE_JMX) {
+			$item['jmx_endpoint'] = getRequest('jmx_endpoint', '');
+		}
+
+		if ($item['value_type'] == ITEM_VALUE_TYPE_FLOAT || $item['value_type'] == ITEM_VALUE_TYPE_UINT64) {
+			$item['trends'] = (getRequest('trends_mode', ITEM_STORAGE_CUSTOM) == ITEM_STORAGE_OFF)
+				? ITEM_NO_STORAGE_VALUE
+				: getRequest('trends');
+		}
+
+		if ($item['type'] == ITEM_TYPE_DEPENDENT) {
+			$item['master_itemid'] = getRequest('master_itemid');
+		}
+
 		if (hasRequest('update')) {
 			$itemId = getRequest('itemid');
 
-			$dbItem = API::ItemPrototype()->get([
-				'output' => ['type', 'snmp_community', 'snmp_oid', 'hostid', 'name', 'key_', 'delay',
-					'history', 'trends', 'status', 'value_type', 'trapper_hosts', 'units', 'multiplier', 'delta',
-					'snmpv3_securityname', 'snmpv3_securitylevel', 'snmpv3_authpassphrase', 'snmpv3_privpassphrase',
-					'formula', 'logtimefmt', 'templateid', 'valuemapid', 'delay_flex', 'params', 'ipmi_sensor',
-					'data_type', 'authtype', 'username', 'password', 'publickey', 'privatekey',	'interfaceid', 'port',
-					'description', 'snmpv3_authprotocol', 'snmpv3_privprotocol', 'snmpv3_contextname'
+			$db_item = API::ItemPrototype()->get([
+				'output' => ['type', 'snmp_community', 'snmp_oid', 'hostid', 'name', 'key_', 'delay', 'history',
+					'trends', 'status', 'value_type', 'trapper_hosts', 'units', 'snmpv3_securityname',
+					'snmpv3_securitylevel', 'snmpv3_authpassphrase', 'snmpv3_privpassphrase', 'logtimefmt',
+					'templateid', 'valuemapid', 'params', 'ipmi_sensor', 'authtype', 'username', 'password',
+					'publickey', 'privatekey', 'interfaceid', 'port', 'description', 'snmpv3_authprotocol',
+					'snmpv3_privprotocol', 'snmpv3_contextname', 'jmx_endpoint', 'master_itemid', 'timeout', 'url',
+					'query_fields', 'posts', 'status_codes', 'follow_redirects', 'post_type', 'http_proxy', 'headers',
+					'retrieve_mode', 'request_method', 'output_format', 'ssl_cert_file', 'ssl_key_file',
+					'ssl_key_password', 'verify_peer', 'verify_host', 'allow_traps'
 				],
 				'selectApplications' => ['applicationid'],
 				'selectApplicationPrototypes' => ['name'],
+				'selectPreprocessing' => ['type', 'params'],
 				'itemids' => [$itemId]
 			]);
 
@@ -385,23 +504,50 @@ elseif (hasRequest('add') || hasRequest('update')) {
 				$item['snmpv3_privprotocol'] = ITEM_PRIVPROTOCOL_DES;
 			}
 
-			$dbItem = $dbItem[0];
+			$db_item = $db_item[0];
 
-			$item = CArrayHelper::unsetEqualValues($item, $dbItem);
+			if ($item['type'] == ITEM_TYPE_HTTPAGENT) {
+				$http_item = [
+					'timeout' => getRequest('timeout', DB::getDefault('items', 'timeout')),
+					'url' => getRequest('url'),
+					'query_fields' => getRequest('query_fields', []),
+					'posts' => getRequest('posts'),
+					'status_codes' => getRequest('status_codes', DB::getDefault('items', 'status_codes')),
+					'follow_redirects' => (int) getRequest('follow_redirects'),
+					'post_type' => (int) getRequest('post_type'),
+					'http_proxy' => getRequest('http_proxy'),
+					'headers' => getRequest('headers', []),
+					'retrieve_mode' => (int) getRequest('retrieve_mode'),
+					'request_method' => (int) getRequest('request_method'),
+					'output_format' => (int) getRequest('output_format'),
+					'allow_traps' => (int) getRequest('allow_traps', HTTPCHECK_ALLOW_TRAPS_OFF),
+					'ssl_cert_file' => getRequest('ssl_cert_file'),
+					'ssl_key_file' => getRequest('ssl_key_file'),
+					'ssl_key_password' => getRequest('ssl_key_password'),
+					'verify_peer' => (int) getRequest('verify_peer'),
+					'verify_host' => (int) getRequest('verify_host'),
+					'authtype' => getRequest('http_authtype', HTTPTEST_AUTH_NONE),
+					'username' => getRequest('http_username', ''),
+					'password' => getRequest('http_password', '')
+				];
+				$item = prepareItemHttpAgentFormData($http_item) + $item;
+			}
+
+			$item = CArrayHelper::unsetEqualValues($item, $db_item);
 			$item['itemid'] = $itemId;
 
-			$dbItem['applications'] = zbx_objectValues($dbItem['applications'], 'applicationid');
+			$db_item['applications'] = zbx_objectValues($db_item['applications'], 'applicationid');
 
 			// compare applications
-			natsort($dbItem['applications']);
+			natsort($db_item['applications']);
 			natsort($applications);
 
-			if (array_values($dbItem['applications']) !== array_values($applications)) {
+			if (array_values($db_item['applications']) !== array_values($applications)) {
 				$item['applications'] = $applications;
 			}
 
 			// compare application prototypes
-			$db_application_prototype_names = zbx_objectValues($dbItem['applicationPrototypes'], 'name');
+			$db_application_prototype_names = zbx_objectValues($db_item['applicationPrototypes'], 'name');
 			natsort($db_application_prototype_names);
 
 			$application_prototype_names = zbx_objectValues($application_prototypes, 'name');
@@ -411,12 +557,47 @@ elseif (hasRequest('add') || hasRequest('update')) {
 				$item['applicationPrototypes'] = $application_prototypes;
 			}
 
+			if ($db_item['preprocessing'] !== $preprocessing) {
+				$item['preprocessing'] = $preprocessing;
+			}
+
 			$result = API::ItemPrototype()->update($item);
 		}
 		else {
 			$item['applications'] = $applications;
 			$item['applicationPrototypes'] = $application_prototypes;
 
+			if (getRequest('type') == ITEM_TYPE_HTTPAGENT) {
+				$http_item = [
+					'timeout' => getRequest('timeout', DB::getDefault('items', 'timeout')),
+					'url' => getRequest('url'),
+					'query_fields' => getRequest('query_fields', []),
+					'posts' => getRequest('posts'),
+					'status_codes' => getRequest('status_codes', DB::getDefault('items', 'status_codes')),
+					'follow_redirects' => (int) getRequest('follow_redirects'),
+					'post_type' => (int) getRequest('post_type'),
+					'http_proxy' => getRequest('http_proxy'),
+					'headers' => getRequest('headers', []),
+					'retrieve_mode' => (int) getRequest('retrieve_mode'),
+					'request_method' => (int) getRequest('request_method'),
+					'output_format' => (int) getRequest('output_format'),
+					'allow_traps' => (int) getRequest('allow_traps', HTTPCHECK_ALLOW_TRAPS_OFF),
+					'ssl_cert_file' => getRequest('ssl_cert_file'),
+					'ssl_key_file' => getRequest('ssl_key_file'),
+					'ssl_key_password' => getRequest('ssl_key_password'),
+					'verify_peer' => (int) getRequest('verify_peer'),
+					'verify_host' => (int) getRequest('verify_host'),
+					'authtype' => getRequest('http_authtype', HTTPTEST_AUTH_NONE),
+					'username' => getRequest('http_username', ''),
+					'password' => getRequest('http_password', '')
+				];
+				$item = prepareItemHttpAgentFormData($http_item) + $item;
+			}
+
+			if ($preprocessing) {
+				$item['preprocessing'] = $preprocessing;
+			}
+
 			$result = API::ItemPrototype()->create($item);
 		}
 	}
@@ -469,33 +650,118 @@ elseif (hasRequest('action') && getRequest('action') == 'itemprototype.massdelet
 	show_messages($result, _('Item prototypes deleted'), _('Cannot delete item prototypes'));
 }
 
+if (hasRequest('action') && hasRequest('group_itemid') && !$result) {
+	$item_prototypes = API::ItemPrototype()->get([
+		'itemids' => getRequest('group_itemid'),
+		'output' => [],
+		'editable' => true
+	]);
+	uncheckTableRows(getRequest('parent_discoveryid'), zbx_objectValues($item_prototypes, 'itemid'));
+}
+
 /*
  * Display
  */
 if (isset($_REQUEST['form'])) {
 	$itemPrototype = [];
+	$has_errors = false;
+
 	if (hasRequest('itemid')) {
 		$itemPrototype = API::ItemPrototype()->get([
 			'itemids' => getRequest('itemid'),
 			'output' => [
 				'itemid', 'type', 'snmp_community', 'snmp_oid', 'hostid', 'name', 'key_', 'delay', 'history',
-				'trends', 'status', 'value_type', 'trapper_hosts', 'units', 'multiplier', 'delta',
-				'snmpv3_securityname', 'snmpv3_securitylevel', 'snmpv3_authpassphrase', 'snmpv3_privpassphrase',
-				'formula', 'logtimefmt', 'templateid', 'valuemapid', 'delay_flex', 'params', 'ipmi_sensor',
-				'data_type', 'authtype', 'username', 'password', 'publickey', 'privatekey', 'interfaceid', 'port',
-				'description', 'snmpv3_authprotocol', 'snmpv3_privprotocol', 'snmpv3_contextname'
-			]
+				'trends', 'status', 'value_type', 'trapper_hosts', 'units', 'snmpv3_securityname',
+				'snmpv3_securitylevel', 'snmpv3_authpassphrase', 'snmpv3_privpassphrase', 'logtimefmt', 'templateid',
+				'valuemapid', 'params', 'ipmi_sensor', 'authtype', 'username', 'password', 'publickey', 'privatekey',
+				'interfaceid', 'port', 'description', 'snmpv3_authprotocol', 'snmpv3_privprotocol',
+				'snmpv3_contextname', 'jmx_endpoint', 'master_itemid', 'timeout', 'url', 'query_fields', 'posts',
+				'status_codes', 'follow_redirects', 'post_type', 'http_proxy', 'headers', 'retrieve_mode',
+				'request_method', 'output_format', 'ssl_cert_file', 'ssl_key_file', 'ssl_key_password',
+				'verify_peer', 'verify_host', 'allow_traps'
+			],
+			'selectPreprocessing' => ['type', 'params']
 		]);
 		$itemPrototype = reset($itemPrototype);
+		foreach ($itemPrototype['preprocessing'] as &$step) {
+			$step['params'] = explode("\n", $step['params']);
+		}
+		unset($step);
+
+		if ($itemPrototype['type'] != ITEM_TYPE_JMX) {
+			$itemPrototype['jmx_endpoint'] = ZBX_DEFAULT_JMX_ENDPOINT;
+		}
+
+		if (getRequest('type', $itemPrototype['type']) == ITEM_TYPE_DEPENDENT) {
+			$master_prototypes = API::Item()->get([
+				'output' => ['itemid', 'hostid', 'name', 'key_'],
+				'itemids' => [getRequest('master_itemid', $itemPrototype['master_itemid'])],
+				'hostids' => [$itemPrototype['hostid']],
+				'webitems' => true
+			])
+			+ API::ItemPrototype()->get([
+				'output' => ['itemid', 'hostid', 'name', 'key_'],
+				'itemids' => getRequest('master_itemid', $itemPrototype['master_itemid'])
+			]);
+
+			if ($master_prototypes) {
+				$itemPrototype['master_item'] = reset($master_prototypes);
+			}
+		}
+	}
+	elseif (getRequest('master_itemid')) {
+		$master_prototypes = API::Item()->get([
+			'output' => ['itemid', 'hostid', 'name', 'key_'],
+			'itemids' => getRequest('master_itemid'),
+			'webitems' => true
+		])
+		+ API::ItemPrototype()->get([
+			'output' => ['itemid', 'hostid', 'name', 'key_'],
+			'itemids' => getRequest('master_itemid')
+		]);
+
+		if ($master_prototypes) {
+			$itemPrototype['master_item'] = reset($master_prototypes);
+		}
+		else {
+			show_messages(false, '', _('No permissions to referred object or it does not exist!'));
+			$has_errors = true;
+		}
 	}
 
 	$data = getItemFormData($itemPrototype);
 	$data['config'] = select_config();
+	$data['trends_default'] = DB::getDefault('items', 'trends');
+
+	$history_in_seconds = timeUnitToSeconds($data['history']);
+	if (!getRequest('form_refresh') && $history_in_seconds !== null && $history_in_seconds == ITEM_NO_STORAGE_VALUE) {
+		$data['history_mode'] = getRequest('history_mode', ITEM_STORAGE_OFF);
+		$data['history'] = DB::getDefault('items', 'history');
+	}
+	else {
+		$data['history_mode'] = getRequest('history_mode', ITEM_STORAGE_CUSTOM);
+	}
+
+	$trends_in_seconds = timeUnitToSeconds($data['trends']);
+	if (!getRequest('form_refresh') && $trends_in_seconds !== null && $trends_in_seconds == ITEM_NO_STORAGE_VALUE) {
+		$data['trends_mode'] = getRequest('trends_mode', ITEM_STORAGE_OFF);
+		$data['trends'] = $data['trends_default'];
+	}
+	else {
+		$data['trends_mode'] = getRequest('trends_mode', ITEM_STORAGE_CUSTOM);
+	}
+
+	// Sort interfaces to be listed starting with one selected as 'main'.
+	CArrayHelper::sort($data['interfaces'], [
+		['field' => 'main', 'order' => ZBX_SORT_DOWN]
+	]);
 
 	// render view
-	$itemView = new CView('configuration.item.prototype.edit', $data);
-	$itemView->render();
-	$itemView->show();
+	if (!$has_errors) {
+		$itemView = new CView('configuration.item.prototype.edit', $data);
+		$itemView->render();
+		$itemView->show();
+	}
 }
 else {
 	$sortField = getRequest('sort', CProfile::get('web.'.$page['file'].'.sort', 'name'));
@@ -523,26 +789,30 @@ else {
 		'limit' => $config['search_limit'] + 1
 	]);
 
-	foreach ($data['items'] as &$item) {
-		if ($item['value_type'] == ITEM_VALUE_TYPE_STR || $item['value_type'] == ITEM_VALUE_TYPE_LOG
-				|| $item['value_type'] == ITEM_VALUE_TYPE_TEXT) {
-			$item['trends'] = '';
-		}
+	$data['items'] = expandItemNamesWithMasterItems($data['items'], 'itemprototypes');
 
-		if ($item['type'] == ITEM_TYPE_TRAPPER || $item['type'] == ITEM_TYPE_SNMPTRAP) {
-			$item['delay'] = '';
-		}
+	switch ($sortField) {
+		case 'delay':
+			orderItemsByDelay($data['items'], $sortOrder, ['usermacros' => true, 'lldmacros' => true]);
+			break;
+
+		case 'history':
+			orderItemsByHistory($data['items'], $sortOrder);
+			break;
+
+		case 'trends':
+			orderItemsByTrends($data['items'], $sortOrder);
+			break;
+
+		default:
+			order_result($data['items'], $sortField, $sortOrder);
 	}
-	unset($item);
-
-	$data['items'] = CMacrosResolverHelper::resolveItemNames($data['items']);
-
-	order_result($data['items'], $sortField, $sortOrder);
 
 	$url = (new CUrl('disc_prototypes.php'))
 		->setArgument('parent_discoveryid', $data['parent_discoveryid']);
 
 	$data['paging'] = getPagingLine($data['items'], $sortOrder, $url);
+	$data['parent_templates'] = getItemParentTemplates($data['items'], ZBX_FLAG_DISCOVERY_PROTOTYPE);
 
 	// render view
 	$itemView = new CView('configuration.item.prototype.list', $data);
