@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2019 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -52,10 +52,18 @@ $fields = [
 	'pairs'           => [T_ZBX_STR, O_OPT, P_NO_TRIM,  null,                    null],
 	'steps'           => [T_ZBX_STR, O_OPT, P_NO_TRIM,  null,                    'isset({add}) || isset({update})', _('Steps')],
 	'authentication'  => [T_ZBX_INT, O_OPT, null,  IN('0,1,2'),             'isset({add}) || isset({update})'],
-	'http_user'       => [T_ZBX_STR, O_OPT, null,  NOT_EMPTY,               '(isset({add}) || isset({update})) && isset({authentication}) && ({authentication} == '.HTTPTEST_AUTH_BASIC.
-		' || {authentication} == '.HTTPTEST_AUTH_NTLM.')', _('User')],
-	'http_password'		=> [T_ZBX_STR, O_OPT, P_NO_TRIM,	NOT_EMPTY,		'(isset({add}) || isset({update})) && isset({authentication}) && ({authentication} == '.HTTPTEST_AUTH_BASIC.
-		' || {authentication} == '.HTTPTEST_AUTH_NTLM.')', _('Password')],
+	'http_user'       => [T_ZBX_STR, O_OPT, null,  null,
+		'(isset({add}) || isset({update})) && isset({authentication}) && ({authentication}=='.HTTPTEST_AUTH_BASIC.
+			' || {authentication}=='.HTTPTEST_AUTH_NTLM.
+		')',
+		_('User')
+	],
+	'http_password'		=> [T_ZBX_STR, O_OPT, P_NO_TRIM, null,
+		'(isset({add}) || isset({update})) && isset({authentication}) && ({authentication}=='.HTTPTEST_AUTH_BASIC.
+			' || {authentication}=='.HTTPTEST_AUTH_NTLM.
+		')',
+		_('Password')
+	],
 	'http_proxy'		=> [T_ZBX_STR, O_OPT, null,	null,				'isset({add}) || isset({update})'],
 	'new_application'	=> [T_ZBX_STR, O_OPT, null,	null,				null],
 	'hostname'			=> [T_ZBX_STR, O_OPT, null,	null,				null],
@@ -101,28 +109,18 @@ if (!empty($_REQUEST['steps'])) {
  * Permissions
  */
 //*
-if (isset($_REQUEST['httptestid']) || !empty($_REQUEST['group_httptestid'])) {
-	$testIds = [];
-	if (isset($_REQUEST['httptestid'])) {
-		$testIds[] = $_REQUEST['httptestid'];
-	}
-	if (!empty($_REQUEST['group_httptestid'])) {
-		$testIds = array_merge($testIds, $_REQUEST['group_httptestid']);
-	}
-	if ($testIds) {
-		$testIds = array_unique($testIds);
+if (hasRequest('httptestid')) {
+	$httptests = API::HttpTest()->get([
+		'output' => [],
+		'httptestids' => getRequest('httptestid'),
+		'editable' => true
+	]);
 
-		$count = API::HttpTest()->get([
-			'countOutput' => true,
-			'httptestids' => $testIds,
-			'editable' => true
-		]);
-
-		if ($count != count($testIds)) {
-			access_deny();
-		}
+	if (!$httptests) {
+		access_deny();
 	}
 }
+
 if (getRequest('hostid') && !isWritableHostTemplates([getRequest('hostid')])) {
 	access_deny();
 }
@@ -364,6 +362,10 @@ elseif (hasRequest('add') || hasRequest('update')) {
 			}
 
 			foreach ($httpTest['steps'] as $snum => $step) {
+				if ($step['httpstepid'] < 1) {
+					unset($step['httpstepid']);
+					unset($httpTest['steps'][$snum]['httpstepid']);
+				}
 				if (array_key_exists('httpstepid', $step) && array_key_exists($step['httpstepid'], $dbHttpSteps)) {
 					$db_step = $dbHttpSteps[$step['httpstepid']];
 					$new_step = CArrayHelper::unsetEqualValues($step, $db_step, ['httpstepid']);
@@ -392,8 +394,10 @@ elseif (hasRequest('add') || hasRequest('update')) {
 			}
 
 			$httpTest['httptestid'] = $httpTestId = $_REQUEST['httptestid'];
+
 			$result = API::HttpTest()->update($httpTest);
 			if (!$result) {
+
 				throw new Exception();
 			}
 			else {
@@ -423,6 +427,7 @@ elseif (hasRequest('add') || hasRequest('update')) {
 	catch (Exception $e) {
 		DBend(false);
 
+
 		$msg = $e->getMessage();
 		if (!empty($msg)) {
 			error($msg);
@@ -516,6 +521,16 @@ elseif (hasRequest('action') && getRequest('action') === 'httptest.massdelete'
 	show_messages($result, _('Web scenario deleted'), _('Cannot delete web scenario'));
 }
 
+if (hasRequest('action') && hasRequest('group_httptestid') && !$result) {
+	$httptests = API::HttpTest()->get([
+		'output' => [],
+		'httptestids' => getRequest('group_httptestid'),
+		'editable' => true
+	]);
+
+	uncheckTableRows(getRequest('hostid'), zbx_objectValues($httptests, 'httptestid'));
+}
+
 show_messages();
 
 /*
@@ -537,32 +552,6 @@ if (isset($_REQUEST['form'])) {
 	]);
 	$data['host'] = reset($host);
 
-	if (isset($data['httptestid'])) {
-		// get templates
-		$httpTestId = $data['httptestid'];
-		while ($httpTestId) {
-			$dbTest = DBfetch(DBselect(
-				'SELECT h.hostid,h.name,ht.httptestid,ht.templateid'.
-					' FROM hosts h,httptest ht'.
-					' WHERE ht.hostid=h.hostid'.
-					' AND ht.httptestid='.zbx_dbstr($httpTestId)
-			));
-			$httpTestId = null;
-
-			if (!empty($dbTest)) {
-				if (!idcmp($data['httptestid'], $dbTest['httptestid'])) {
-					$data['templates'][] = new CLink($dbTest['name'],
-						'httpconf.php?form=update&httptestid='.$dbTest['httptestid'].'&hostid='.$dbTest['hostid']
-					);
-					$data['templates'][] = SPACE.'&rArr;'.SPACE;
-				}
-				$httpTestId = $dbTest['templateid'];
-			}
-		}
-		$data['templates'] = array_reverse($data['templates']);
-		array_shift($data['templates']);
-	}
-
 	if (hasRequest('httptestid') && !hasRequest('form_refresh')) {
 		$db_httptests = API::HttpTest()->get([
 			'output' => ['name', 'applicationid', 'delay', 'retries', 'status', 'agent', 'authentication',
@@ -583,6 +572,9 @@ if (isset($_REQUEST['form'])) {
 		$data['delay'] = $db_httptest['delay'];
 		$data['retries'] = $db_httptest['retries'];
 		$data['status'] = $db_httptest['status'];
+		$data['templates'] = makeHttpTestTemplatesHtml($db_httptest['httptestid'],
+			getHttpTestParentTemplates($db_httptests)
+		);
 
 		$data['agent'] = ZBX_AGENT_OTHER;
 		$data['agent_other'] = $db_httptest['agent'];
@@ -720,6 +712,22 @@ if (isset($_REQUEST['form'])) {
 		}
 	}
 
+	foreach($data['steps'] as $stepid => $step) {
+		$pairs_grouped = [
+			'query_fields' => [],
+			'post_fields' => [],
+			'variables' => [],
+			'headers' => [],
+		];
+
+		if (array_key_exists('pairs', $step)) {
+			foreach ($step['pairs'] as $field) {
+				$pairs_grouped[$field['type']][] = $field;
+			}
+			$data['steps'][$stepid]['pairs'] = $pairs_grouped;
+		}
+	}
+
 	// render view
 	$httpView = new CView('configuration.httpconf.edit', $data);
 	$httpView->render();
@@ -841,8 +849,7 @@ else {
 
 		order_result($httpTests, $sortField, $sortOrder);
 
-		$data['parentTemplates'] = getHttpTestsParentTemplates($httpTests);
-
+		$data['parent_templates'] = getHttpTestParentTemplates($httpTests);
 		$data['httpTests'] = $httpTests;
 		$data['httpTestsLastData'] = $httpTestsLastData;
 	}
