@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2019 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -455,12 +455,16 @@ class CApiService {
 	 * @return array		The resulting SQL parts array
 	 */
 	protected function applyQueryOutputOptions($tableName, $tableAlias, array $options, array $sqlParts) {
-		$pkFieldId = $this->fieldId($this->pk($tableName), $tableAlias);
+		// If table do not have a primary key, use COUNT(*) to select number of rows.
+		$pk = $this->pk($tableName);
+		$pkFieldId = $this->fieldId($pk, $tableAlias);
 
 		// count
 		if (array_key_exists('countOutput', $options) && $options['countOutput']
 				&& !$this->requiresPostSqlFiltering($options)) {
-			$sqlParts['select'] = ['COUNT(DISTINCT '.$pkFieldId.') AS rowscount'];
+			$sqlParts['select'] = ($pk !== '')
+				? ['COUNT(DISTINCT '.$pkFieldId.') AS rowscount']
+				: ['COUNT(*) AS rowscount'];
 
 			// select columns used by group count
 			if (array_key_exists('groupCount', $options) && $options['groupCount']) {
@@ -472,7 +476,7 @@ class CApiService {
 		// custom output
 		elseif (is_array($options['output'])) {
 			// the pk field must always be included for the API to work properly
-			$sqlParts['select'] = [$pkFieldId];
+			$sqlParts['select'] = ($pk !== '') ? [$pkFieldId] : [];
 			foreach ($options['output'] as $field) {
 				if ($this->hasField($field, $tableName)) {
 					$sqlParts['select'][] = $this->fieldId($field, $tableAlias);
@@ -644,8 +648,8 @@ class CApiService {
 	 * Adds the related objects requested by "select*" options to the resulting object set.
 	 *
 	 * @param array $options
-	 * @param array $result		an object hash with PKs as keys
-
+	 * @param array $result   An object hash with PKs as keys.
+	 *
 	 * @return array mixed
 	 */
 	protected function addRelatedObjects(array $options, array $result) {
@@ -938,7 +942,7 @@ class CApiService {
 	}
 
 	/**
-	 * Check if a set of parameters contains a deprecated parameter or a a parameter with a deprecated value.
+	 * Check if a set of parameters contains a deprecated parameter or a parameter with a deprecated value.
 	 * If $value is not set, the method will trigger a deprecated notice if $params contains the $paramName key.
 	 * If $value is set, the method will trigger a notice if the value of the parameter is equal to the deprecated value
 	 * or the parameter is an array and contains a deprecated value.
