@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2019 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -517,10 +517,8 @@ class CDiscoveryRule extends CItemGeneral {
 		while ($item = DBfetch($dbItems)) {
 			$iprototypeids[$item['itemid']] = $item['itemid'];
 		}
-		if (!empty($iprototypeids)) {
-			if (!API::ItemPrototype()->delete($iprototypeids, true)) {
-				self::exception(ZBX_API_ERROR_PARAMETERS, _('Cannot delete discovery rule'));
-			}
+		if ($iprototypeids) {
+			CItemPrototypeManager::delete($iprototypeids);
 		}
 
 		// delete host prototypes
@@ -679,13 +677,12 @@ class CDiscoveryRule extends CItemGeneral {
 	 * @throws APIException if trigger saving fails
 	 *
 	 * @param array $srcDiscovery    The source discovery rule to copy from
-	 * @param array $dstDiscovery    The target discovery rule to copy to
 	 * @param array $srcHost         The host the source discovery belongs to
 	 * @param array $dstHost         The host the target discovery belongs to
 	 *
 	 * @return array
 	 */
-	protected function copyTriggerPrototypes(array $srcDiscovery, array $dstDiscovery, array $srcHost, array $dstHost) {
+	protected function copyTriggerPrototypes(array $srcDiscovery, array $srcHost, array $dstHost) {
 		$srcTriggers = API::TriggerPrototype()->get([
 			'discoveryids' => $srcDiscovery['itemid'],
 			'output' => ['triggerid', 'expression', 'description', 'url', 'status', 'priority', 'comments',
@@ -700,10 +697,6 @@ class CDiscoveryRule extends CItemGeneral {
 			'preservekeys' => true
 		]);
 
-		if (!$srcTriggers) {
-			return [];
-		}
-
 		foreach ($srcTriggers as $id => $trigger) {
 			// Skip trigger prototypes with web items and remove them from source.
 			if (httpItemExists($trigger['items'])) {
@@ -711,6 +704,10 @@ class CDiscoveryRule extends CItemGeneral {
 			}
 		}
 
+		if (!$srcTriggers) {
+			return [];
+		}
+
 		/*
 		 * Copy the remaining trigger prototypes to a new source. These will contain IDs and original dependencies.
 		 * The dependencies from $srcTriggers will be removed.
@@ -743,7 +740,7 @@ class CDiscoveryRule extends CItemGeneral {
 			['sources' => ['expression', 'recovery_expression']]
 		);
 		foreach ($dstTriggers as $id => &$trigger) {
-			unset($dstTriggers[$id]['triggerid'], $dstTriggers[$id]['templateid']);
+			unset($trigger['triggerid'], $trigger['templateid']);
 
 			// Update the destination expressions.
 			$trigger['expression'] = triggerExpressionReplaceHost($trigger['expression'], $srcHost['host'],
@@ -845,15 +842,13 @@ class CDiscoveryRule extends CItemGeneral {
 									$new_trigger_prototype['src_host'],
 									$new_trigger_prototype['new_host']
 								);
-								error(_s(
+								self::exception(ZBX_API_ERROR_PARAMETERS, _s(
 									'Cannot add dependency from trigger "%1$s:%2$s" to non existing trigger "%3$s:%4$s".',
 									$trigger_prototype['description'],
 									$trigger_prototype['expression'],
 									$dep_triggers[$dep_triggerid]['description'],
 									$expr2
 								));
-
-								return false;
 							}
 						}
 					}
@@ -913,8 +908,8 @@ class CDiscoveryRule extends CItemGeneral {
 		}
 	}
 
-	protected function updateReal($items) {
-		$items = zbx_toArray($items);
+	protected function updateReal(array $items) {
+		CArrayHelper::sort($items, ['itemid']);
 
 		$ruleIds = zbx_objectValues($items, 'itemid');
 
@@ -1145,57 +1140,6 @@ class CDiscoveryRule extends CItemGeneral {
 		}
 	}
 
-	protected function inherit(array $items, array $hostids = null) {
-		if (!$items) {
-			return;
-		}
-
-		// Prepare the child discovery rules.
-		$new_items = $this->prepareInheritedItems($items, $hostids);
-		if (!$new_items) {
-			return;
-		}
-
-		$ins_items = [];
-		$upd_items = [];
-		foreach ($new_items as $new_item) {
-			if (array_key_exists('itemid', $new_item)) {
-				$upd_items[] = $new_item;
-			}
-			else {
-				$ins_items[] = $new_item;
-			}
-		}
-
-		// Save the new items.
-		$this->createReal($ins_items);
-		$this->updateReal($upd_items);
-
-		$new_items = array_merge($upd_items, $ins_items);
-
-		// Inheriting items from the templates.
-		$tpl_items = DBselect(
-			'SELECT i.itemid'.
-			' FROM items i,hosts h'.
-			' WHERE i.hostid=h.hostid'.
-				' AND '.dbConditionInt('i.itemid', zbx_objectValues($new_items, 'itemid')).
-				' AND '.dbConditionInt('h.status', [HOST_STATUS_TEMPLATE])
-		);
-
-		$tpl_itemids = [];
-		while ($tpl_item = DBfetch($tpl_items)) {
-			$tpl_itemids[$tpl_item['itemid']] = true;
-		}
-
-		foreach ($new_items as $index => $new_item) {
-			if (!array_key_exists($new_item['itemid'], $tpl_itemids)) {
-				unset($new_items[$index]);
-			}
-		}
-
-		$this->inherit($new_items);
-	}
-
 	/**
 	 * Copies the given discovery rule to the specified host.
 	 *
@@ -1213,7 +1157,7 @@ class CDiscoveryRule extends CItemGeneral {
 			'itemids' => $discoveryid,
 			'output' => ['itemid', 'type', 'snmp_community', 'snmp_oid', 'hostid', 'name', 'key_', 'delay', 'history',
 				'trends', 'status', 'value_type', 'trapper_hosts', 'units', 'snmpv3_securityname',
-				'snmpv3_securitylevel',	'snmpv3_authpassphrase', 'snmpv3_privpassphrase', 'lastlogsize', 'logtimefmt',
+				'snmpv3_securitylevel', 'snmpv3_authpassphrase', 'snmpv3_privpassphrase', 'lastlogsize', 'logtimefmt',
 				'valuemapid', 'params', 'ipmi_sensor', 'authtype', 'username', 'password', 'publickey', 'privatekey',
 				'mtime', 'flags', 'interfaceid', 'port', 'description', 'inventory_link', 'lifetime',
 				'snmpv3_authprotocol', 'snmpv3_privprotocol', 'snmpv3_contextname', 'jmx_endpoint', 'url',
@@ -1269,39 +1213,33 @@ class CDiscoveryRule extends CItemGeneral {
 		$dstDiscovery['itemid'] = $newDiscovery['itemids'][0];
 
 		// copy prototypes
-		$newPrototypes = $this->copyItemPrototypes($srcDiscovery, $dstDiscovery, $dstHost);
+		$new_prototypeids = $this->copyItemPrototypes($srcDiscovery, $dstDiscovery, $dstHost);
 
 		// if there were prototypes defined, clone everything else
-		if ($newPrototypes) {
+		if ($new_prototypeids) {
 			// fetch new prototypes
-			$newPrototypes = API::ItemPrototype()->get([
-				'itemids' => $newPrototypes['itemids'],
-				'output' => API_OUTPUT_EXTEND,
+			$dstDiscovery['items'] = API::ItemPrototype()->get([
+				'output' => ['itemid', 'key_'],
+				'itemids' => $new_prototypeids,
 				'preservekeys' => true
 			]);
 
-			foreach ($newPrototypes as $i => $newPrototype) {
-				unset($newPrototypes[$i]['templateid']);
-			}
-
-			$dstDiscovery['items'] = $newPrototypes;
-
 			// copy graphs
 			$this->copyGraphPrototypes($srcDiscovery, $dstDiscovery);
 
 			// copy triggers
-			$this->copyTriggerPrototypes($srcDiscovery, $dstDiscovery, $srcHost, $dstHost);
+			$this->copyTriggerPrototypes($srcDiscovery, $srcHost, $dstHost);
 		}
 
 		// copy host prototypes
-		$this->copyHostPrototypes($srcDiscovery, $dstDiscovery);
+		$this->copyHostPrototypes($discoveryid, $dstDiscovery);
 
 		return true;
 	}
 
 	/**
 	 * Copies all of the item prototypes from the source discovery to the target
-	 * discovery rule.
+	 * discovery rule. Return array of created item prototype ids.
 	 *
 	 * @throws APIException if prototype saving fails
 	 *
@@ -1328,6 +1266,9 @@ class CDiscoveryRule extends CItemGeneral {
 			'discoveryids' => $srcDiscovery['itemid'],
 			'preservekeys' => true
 		]);
+		$new_itemids = [];
+		$itemkey_to_id = [];
+		$create_items = [];
 
 		if ($item_prototypes) {
 			$create_order = [];
@@ -1389,8 +1330,6 @@ class CDiscoveryRule extends CItemGeneral {
 			}
 			asort($create_order);
 
-			$itemkey_to_id = [];
-			$create_items = [];
 			$current_dependency = reset($create_order);
 
 			foreach ($create_order as $key => $dependency_level) {
@@ -1401,7 +1340,9 @@ class CDiscoveryRule extends CItemGeneral {
 					if (!$created_itemids) {
 						self::exception(ZBX_API_ERROR_PARAMETERS, _('Cannot clone item prototypes.'));
 					}
+
 					$created_itemids = $created_itemids['itemids'];
+					$new_itemids = array_merge($new_itemids, $created_itemids);
 
 					foreach ($create_items as $index => $created_item) {
 						$itemkey_to_id[$created_item['key_']] = $created_itemids[$index];
@@ -1467,12 +1408,18 @@ class CDiscoveryRule extends CItemGeneral {
 				$create_items[] = $item_prototype;
 			}
 
-			if ($create_items && !API::ItemPrototype()->create($create_items)) {
-				self::exception(ZBX_API_ERROR_PARAMETERS, _('Cannot clone item prototypes.'));
+			if ($create_items) {
+				$created_itemids = API::ItemPrototype()->create($create_items);
+
+				if (!$created_itemids) {
+					self::exception(ZBX_API_ERROR_PARAMETERS, _('Cannot clone item prototypes.'));
+				}
+
+				$new_itemids = array_merge($new_itemids, $created_itemids['itemids']);
 			}
 		}
 
-		return true;
+		return $new_itemids;
 	}
 
 	/**
@@ -1596,16 +1543,16 @@ class CDiscoveryRule extends CItemGeneral {
 	 * Copies all of the host prototypes from the source discovery to the target
 	 * discovery rule.
 	 *
-	 * @throws APIException if prototype saving fails
+	 * @throws APIException if prototype saving fails.
 	 *
-	 * @param array $srcDiscovery   The source discovery rule to copy from
-	 * @param array $dstDiscovery   The target discovery rule to copy to
+	 * @param int   $srcid          The source discovery rule id to copy from.
+	 * @param array $dstDiscovery   The target discovery rule to copy to.
 	 *
 	 * @return array
 	 */
-	protected function copyHostPrototypes(array $srcDiscovery, array $dstDiscovery) {
+	protected function copyHostPrototypes($srcid, array $dstDiscovery) {
 		$prototypes = API::HostPrototype()->get([
-			'discoveryids' => $srcDiscovery['itemid'],
+			'discoveryids' => $srcid,
 			'output' => ['host', 'name', 'status'],
 			'selectGroupLinks' => ['groupid'],
 			'selectGroupPrototypes' => ['name'],
@@ -1691,7 +1638,7 @@ class CDiscoveryRule extends CItemGeneral {
 
 				$items = zbx_toHash($items, 'parent_itemid');
 				foreach ($result as $itemid => $item) {
-					$result[$itemid]['items'] = isset($items[$itemid]) ? $items[$itemid]['rowscount'] : 0;
+					$result[$itemid]['items'] = array_key_exists($itemid, $items) ? $items[$itemid]['rowscount'] : '0';
 				}
 			}
 		}
@@ -1727,7 +1674,9 @@ class CDiscoveryRule extends CItemGeneral {
 
 				$triggers = zbx_toHash($triggers, 'parent_itemid');
 				foreach ($result as $itemid => $item) {
-					$result[$itemid]['triggers'] = isset($triggers[$itemid]) ? $triggers[$itemid]['rowscount'] : 0;
+					$result[$itemid]['triggers'] = array_key_exists($itemid, $triggers)
+						? $triggers[$itemid]['rowscount']
+						: '0';
 				}
 			}
 		}
@@ -1763,7 +1712,9 @@ class CDiscoveryRule extends CItemGeneral {
 
 				$graphs = zbx_toHash($graphs, 'parent_itemid');
 				foreach ($result as $itemid => $item) {
-					$result[$itemid]['graphs'] = isset($graphs[$itemid]) ? $graphs[$itemid]['rowscount'] : 0;
+					$result[$itemid]['graphs'] = array_key_exists($itemid, $graphs)
+						? $graphs[$itemid]['rowscount']
+						: '0';
 				}
 			}
 		}
@@ -1790,7 +1741,9 @@ class CDiscoveryRule extends CItemGeneral {
 				$hostPrototypes = zbx_toHash($hostPrototypes, 'parent_itemid');
 
 				foreach ($result as $itemid => $item) {
-					$result[$itemid]['hostPrototypes'] = isset($hostPrototypes[$itemid]) ? $hostPrototypes[$itemid]['rowscount'] : 0;
+					$result[$itemid]['hostPrototypes'] = array_key_exists($itemid, $hostPrototypes)
+						? $hostPrototypes[$itemid]['rowscount']
+						: '0';
 				}
 			}
 		}
