@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2019 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -58,7 +58,6 @@ class CMap extends CMapElement {
 		'limit'						=> null
 	];
 
-
 	/**
 	 * Get map data.
 	 *
@@ -97,8 +96,6 @@ class CMap extends CMapElement {
 	 *						 parameter has been used.
 	 */
 	public function get(array $options = []) {
-		$user_data = self::$userData;
-
 		$options = zbx_array_merge($this->defOptions, $options);
 
 		$limit = $options['limit'];
@@ -126,7 +123,7 @@ class CMap extends CMapElement {
 		}
 
 		if ($count_output) {
-			return count($result);
+			return (string) count($result);
 		}
 
 		if ($limit !== null) {
@@ -630,6 +627,9 @@ class CMap extends CMapElement {
 			'values' => [PERM_READ, PERM_READ_WRITE]
 		]);
 
+		$show_suppressed_types = [ZBX_PROBLEM_SUPPRESSED_FALSE, ZBX_PROBLEM_SUPPRESSED_TRUE];
+		$show_suppressed_validator = new CLimitedSetValidator(['values' => $show_suppressed_types]);
+
 		$expandproblem_types = [SYSMAP_PROBLEMS_NUMBER, SYSMAP_SINGLE_PROBLEM, SYSMAP_PROBLEMS_NUMBER_CRITICAL];
 		$expandproblem_validator = new CLimitedSetValidator(['values' => $expandproblem_types]);
 
@@ -668,6 +668,14 @@ class CMap extends CMapElement {
 				}
 			}
 
+			// Check for invalid "show_suppressed" values.
+			if (array_key_exists('show_suppressed', $map)
+					&& !$show_suppressed_validator->validate($map['show_suppressed'])) {
+				self::exception(ZBX_API_ERROR_PARAMETERS, _s('Incorrect value for field "%1$s": %2$s.',
+					'show_suppressed', _s('value must be one of %1$s', implode(', ', $show_suppressed_types))
+				));
+			}
+
 			if (array_key_exists('expandproblem', $map) && !$expandproblem_validator->validate($map['expandproblem'])) {
 				self::exception(ZBX_API_ERROR_PARAMETERS, _s('Incorrect value for field "%1$s": %2$s.', 'expandproblem',
 					_s('value must be one of %1$s', implode(', ', $expandproblem_types))
@@ -954,8 +962,15 @@ class CMap extends CMapElement {
 				}
 			}
 
-			if (array_key_exists('selements', $map) && !is_array($map['selements'])) {
-				self::exception(ZBX_API_ERROR_PARAMETERS, _('Incorrect arguments passed to function.'));
+			if (array_key_exists('selements', $map)) {
+				if (!is_array($map['selements'])) {
+					self::exception(ZBX_API_ERROR_PARAMETERS, _('Incorrect arguments passed to function.'));
+				}
+				elseif (!CMapHelper::checkSelementPermissions($map['selements'])) {
+					self::exception(ZBX_API_ERROR_PERMISSIONS,
+						_('No permissions to referred object or it does not exist!')
+					);
+				}
 			}
 
 			// Map selement links.
@@ -1065,6 +1080,9 @@ class CMap extends CMapElement {
 			'values' => [PERM_READ, PERM_READ_WRITE]
 		]);
 
+		$show_suppressed_types = [ZBX_PROBLEM_SUPPRESSED_FALSE, ZBX_PROBLEM_SUPPRESSED_TRUE];
+		$show_suppressed_validator = new CLimitedSetValidator(['values' => $show_suppressed_types]);
+
 		$expandproblem_types = [SYSMAP_PROBLEMS_NUMBER, SYSMAP_SINGLE_PROBLEM, SYSMAP_PROBLEMS_NUMBER_CRITICAL];
 		$expandproblem_validator = new CLimitedSetValidator(['values' => $expandproblem_types]);
 
@@ -1104,6 +1122,14 @@ class CMap extends CMapElement {
 				);
 			}
 
+			// Check for invalid "show_suppressed" values.
+			if (array_key_exists('show_suppressed', $map)
+					&& !$show_suppressed_validator->validate($map['show_suppressed'])) {
+				self::exception(ZBX_API_ERROR_PARAMETERS, _s('Incorrect value for field "%1$s": %2$s.',
+					'show_suppressed', _s('value must be one of %1$s', implode(', ', $show_suppressed_types))
+				));
+			}
+
 			if (array_key_exists('expandproblem', $map) && !$expandproblem_validator->validate($map['expandproblem'])) {
 				self::exception(ZBX_API_ERROR_PARAMETERS, _s('Incorrect value for field "%1$s": %2$s.', 'expandproblem',
 					_s('value must be one of %1$s', implode(', ', $expandproblem_types))
@@ -1423,6 +1449,117 @@ class CMap extends CMapElement {
 				}
 			}
 		}
+
+		// Validate circular reference.
+		foreach ($maps as &$map) {
+			$map = array_merge($db_maps[$map['sysmapid']], $map);
+			$this->cref_maps[$map['sysmapid']] = $map;
+		}
+		unset($map);
+
+		$this->validateCircularReference($maps);
+	}
+
+	/**
+	 * Hash of maps data for circular reference validation. Map id is used as key.
+	 *
+	 * @var array
+	 */
+	protected $cref_maps;
+
+	/**
+	 * Validate maps for circular reference.
+	 *
+	 * @param array $maps   Array of maps to be validated for circular reference.
+	 *
+	 * @throws APIException if input is invalid.
+	 */
+	protected function validateCircularReference(array $maps) {
+		foreach ($maps as $map) {
+			if (!array_key_exists('selements', $map) || !$map['selements']) {
+				continue;
+			}
+			$cref_mapids = array_key_exists('sysmapid', $map) ? [$map['sysmapid']] : [];
+
+			foreach ($map['selements'] as $selement) {
+				if (!$this->validateCircularReferenceRecursive($selement, $cref_mapids)) {
+					self::exception(ZBX_API_ERROR_PARAMETERS,
+						_s('Cannot add map element of the map "%1$s" due to circular reference.', $map['name'])
+					);
+				}
+			}
+		}
+	}
+
+	/**
+	 * Recursive map element circular reference validation.
+	 *
+	 * @param array $selement       Map selement data array.
+	 * @param array $cref_mapids    Array of map ids for current recursion step.
+	 *
+	 * @return bool
+	 */
+	protected function validateCircularReferenceRecursive(array $selement, &$cref_mapids) {
+		if ($selement['elementtype'] != SYSMAP_ELEMENT_TYPE_MAP) {
+			return true;
+		}
+
+		$sysmapid = $selement['elements'][0]['sysmapid'];
+
+		if ($sysmapid !== null && !array_key_exists($sysmapid, $this->cref_maps)) {
+			$db_maps = DB::select($this->tableName, [
+				'output' => ['name'],
+				'filter' => ['sysmapid' => $sysmapid]
+			]);
+
+			if ($db_maps) {
+				$db_map = $db_maps[0];
+				$db_map['selements'] = [];
+				$selements = DB::select('sysmaps_elements', [
+					'output' => ['elementid'],
+					'filter' => [
+						'sysmapid' => $sysmapid,
+						'elementtype' => SYSMAP_ELEMENT_TYPE_MAP
+					]
+				]);
+
+				foreach ($selements as $selement) {
+					$db_map['selements'][] = [
+						'elementtype' => SYSMAP_ELEMENT_TYPE_MAP,
+						'elements' => [
+							['sysmapid' => $selement['elementid']]
+						]
+					];
+				}
+
+				$this->cref_maps[$sysmapid] = $db_map;
+				unset($selements);
+			}
+			else {
+				self::exception(ZBX_API_ERROR_PARAMETERS, _('No permissions to referred object or it does not exist!'));
+			}
+		}
+
+		if (in_array($sysmapid, $cref_mapids)) {
+			$cref_mapids[] = $sysmapid;
+			return false;
+		}
+
+		// Find maps that reference the current element, and if one has selements, check all of them recursively.
+		if ($sysmapid !== null && array_key_exists('selements', $this->cref_maps[$sysmapid])
+				&& is_array($this->cref_maps[$sysmapid]['selements'])) {
+			$cref_mapids[] = $sysmapid;
+
+			foreach ($this->cref_maps[$sysmapid]['selements'] as $selement) {
+				if (!$this->validateCircularReferenceRecursive($selement, $cref_mapids)) {
+					return false;
+				}
+			}
+
+			array_pop($cref_mapids);
+		}
+
+		return true;
 	}
 
 	/**
@@ -1526,7 +1663,7 @@ class CMap extends CMapElement {
 					$maps[$key]['selements'][$snum]['sysmapid'] = $sysmapid;
 				}
 
-				$selements += $maps[$key]['selements'];
+				$selements = array_merge($selements, $maps[$key]['selements']);
 			}
 
 			if (array_key_exists('shapes', $maps[$key])) {
@@ -1550,7 +1687,7 @@ class CMap extends CMapElement {
 					$maps[$key]['shapes'][$snum]['sysmapid'] = $sysmapid;
 				}
 
-				$shapes += $maps[$key]['shapes'];
+				$shapes = array_merge($shapes, $maps[$key]['shapes']);
 			}
 
 			if (array_key_exists('lines', $maps[$key])) {
@@ -1582,7 +1719,7 @@ class CMap extends CMapElement {
 					$maps[$key]['links'][$lnum]['sysmapid'] = $sysmapid;
 				}
 
-				$links += $maps[$key]['links'];
+				$links = array_merge($links, $maps[$key]['links']);
 			}
 		}
 
@@ -1809,6 +1946,11 @@ class CMap extends CMapElement {
 					$selements_to_add[] = $new_selement;
 				}
 
+				foreach ($selement_diff['both'] as &$selement) {
+					$selement['sysmapid'] = $map['sysmapid'];
+				}
+				unset($selement);
+
 				$selements_to_update = array_merge($selements_to_update, $selement_diff['both']);
 				$selements_to_delete = array_merge($selements_to_delete, $selement_diff['second']);
 			}
@@ -2160,6 +2302,53 @@ class CMap extends CMapElement {
 			]);
 			$relation_map = $this->createRelationMap($selements, 'sysmapid', 'selementid');
 
+			if ($this->outputIsRequested('elements', $options['selectSelements']) && $selements) {
+				foreach ($selements as &$selement) {
+					$selement['elements'] = [];
+				}
+				unset($selement);
+
+				$selement_triggers = DBselect(
+					'SELECT st.selementid,st.triggerid,st.selement_triggerid'.
+					' FROM sysmap_element_trigger st,triggers tr'.
+					' WHERE '.dbConditionInt('st.selementid', array_keys($selements)).
+						' AND st.triggerid=tr.triggerid'.
+					' ORDER BY tr.priority DESC,st.selement_triggerid'
+				);
+				while ($selement_trigger = DBfetch($selement_triggers)) {
+					$selements[$selement_trigger['selementid']]['elements'][] = [
+						'triggerid' => $selement_trigger['triggerid']
+					];
+					if ($selements[$selement_trigger['selementid']]['elementid'] == 0) {
+						$selements[$selement_trigger['selementid']]['elementid'] = $selement_trigger['triggerid'];
+					}
+				}
+
+				$single_element_types = [SYSMAP_ELEMENT_TYPE_HOST, SYSMAP_ELEMENT_TYPE_MAP,
+					SYSMAP_ELEMENT_TYPE_HOST_GROUP
+				];
+
+				foreach ($selements as &$selement) {
+					if (in_array($selement['elementtype'], $single_element_types)) {
+						switch ($selement['elementtype']) {
+							case SYSMAP_ELEMENT_TYPE_HOST_GROUP:
+								$field = 'groupid';
+								break;
+
+							case SYSMAP_ELEMENT_TYPE_HOST:
+								$field = 'hostid';
+								break;
+
+							case SYSMAP_ELEMENT_TYPE_MAP:
+								$field = 'sysmapid';
+								break;
+						}
+						$selement['elements'][] = [$field => $selement['elementid']];
+					}
+				}
+				unset($selement);
+			}
+
 			// add selement URLs
 			if ($this->outputIsRequested('urls', $options['selectSelements'])) {
 				foreach ($selements as &$selement) {
@@ -2200,47 +2389,6 @@ class CMap extends CMapElement {
 				}
 			}
 
-			if ($this->outputIsRequested('elements', $options['selectSelements']) && $selements) {
-				foreach ($selements as &$selement) {
-					$selement['elements'] = [];
-				}
-				unset($selement);
-
-				$selement_triggers = API::getApiService()->select('sysmap_element_trigger', [
-					'output' => ['selementid', 'triggerid'],
-					'filter' => ['selementid' => array_keys($selements)]
-				]);
-				foreach ($selement_triggers as $selement_trigger) {
-					$selements[$selement_trigger['selementid']]['elements'][] = [
-						'triggerid' => $selement_trigger['triggerid']
-					];
-				}
-
-				$single_element_types = [SYSMAP_ELEMENT_TYPE_HOST, SYSMAP_ELEMENT_TYPE_MAP,
-					SYSMAP_ELEMENT_TYPE_HOST_GROUP
-				];
-
-				foreach ($selements as &$selement) {
-					if (in_array($selement['elementtype'], $single_element_types)) {
-						switch ($selement['elementtype']) {
-							case SYSMAP_ELEMENT_TYPE_HOST_GROUP:
-								$field = 'groupid';
-								break;
-
-							case SYSMAP_ELEMENT_TYPE_HOST:
-								$field = 'hostid';
-								break;
-
-							case SYSMAP_ELEMENT_TYPE_MAP:
-								$field = 'sysmapid';
-								break;
-						}
-						$selement['elements'][] = [$field => $selement['elementid']];
-					}
-				}
-				unset($selement);
-			}
-
 			if ($this->outputIsRequested('permission', $options['selectSelements']) && $selements) {
 				if ($options['editable']) {
 					foreach ($selements as &$selement) {
@@ -2483,11 +2631,6 @@ class CMap extends CMapElement {
 					unset($link);
 				}
 				else {
-					foreach ($links as &$link) {
-						$link['permission'] = PERM_NONE;
-					}
-					unset($link);
-
 					$db_link_triggers = DBselect(
 						'SELECT slt.linkid,slt.triggerid'.
 						' FROM sysmaps_link_triggers slt'.
@@ -2495,11 +2638,18 @@ class CMap extends CMapElement {
 					);
 
 					$triggerids = [];
+					$has_triggers = [];
 
 					while ($db_link_trigger = DBfetch($db_link_triggers)) {
 						$triggerids[$db_link_trigger['triggerid']][] = $db_link_trigger['linkid'];
+						$has_triggers[$db_link_trigger['linkid']] = true;
 					}
 
+					foreach ($links as &$link) {
+						$link['permission'] = array_key_exists($link['linkid'], $has_triggers) ? PERM_NONE : PERM_READ;
+					}
+					unset($link);
+
 					$db_triggers = $triggerids
 						? API::Trigger()->get([
 							'output' => [],
