@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2019 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -98,6 +98,7 @@ class ZBase {
 		require_once $this->getRootDir().'/include/draw.inc.php';
 		require_once $this->getRootDir().'/include/events.inc.php';
 		require_once $this->getRootDir().'/include/graphs.inc.php';
+		require_once $this->getRootDir().'/include/hostgroups.inc.php';
 		require_once $this->getRootDir().'/include/hosts.inc.php';
 		require_once $this->getRootDir().'/include/httptest.inc.php';
 		require_once $this->getRootDir().'/include/ident.inc.php';
@@ -126,13 +127,14 @@ class ZBase {
 				$this->loadConfigFile();
 				$this->initDB();
 				$this->authenticateUser();
-				$this->initLocales();
+				$this->initLocales(CWebUser::$data);
+				$this->setLayoutModeByUrl();
 				break;
 
 			case self::EXEC_MODE_API:
 				$this->loadConfigFile();
 				$this->initDB();
-				$this->initLocales();
+				$this->initLocales(['lang' => 'en_gb']);
 				break;
 
 			case self::EXEC_MODE_SETUP:
@@ -141,7 +143,7 @@ class ZBase {
 					$this->loadConfigFile();
 					$this->initDB();
 					$this->authenticateUser();
-					$this->initLocales();
+					$this->initLocales(CWebUser::$data);
 				}
 				catch (ConfigFileException $e) {}
 				break;
@@ -150,6 +152,7 @@ class ZBase {
 		// new MVC processing, otherwise we continue execution old style
 		if (hasRequest('action')) {
 			$router = new CRouter(getRequest('action'));
+
 			if ($router->getController() !== null) {
 				CProfiler::getInstance()->start();
 				$this->processRequest($router);
@@ -306,8 +309,11 @@ class ZBase {
 
 	/**
 	 * Initialize translations.
+	 *
+	 * @param array  $user_data          Array of user data.
+	 * @param string $user_data['lang']  Language.
 	 */
-	protected function initLocales() {
+	protected function initLocales(array $user_data) {
 		init_mbstrings();
 
 		$defaultLocales = [
@@ -316,7 +322,7 @@ class ZBase {
 
 		if (function_exists('bindtextdomain')) {
 			// initializing gettext translations depending on language selected by user
-			$locales = zbx_locale_variants(CWebUser::$data['lang']);
+			$locales = zbx_locale_variants($user_data['lang']);
 			$locale_found = false;
 			foreach ($locales as $locale) {
 				// since LC_MESSAGES may be unavailable on some systems, try to set all of the locales
@@ -328,7 +334,6 @@ class ZBase {
 
 				if (setlocale(LC_ALL, $locale)) {
 					$locale_found = true;
-					CWebUser::$data['locale'] = $locale;
 					break;
 				}
 			}
@@ -339,8 +344,8 @@ class ZBase {
 			// this will be unnecessary in PHP 5.5
 			setlocale(LC_CTYPE, $defaultLocales);
 
-			if (!$locale_found && CWebUser::$data['lang'] != 'en_GB' && CWebUser::$data['lang'] != 'en_gb') {
-				error('Locale for language "'.CWebUser::$data['lang'].'" is not found on the web server. Tried to set: '.implode(', ', $locales).'. Unable to translate Zabbix interface.');
+			if (!$locale_found && $user_data['lang'] != 'en_GB' && $user_data['lang'] != 'en_gb') {
+				error('Locale for language "'.$user_data['lang'].'" is not found on the web server. Tried to set: '.implode(', ', $locales).'. Unable to translate Zabbix interface.');
 			}
 			bindtextdomain('frontend', 'locale');
 			bind_textdomain_codeset('frontend', 'UTF-8');
@@ -438,7 +443,7 @@ class ZBase {
 			foreach ($_REQUEST as $key => $value) {
 				// do not output SID
 				if ($key != 'sid') {
-					$response->addMessage($key.': '.$value);
+					$response->addMessage(is_scalar($value) ? $key.': '.$value : $key.': '.gettype($value));
 				}
 			}
 			CSession::setValue('messages', $response->getMessages());
@@ -446,4 +451,19 @@ class ZBase {
 			redirect('zabbix.php?action=system.warning');
 		}
 	}
+
+	/**
+	 * Set layout to fullscreen or kiosk mode if URL contains 'fullscreen' and/or 'kiosk' arguments.
+	 */
+	private function setLayoutModeByUrl() {
+		if (array_key_exists('kiosk', $_GET) && $_GET['kiosk'] === '1') {
+			CView::setLayoutMode(ZBX_LAYOUT_KIOSKMODE);
+		}
+		elseif (array_key_exists('fullscreen', $_GET)) {
+			CView::setLayoutMode($_GET['fullscreen'] === '1' ? ZBX_LAYOUT_FULLSCREEN : ZBX_LAYOUT_NORMAL);
+		}
+
+		// Remove $_GET arguments to prevent CUrl from generating URL with 'fullscreen'/'kiosk' arguments.
+		unset($_GET['fullscreen'], $_GET['kiosk']);
+	}
 }
