@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2019 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -131,7 +131,8 @@ class CXmlExportWriter extends CExportWriter {
 			'query_fields' => 'query_field',
 			'posts' => 'post_field',
 			'shapes' => 'shape',
-			'lines' => 'line'
+			'lines' => 'line',
+			'headers' => 'header'
 		];
 
 		return isset($map[$name]) ? $map[$name] : false;
