@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2017 Zabbix SIA
+** Copyright (C) 2001-2019 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -32,34 +32,38 @@ class CTableInfo extends CTable {
 	}
 
 	public function toString($destroy = true) {
-		$tableId = $this->getId();
+		$tableid = $this->getId();
 
-		if(!$tableId) {
-			$tableId = uniqid('t');
-			$this->setId($tableId);
+		if (!$tableid) {
+			$tableid = uniqid('t', true);
+			$tableid = str_replace('.', '', $tableid);
+			$this->setId($tableid);
 		}
 
 		$string = parent::toString($destroy);
 
 		if ($this->addMakeVerticalRotationJs) {
 			$string .= get_js(
-				'var makeVerticalRotationForTable = function() {'."\n".
-				'	jQuery("#'.$tableId.'").makeVerticalRotation();'."\n".
-				'}'."\n".
+				'var makeVerticalRotationForTable = function() {'.
+					'jQuery("#'.$tableid.'").makeVerticalRotation();'.
+				'}'.
 				"\n".
-				'if (!jQuery.isReady) {'."\n".
-				'	jQuery(document).ready(makeVerticalRotationForTable);'."\n".
-				'}'."\n".
-				'else {'."\n".
-				'	makeVerticalRotationForTable();'."\n".
+				'if (!jQuery.isReady) {'.
+					'jQuery(document).ready(makeVerticalRotationForTable);'.
+				'}'.
+				'else {'.
+					'makeVerticalRotationForTable();'.
 				'}',
-			true);
+				true
+			);
 		}
+
 		return $string;
 	}
 
 	public function setNoDataMessage($message) {
 		$this->message = $message;
+
 		return $this;
 	}
 
@@ -69,15 +73,17 @@ class CTableInfo extends CTable {
 	 */
 	public function makeVerticalRotation() {
 		$this->addMakeVerticalRotationJs = true;
+
 		return $this;
 	}
 
-	public function endToString() {
+	protected function endToString() {
 		$ret = '';
 		if ($this->rownum == 0 && $this->message !== null) {
 			$ret .= $this->prepareRow(new CCol($this->message), ZBX_STYLE_NOTHING_TO_SHOW)->toString();
 		}
 		$ret .= parent::endToString();
+
 		return $ret;
 	}
 }
