@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2019 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -269,11 +269,11 @@ class CHelpItems {
 					'description' => _('Disk write statistics. Returns integer with type in sectors, operations, bytes; float with type in sps, ops, bps')
 				],
 				[
-					'key' => 'vfs.dir.count[dir,<regex_incl>,<regex_excl>,<types_incl>,<types_excl>,<max_depth>,<min_size>,<max_size>,<min_age>,<max_age>]',
+					'key' => 'vfs.dir.count[dir,<regex_incl>,<regex_excl>,<types_incl>,<types_excl>,<max_depth>,<min_size>,<max_size>,<min_age>,<max_age>,<regex_excl_dir>]',
 					'description' => _('Count of directory entries, recursively. Returns integer')
 				],
 				[
-					'key' => 'vfs.dir.size[dir,<regex_incl>,<regex_excl>,<mode>,<max_depth>]',
+					'key' => 'vfs.dir.size[dir,<regex_incl>,<regex_excl>,<mode>,<max_depth>,<regex_excl_dir>]',
 					'description' => _('Directory size (in bytes). Returns integer')
 				],
 				[
@@ -333,12 +333,20 @@ class CHelpItems {
 					'description' => _('Loading time of full web page (in seconds). Returns float')
 				],
 				[
-					'key' => 'web.page.regexp[host,<path>,<port>,<regexp>,<length>,<output>]',
+					'key' => 'web.page.regexp[host,<path>,<port>,regexp,<length>,<output>]',
 					'description' => _('Find string on a web page. Returns the matched string, or as specified by the optional output parameter')
 				],
 				[
 					'key' => 'wmi.get[<namespace>,<query>]',
 					'description' => _('Execute WMI query and return the first selected object. Returns integer, float, string or text (depending on the request)')
+				],
+				[
+					'key' => 'zabbix.stats[<ip>,<port>]',
+					'description' => _('Returns a JSON object containing Zabbix server or proxy internal metrics.')
+				],
+				[
+					'key' => 'zabbix.stats[<ip>,<port>,queue,<from>,<to>]',
+					'description' => _('Number of items in the queue which are delayed in Zabbix server or proxy by "from" till "to" seconds, inclusive.')
 				]
 			],
 			ITEM_TYPE_ZABBIX_ACTIVE => [
@@ -571,11 +579,11 @@ class CHelpItems {
 					'description' => _('Disk write statistics. Returns integer with type in sectors, operations, bytes; float with type in sps, ops, bps')
 				],
 				[
-					'key' => 'vfs.dir.count[dir,<regex_incl>,<regex_excl>,<types_incl>,<types_excl>,<max_depth>,<min_size>,<max_size>,<min_age>,<max_age>]',
+					'key' => 'vfs.dir.count[dir,<regex_incl>,<regex_excl>,<types_incl>,<types_excl>,<max_depth>,<min_size>,<max_size>,<min_age>,<max_age>,<regex_excl_dir>]',
 					'description' => _('Count of directory entries, recursively. Returns integer')
 				],
 				[
-					'key' => 'vfs.dir.size[dir,<regex_incl>,<regex_excl>,<mode>,<max_depth>]',
+					'key' => 'vfs.dir.size[dir,<regex_incl>,<regex_excl>,<mode>,<max_depth>,<regex_excl_dir>]',
 					'description' => _('Directory size (in bytes). Returns integer')
 				],
 				[
@@ -635,12 +643,20 @@ class CHelpItems {
 					'description' => _('Loading time of full web page (in seconds). Returns float')
 				],
 				[
-					'key' => 'web.page.regexp[host,<path>,<port>,<regexp>,<length>,<output>]',
+					'key' => 'web.page.regexp[host,<path>,<port>,regexp,<length>,<output>]',
 					'description' => _('Find string on a web page. Returns the matched string, or as specified by the optional output parameter')
 				],
 				[
 					'key' => 'wmi.get[<namespace>,<query>]',
 					'description' => _('Execute WMI query and return the first selected object. Returns integer, float, string or text (depending on the request)')
+				],
+				[
+					'key' => 'zabbix.stats[<ip>,<port>]',
+					'description' => _('Returns a JSON object containing Zabbix server or proxy internal metrics.')
+				],
+				[
+					'key' => 'zabbix.stats[<ip>,<port>,queue,<from>,<to>]',
+					'description' => _('Number of items in the queue which are delayed in Zabbix server or proxy by "from" till "to" seconds, inclusive.')
 				]
 			],
 			ITEM_TYPE_AGGREGATE => [
@@ -683,13 +699,29 @@ class CHelpItems {
 					'description' => _('VMware cluster status, <url> - VMware service URL, <name> - VMware cluster name')
 				],
 				[
-					'key' => 'vmware.eventlog[<url>]',
-					'description' => _('VMware event log, <url> - VMware service URL')
+					'key' => 'vmware.eventlog[<url>,<mode>]',
+					'description' => _('VMware event log, <url> - VMware service URL, <mode> - all (default), skip - skip processing of older data')
 				],
 				[
 					'key' => 'vmware.fullname[<url>]',
 					'description' => _('VMware service full name, <url> - VMware service URL')
 				],
+				[
+					'key' => 'vmware.datastore.read[<url>,<datastore>,<mode>]',
+					'description' => _('VMware datastore read statistics, <url> - VMware service URL, <datastore> - datastore name, <mode> - latency/maxlatency - average or maximum')
+				],
+				[
+					'key' => 'vmware.datastore.size[<url>,<datastore>,<mode>]',
+					'description' => _('VMware datastore capacity statistics in bytes or in percentage from total. Returns integer for bytes; float for percentage')
+				],
+				[
+					'key' => 'vmware.datastore.write[<url>,<datastore>,<mode>]',
+					'description' => _('VMware datastore write statistics, <url> - VMware service URL, <datastore> - datastore name, <mode> - latency/maxlatency - average or maximum')
+				],
+				[
+					'key' => 'vmware.datastore.hv.list[<url>,<datastore>]',
+					'description' => _('VMware datastore hypervisors list, <url> - VMware service URL, <datastore> - datastore name')
+				],
 				[
 					'key' => 'vmware.hv.cluster.name[<url>,<uuid>]',
 					'description' => _('VMware hypervisor cluster name, <url> - VMware service URL, <uuid> - VMware hypervisor host name')
@@ -710,6 +742,10 @@ class CHelpItems {
 					'key' => 'vmware.hv.datastore.write[<url>,<uuid>,<datastore>,<mode>]',
 					'description' => _('VMware hypervisor datastore write statistics, <url> - VMware service URL, <uuid> - VMware hypervisor host name, <datastore> - datastore name, <mode> - latency')
 				],
+				[
+					'key' => 'vmware.hv.datastore.list[<url>,<uuid>]',
+					'description' => _('VMware hypervisor datastores list, <url> - VMware service URL, <uuid> - VMware hypervisor host name')
+				],
 				[
 					'key' => 'vmware.hv.full.name[<url>,<uuid>]',
 					'description' => _('VMware hypervisor name, <url> - VMware service URL, <uuid> - VMware hypervisor host name')
@@ -978,6 +1014,14 @@ class CHelpItems {
 					'key' => 'zabbix[requiredperformance]',
 					'description' => _('Required performance of the Zabbix server, in new values per second expected.')
 				],
+				[
+					'key' => 'zabbix[stats,<ip>,<port>]',
+					'description' => _('Returns a JSON object containing Zabbix server or proxy internal metrics.')
+				],
+				[
+					'key' => 'zabbix[stats,<ip>,<port>,queue,<from>,<to>]',
+					'description' => _('Number of items in the queue which are delayed in Zabbix server or proxy by "from" till "to" seconds, inclusive.')
+				],
 				[
 					'key' => 'zabbix[trends]',
 					'description' => _('Number of values stored in table TRENDS.')
@@ -1013,11 +1057,11 @@ class CHelpItems {
 			],
 			ITEM_TYPE_DB_MONITOR => [
 				[
-					'key' => 'db.odbc.select[<unique short description>,<dsn>]',
+					'key' => 'db.odbc.select[<unique short description>,dsn]',
 					'description' => _('Return first column of the first row of the SQL query result.')
 				],
 				[
-					'key' => 'db.odbc.discovery[<unique short description>,<dsn>]',
+					'key' => 'db.odbc.discovery[<unique short description>,dsn]',
 					'description' => _('Transform SQL query result into a JSON object for low-level discovery.')
 				]
 			],
