@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2019 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -403,7 +403,11 @@ function DBselect($query, $limit = null, $offset = 0) {
 		$DB['TRANSACTION_NO_FAILED_SQLS'] = false;
 	}
 
-	CProfiler::getInstance()->profileSql(microtime(true) - $time_start, $query);
+	if (CApiService::$userData !== null && array_key_exists('debug_mode', CApiService::$userData)
+			&& CApiService::$userData['debug_mode'] == GROUP_DEBUG_MODE_ENABLED) {
+		CProfiler::getInstance()->profileSql(microtime(true) - $time_start, $query);
+	}
+
 	return $result;
 }
 
@@ -515,7 +519,11 @@ function DBexecute($query, $skip_error_messages = 0) {
 		$DB['TRANSACTION_NO_FAILED_SQLS'] = false;
 	}
 
-	CProfiler::getInstance()->profileSql(microtime(true) - $time_start, $query);
+	if (CApiService::$userData !== null && array_key_exists('debug_mode', CApiService::$userData)
+			&& CApiService::$userData['debug_mode'] == GROUP_DEBUG_MODE_ENABLED) {
+		CProfiler::getInstance()->profileSql(microtime(true) - $time_start, $query);
+	}
+
 	return (bool) $result;
 }
 
@@ -523,7 +531,7 @@ function DBexecute($query, $skip_error_messages = 0) {
  * Returns the next data set from a DB resource or false if there are no more results.
  *
  * @param resource $cursor
- * @param bool $convertNulls	convert all null values to string zeroes
+ * @param bool     $convertNulls  Convert all null values to string zeros.
  *
  * @return array|bool
  */
@@ -772,15 +780,12 @@ function check_db_fields($dbFields, &$args) {
  * @param string $fieldName		field name to be used in SQL WHERE condition
  * @param array  $values		array of numerical values sorted in ascending order to be included in WHERE
  * @param bool   $notIn			builds inverted condition
- * @param bool   $sort			values mandatory must be sorted
- * @param bool   $quote
  * @param bool   $zero_to_null
  *
  * @return string
  */
-function dbConditionInt($fieldName, array $values, $notIn = false, $sort = true, $quote = true, $zero_to_null = false) {
-	$MAX_EXPRESSIONS = 950; // maximum  number of values for using "IN (id1>,<id2>,...,<idN>)"
-	$MIN_NUM_BETWEEN = 4; // minimum number of consecutive values for using "BETWEEN <id1> AND <idN>"
+function dbConditionInt($fieldName, array $values, $notIn = false, $zero_to_null = false) {
+	$MAX_EXPRESSIONS = 950; // maximum  number of values for using "IN (<id1>,<id2>,...,<idN>)"
 
 	if (is_bool(reset($values))) {
 		return '1=0';
@@ -796,69 +801,34 @@ function dbConditionInt($fieldName, array $values, $notIn = false, $sort = true,
 	}
 
 	$values = array_keys($values);
+	natsort($values);
+	$values = array_values($values);
 
-	if ($sort) {
-		natsort($values);
-
-		$values = array_values($values);
-	}
-
-	$betweens = [];
-	$data = [];
-
-	for ($i = 0, $size = count($values); $i < $size; $i++) {
-		// analyze by chunk
-		if ($i + $MIN_NUM_BETWEEN < $size && bcsub($values[$i + $MIN_NUM_BETWEEN], $values[$i]) == $MIN_NUM_BETWEEN) {
-			$between = array_slice($values, $i, $MIN_NUM_BETWEEN);
-
-			// analyze by one
-			for ($i += $MIN_NUM_BETWEEN; $i < $size && bcsub($values[$i], $values[$i - 1]) == 1; $i++) {
-				$between[] = $values[$i];
-			}
-
-			$betweens[] = $between;
-			$i--;
-		}
-		else {
-			$data[] = $values[$i];
+	foreach ($values as $i => $value) {
+		if (!ctype_digit((string) $value) || bccomp($value, ZBX_MAX_UINT64) > 0) {
+			$values[$i] = zbx_dbstr($value);
 		}
 	}
 
 	// concatenate conditions
-	$dataSize = count($data);
-
 	$condition = '';
 	$operatorAnd = $notIn ? ' AND ' : ' OR ';
 
-	if ($betweens) {
-		$operatorNot = $notIn ? 'NOT ' : '';
-
-		foreach ($betweens as $between) {
-			$between = $quote
-				? $operatorNot.$fieldName.' BETWEEN '.zbx_dbstr($between[0]).' AND '.zbx_dbstr(end($between))
-				: $operatorNot.$fieldName.' BETWEEN '.$between[0].' AND '.end($between);
-
-			$condition .= ($condition !== '') ? $operatorAnd.$between : $between;
-		}
-	}
-
 	$operatorNot = $notIn ? ' NOT' : '';
-	$chunks = array_chunk($data, $MAX_EXPRESSIONS);
-	$chunk_count = (int) $has_zero + count($betweens) + count($chunks);
+	$chunks = array_chunk($values, $MAX_EXPRESSIONS);
+	$chunk_count = (int) $has_zero + count($chunks);
 
 	foreach ($chunks as $chunk) {
 		if (count($chunk) == 1) {
 			$operator = $notIn ? '!=' : '=';
 
-			$condition .= $quote
-				? ($condition !== '' ? $operatorAnd : '').$fieldName.$operator.zbx_dbstr($chunk[0])
-				: ($condition !== '' ? $operatorAnd : '').$fieldName.$operator.$chunk[0];
+			$condition .= ($condition !== '' ? $operatorAnd : '').$fieldName.$operator.$chunk[0];
 		}
 		else {
 			$chunkIns = '';
 
 			foreach ($chunk as $value) {
-				$chunkIns .= $quote ? ','.zbx_dbstr($value) : ','.$value;
+				$chunkIns .= ','.$value;
 			}
 
 			$chunkIns = $fieldName.$operatorNot.' IN ('.substr($chunkIns, 1).')';
@@ -882,13 +852,11 @@ function dbConditionInt($fieldName, array $values, $notIn = false, $sort = true,
  * @param string $fieldName		field name to be used in SQL WHERE condition
  * @param array  $values		array of numerical values sorted in ascending order to be included in WHERE
  * @param bool   $notIn			builds inverted condition
- * @param bool   $sort			values mandatory must be sorted
- * @param bool   $quote
  *
  * @return string
  */
-function dbConditionId($fieldName, array $values, $notIn = false, $sort = true, $quote = true) {
-	return dbConditionInt($fieldName, $values, $notIn, $sort, $quote, true);
+function dbConditionId($fieldName, array $values, $notIn = false) {
+	return dbConditionInt($fieldName, $values, $notIn, true);
 }
 
 /**
@@ -954,20 +922,14 @@ function DBfetchArrayAssoc($cursor, $field) {
  *
  * @param resource $cursor
  * @param string   $column
- * @param bool     $asHash
  *
  * @return array
  */
-function DBfetchColumn($cursor, $column, $asHash = false) {
+function DBfetchColumn($cursor, $column) {
 	$result = [];
 
 	while ($dbResult = DBfetch($cursor)) {
-		if ($asHash) {
-			$result[$dbResult[$column]] = $dbResult[$column];
-		}
-		else {
-			$result[] = $dbResult[$column];
-		}
+		$result[] = $dbResult[$column];
 	}
 
 	return $result;
