@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2019 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -66,177 +66,30 @@ function sysmapElementLabel($label = null) {
  *
  * @param array $sysmap
  * @param array $sysmap['selements']
- * @param int   $sysmap['show_suppressed']  Whether suppressed problems are shown.
  * @param array $options                    Options used to retrieve actions.
  * @param int   $options['severity_min']    Minimal severity used.
  *
  * @return array
  */
 function getActionsBySysmap(array $sysmap, array $options = []) {
-	$sysmap['links'] = zbx_toHash($sysmap['links'], 'linkid');
-
 	$actions = [];
-	$hostids = [];
-	$triggerids = [];
-	$host_groupids = [];
-
-	foreach ($sysmap['selements'] as &$selement) {
-		if ($selement['elementtype'] == SYSMAP_ELEMENT_TYPE_HOST) {
-			$hostid = $selement['elements'][0]['hostid'];
-			$hostids[$hostid] = $hostid;
-
-			/*
-			 * Expanding host URL macros again as some hosts were added from hostgroup areas and automatic expanding
-			 * only happens for elements that are defined for map in DB.
-			 */
-			foreach ($selement['urls'] as $urlid => $url) {
-				$selement['urls'][$urlid]['url'] = str_replace('{HOST.ID}', $hostid, $url['url']);
-			}
-		}
-		elseif ($selement['elementtype'] == SYSMAP_ELEMENT_TYPE_TRIGGER) {
-			foreach ($selement['elements'] as $element) {
-				$triggerids[$element['triggerid']] = $element['triggerid'];
-			}
-		}
-		elseif ($selement['elementtype'] == SYSMAP_ELEMENT_TYPE_HOST_GROUP) {
-			$groupid = $selement['elements'][0]['groupid'];
-			$host_groupids[$groupid] = $groupid;
-		}
-	}
-	unset($selement);
-
-	$host_scripts = API::Script()->getScriptsByHosts($hostids);
-
-	$hosts = API::Host()->get([
-		'output' => ['hostid', 'status'],
-		'selectGraphs' => API_OUTPUT_COUNT,
-		'selectScreens' => API_OUTPUT_COUNT,
-		'hostids' => $hostids,
-		'preservekeys' => true
-	]);
-
-	$monitored_triggers_hosts = API::Host()->get([
-		'output' => ['hostid'],
-		'hostids' => $hostids,
-		'with_monitored_triggers' => true,
-		'preservekeys' => true
-	]);
-
-	$triggers = API::Trigger()->get([
-		'output' => [],
-		'triggerids' => $triggerids,
-		'preservekeys' => true
-	]);
-
-	$monitored_triggers = API::Trigger()->get([
-		'output' => [],
-		'triggerids' => array_keys($triggers),
-		'monitored' => true,
-		'preservekeys' => true
-	]);
-
-	$host_groups = API::HostGroup()->get([
-		'output' => ['groupid'],
-		'groupids' => $host_groupids,
-		'with_monitored_triggers' => true,
-		'preservekeys' => true
-	]);
+	$severity_min = array_key_exists('severity_min', $options)
+		? $options['severity_min']
+		: TRIGGER_SEVERITY_NOT_CLASSIFIED;
 
 	foreach ($sysmap['selements'] as $selementid => $elem) {
-		$hostid = null;
-		$scripts = null;
-		$gotos = null;
-
 		if ($elem['permission'] < PERM_READ) {
 			continue;
 		}
 
-		switch ($elem['elementtype']) {
-			case SYSMAP_ELEMENT_TYPE_HOST:
-				$hostid = $elem['elements'][0]['hostid'];
-				$host = $hosts[$hostid];
+		$hostid = ($elem['elementtype_orig'] == SYSMAP_ELEMENT_TYPE_HOST_GROUP
+				&& $elem['elementsubtype_orig'] == SYSMAP_ELEMENT_SUBTYPE_HOST_GROUP_ELEMENTS)
+			? $elem['elements'][0]['hostid']
+			: 0;
 
-				if (array_key_exists($hostid, $host_scripts)) {
-					$scripts = $host_scripts[$hostid];
-				}
+		$map = CMenuPopupHelper::getMapElement($sysmap['sysmapid'], $elem['selementid_orig'], $severity_min, $hostid);
 
-				$gotos['triggerStatus'] = [
-					'filter_hostids' => [$hostid],
-					'filter_severity' => isset($options['severity_min']) ? $options['severity_min'] : null
-				];
-				$gotos['showTriggers'] = ($host['status'] == HOST_STATUS_MONITORED
-						&& array_key_exists($hostid, $monitored_triggers_hosts));
-
-				$gotos['graphs'] = ['hostid' => $host['hostid']];
-				$gotos['showGraphs'] = (bool) $host['graphs'];
-
-				$gotos['screens'] = ['hostid' => $host['hostid']];
-				$gotos['showScreens'] = (bool) $host['screens'];
-
-				$gotos['inventory'] = ['hostid' => $host['hostid']];
-
-				$gotos['latestData'] = ['hostids' => [$host['hostid']]];
-
-				if ($sysmap['show_suppressed']) {
-					$gotos['show_suppressed'] = true;
-				}
-				break;
-
-			case SYSMAP_ELEMENT_TYPE_MAP:
-				$gotos['submap'] = [
-					'sysmapid' => $elem['elements'][0]['sysmapid'],
-					'severity_min' => isset($options['severity_min']) ? $options['severity_min'] : null
-				];
-
-				break;
-
-			case SYSMAP_ELEMENT_TYPE_TRIGGER:
-				$gotos['showEvents'] = false;
-				foreach ($elem['elements'] as $element) {
-					if (array_key_exists($element['triggerid'], $triggers)) {
-						if (array_key_exists($element['triggerid'], $monitored_triggers)) {
-							$gotos['showEvents'] = true;
-						}
-
-						$gotos['events']['triggerids'][] = $element['triggerid'];
-
-						if (array_key_exists('severity_min', $options) && zbx_ctype_digit($options['severity_min'])) {
-							$gotos['events']['severity_min'] = $options['severity_min'];
-						}
-					}
-				}
-
-				if ($sysmap['show_suppressed']) {
-					$gotos['show_suppressed'] = true;
-				}
-				break;
-
-			case SYSMAP_ELEMENT_TYPE_HOST_GROUP:
-				$gotos['triggerStatus'] = [
-					'filter_groupids' => [$elem['elements'][0]['groupid']],
-					'filter_severity' => isset($options['severity_min']) ? $options['severity_min'] : null
-				];
-
-				// Always show active trigger link for host group map elements.
-				$gotos['showTriggers'] = array_key_exists($elem['elements'][0]['groupid'], $host_groups);
-
-				if ($sysmap['show_suppressed']) {
-					$gotos['show_suppressed'] = true;
-				}
-				break;
-		}
-
-		order_result($elem['urls'], 'name');
-
-		$map = CMenuPopupHelper::getMap($hostid, $scripts, $gotos, $elem['urls']);
-		if ($map == ['type' => 'map']) {
-			$map = null;
-		}
-		else {
-			$map = CJs::encodeJson($map);
-		}
-
-		$actions[$selementid] = $map;
+		$actions[$selementid] = CJs::encodeJson($map);
 	}
 
 	return $actions;
@@ -661,17 +514,6 @@ function getHostGroupsInfo($selement, $i, $show_unack) {
 		];
 		$hasStatus = true;
 	}
-	elseif ($i['disabled']) {
-		if (!$hasProblem) {
-			$info['icon_type'] = SYSMAP_ELEMENT_ICON_DISABLED;
-			$info['iconid'] = $selement['iconid_disabled'];
-		}
-		$info['info']['disabled'] = [
-			'msg' => _('DISABLED'),
-			'color' => '960000'
-		];
-		$hasStatus = true;
-	}
 
 	if (!$hasStatus && !$hasProblem) {
 		$info['icon_type'] = SYSMAP_ELEMENT_ICON_OFF;
@@ -761,17 +603,6 @@ function getMapsInfo($selement, $i, $show_unack) {
 		];
 		$hasStatus = true;
 	}
-	elseif ($i['disabled']) {
-		if (!$hasProblem) {
-			$info['icon_type'] = SYSMAP_ELEMENT_ICON_DISABLED;
-			$info['iconid'] = $selement['iconid_disabled'];
-		}
-		$info['info']['disabled'] = [
-			'msg' => _('DISABLED'),
-			'color' => '960000'
-		];
-		$hasStatus = true;
-	}
 
 	if (!$hasStatus && !$hasProblem) {
 		$info['icon_type'] = SYSMAP_ELEMENT_ICON_OFF;
@@ -1147,6 +978,9 @@ function getSelementsInfo(array $sysmap, array $options = []) {
 		}
 
 		$critical_problem = [];
+		$trigger_order = ($selement['elementtype'] == SYSMAP_ELEMENT_TYPE_TRIGGER)
+			? zbx_objectValues($selement['elements'], 'triggerid')
+			: [];
 		$lately_changed = 0;
 
 		foreach ($selement['triggers'] as $trigger) {
@@ -1163,11 +997,25 @@ function getSelementsInfo(array $sysmap, array $options = []) {
 						if ($problem['acknowledged'] == EVENT_NOT_ACKNOWLEDGED) {
 							$i['problem_unack']++;
 						}
-					}
 
-					if (!$critical_problem || ($critical_problem['severity'] <= $problem['severity']
-							&& $critical_problem['eventid'] < $problem['eventid'])) {
-						$critical_problem = $problem;
+						if (!$critical_problem || $critical_problem['severity'] < $problem['severity']) {
+							$critical_problem = $problem;
+						}
+						elseif ($critical_problem['severity'] === $problem['severity']) {
+							if ($selement['elementtype'] == SYSMAP_ELEMENT_TYPE_TRIGGER) {
+								if ($problem['objectid'] === $critical_problem['objectid']
+										&& $critical_problem['eventid'] < $problem['eventid']) {
+									$critical_problem = $problem;
+								}
+								elseif (array_search($critical_problem['objectid'], $trigger_order)
+										> array_search($problem['objectid'], $trigger_order)) {
+									$critical_problem = $problem;
+								}
+							}
+							elseif ($critical_problem['eventid'] < $problem['eventid']) {
+								$critical_problem = $problem;
+							}
+						}
 					}
 
 					if ($problem['r_clock'] > $lately_changed) {
@@ -1854,7 +1702,7 @@ function calculateMapAreaLinkCoord($ax, $ay, $aWidth, $aHeight, $x2, $y2) {
  * @return int
  */
 function getIconByMapping($iconMap, $inventory) {
-	if (!empty($inventory['inventory'])) {
+	if ($inventory['inventory']['inventory_mode'] != HOST_INVENTORY_DISABLED) {
 		$inventories = getHostInventories();
 
 		foreach ($iconMap['mappings'] as $mapping) {
@@ -2155,6 +2003,8 @@ function getMapHighligts(array $map, array $map_info) {
  *
  * @param array $sysmap
  * @param array $sysmap['show_suppressed']  Whether to show suppressed problems.
+ * @param array $sysmap['show_unack']       Property specified in sysmap's 'Problem display' field. Used to determine
+ *                                          whether to show unacknowledged problems only.
  * @param array $options                    Options used to retrieve actions.
  * @param int   $options['severity_min']    Minimal severity used.
  *
@@ -2169,17 +2019,23 @@ function getMapLinktriggerInfo($sysmap, $options) {
 
 	foreach ($sysmap['links'] as $link) {
 		foreach ($link['linktriggers'] as $linktrigger) {
-			$triggerids[$linktrigger['triggerid']] = $linktrigger['triggerid'];
+			$triggerids[$linktrigger['triggerid']] = true;
 		}
 	}
 
 	$trigger_options = [
 		'output' => ['status', 'value', 'priority'],
-		'triggerids' => $triggerids,
+		'triggerids' => array_keys($triggerids),
+		'monitored' => true,
 		'preservekeys' => true
 	];
 
-	return getTriggersWithActualSeverity($trigger_options, $sysmap['show_suppressed']);
+	$problem_options = [
+		'show_suppressed' => $sysmap['show_suppressed'],
+		'acknowledged' => ($sysmap['show_unack'] == EXTACK_OPTION_UNACK) ? false : null
+	];
+
+	return getTriggersWithActualSeverity($trigger_options, $problem_options);
 }
 
 /**
