@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2019 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -34,11 +34,11 @@ function screen_resources($resource = null) {
 		SCREEN_RESOURCE_MAP => _('Map'),
 		SCREEN_RESOURCE_PLAIN_TEXT => _('Plain text'),
 		SCREEN_RESOURCE_SCREEN => _('Screen'),
-		SCREEN_RESOURCE_SERVER_INFO => _('Status of Zabbix'),
+		SCREEN_RESOURCE_SERVER_INFO => _('System information'),
 		SCREEN_RESOURCE_SIMPLE_GRAPH => _('Simple graph'),
 		SCREEN_RESOURCE_HOSTGROUP_TRIGGERS => _('Host group issues'),
 		SCREEN_RESOURCE_HOST_TRIGGERS => _('Host issues'),
-		SCREEN_RESOURCE_SYSTEM_STATUS => _('System status'),
+		SCREEN_RESOURCE_SYSTEM_STATUS => _('Problems by severity'),
 		SCREEN_RESOURCE_TRIGGER_INFO => _('Trigger info'),
 		SCREEN_RESOURCE_TRIGGER_OVERVIEW => _('Trigger overview'),
 		SCREEN_RESOURCE_URL => _('URL'),
