@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2014 Zabbix SIA
+** Copyright (C) 2001-2019 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -23,23 +23,6 @@ function unset_request($key) {
 	unset($_GET[$key], $_POST[$key], $_REQUEST[$key]);
 }
 
-function is_int_range($value) {
-	if (!empty($value)) {
-		foreach (explode(',', $value) as $int_range) {
-			$int_range = explode('-', $int_range);
-			if (count($int_range) > 2) {
-				return false;
-			}
-			foreach ($int_range as $int_val) {
-				if (!is_numeric($int_val)) {
-					return false;
-				}
-			}
-		}
-	}
-	return true;
-}
-
 function BETWEEN($min, $max, $var = null) {
 	return '({'.$var.'}>='.$min.'&&{'.$var.'}<='.$max.')&&';
 }
@@ -56,147 +39,6 @@ function HEX($var = null) {
 	return 'preg_match("/^([a-zA-Z0-9]+)$/",{'.$var.'})&&';
 }
 
-function KEY_PARAM($var = null) {
-	return 'preg_match("/'.ZBX_PREG_PARAMS.'/",{'.$var.'})&&';
-}
-
-function validate_ipv4($str, &$arr) {
-	if (!preg_match('/^([0-9]{1,3})\.([0-9]{1,3})\.([0-9]{1,3})\.([0-9]{1,3})$/', $str, $arr)) {
-		return false;
-	}
-
-	for ($i = 1; $i <= 4; $i++) {
-		if (!is_numeric($arr[$i]) || $arr[$i] > 255 || $arr[$i] < 0 ) {
-			return false;
-		}
-	}
-
-	return true;
-}
-
-function validate_ipv6($str) {
-	$pattern1 = '([a-f0-9]{1,4}:){7}[a-f0-9]{1,4}';
-	$pattern2 = ':(:[a-f0-9]{1,4}){1,7}';
-	$pattern3 = '[a-f0-9]{1,4}::([a-f0-9]{1,4}:){0,5}[a-f0-9]{1,4}';
-	$pattern4 = '([a-f0-9]{1,4}:){2}:([a-f0-9]{1,4}:){0,4}[a-f0-9]{1,4}';
-	$pattern5 = '([a-f0-9]{1,4}:){3}:([a-f0-9]{1,4}:){0,3}[a-f0-9]{1,4}';
-	$pattern6 = '([a-f0-9]{1,4}:){4}:([a-f0-9]{1,4}:){0,2}[a-f0-9]{1,4}';
-	$pattern7 = '([a-f0-9]{1,4}:){5}:([a-f0-9]{1,4}:){0,1}[a-f0-9]{1,4}';
-	$pattern8 = '([a-f0-9]{1,4}:){6}:[a-f0-9]{1,4}';
-	$pattern9 = '([a-f0-9]{1,4}:){1,7}:';
-	$pattern10 = '::';
-
-	$full = "/^($pattern1)$|^($pattern2)$|^($pattern3)$|^($pattern4)$|^($pattern5)$|^($pattern6)$|^($pattern7)$|^($pattern8)$|^($pattern9)$|^($pattern10)$/i";
-
-	if (!preg_match($full, $str)) {
-		return false;
-	}
-	return true;
-}
-
-function validate_ip($str, &$arr) {
-	if (validate_ipv4($str, $arr)) {
-		return true;
-	}
-
-	if (ZBX_HAVE_IPV6) {
-		return validate_ipv6($str);
-	}
-
-	return false;
-}
-
-/**
- * Validate IP mask. IP/bits.
- * bits range for IPv4: 16 - 30
- * bits range for IPv6: 112 - 128
- *
- * @param string $ip_range
- *
- * @return bool
- */
-function validate_ip_range_mask($ip_range) {
-	$parts = explode('/', $ip_range);
-
-	if (count($parts) != 2) {
-		return false;
-	}
-	$ip = $parts[0];
-	$bits = $parts[1];
-
-	if (validate_ipv4($ip, $arr)) {
-		return preg_match('/^\d{1,2}$/', $bits) && $bits >= 16 && $bits <= 30;
-	}
-	elseif (ZBX_HAVE_IPV6 && validate_ipv6($ip, $arr)) {
-		return preg_match('/^\d{1,3}$/', $bits) && $bits >= 112 && $bits <= 128;
-	}
-	else {
-		return false;
-	}
-}
-
-/*
- * Validate IP range. ***.***.***.***[-***]
- */
-function validate_ip_range_range($ip_range) {
-	$parts = explode('-', $ip_range);
-	if (($parts_count = count($parts)) > 2) {
-		return false;
-	}
-
-	if (validate_ipv4($parts[0], $arr)) {
-		$ip_parts = explode('.', $parts[0]);
-
-		if ($parts_count == 2) {
-			if (!preg_match('/^([0-9]{1,3})$/', $parts[1])) {
-				return false;
-			}
-			sscanf($ip_parts[3], "%d", $from_value);
-			sscanf($parts[1], "%d", $to_value);
-
-			if (($to_value > 255) || ($from_value > $to_value)) {
-				return false;
-			}
-		}
-	}
-	elseif (ZBX_HAVE_IPV6 && validate_ipv6($parts[0])) {
-		$ip_parts = explode(':', $parts[0]);
-		$ip_parts_count = count($ip_parts);
-
-		if ($parts_count == 2) {
-			if (!preg_match('/^([a-f0-9]{1,4})$/i', $parts[1])) {
-				return false;
-			}
-			sscanf($ip_parts[$ip_parts_count - 1], "%x", $from_value);
-			sscanf($parts[1], "%x", $to_value);
-
-			if ($from_value > $to_value) {
-				return false;
-			}
-		}
-	}
-	else {
-		return false;
-	}
-	return true;
-}
-
-function validate_ip_range($str) {
-	foreach (explode(',', $str) as $ip_range) {
-		if (strpos($ip_range, '/') !== false) {
-			if (!validate_ip_range_mask($ip_range)) {
-				return false;
-			}
-		}
-		else {
-			if (!validate_ip_range_range($ip_range)) {
-				return false;
-			}
-		}
-	}
-	return true;
-}
-
 function validate_port_list($str) {
 	foreach (explode(',', $str) as $port_range) {
 		$port_range = explode('-', $port_range);
@@ -281,7 +123,7 @@ function check_type(&$field, $flags, &$var, $type, $caption = null) {
 		$caption = $field;
 	}
 
-	if (is_array($var) && $type != T_ZBX_IP) {
+	if (is_array($var) && $type != T_ZBX_RANGE_TIME) {
 		$err = ZBX_VALID_OK;
 
 		foreach ($var as $v) {
@@ -294,32 +136,14 @@ function check_type(&$field, $flags, &$var, $type, $caption = null) {
 	$error = false;
 	$message = '';
 
-	if ($type == T_ZBX_IP) {
-		if (!validate_ip($var, $arr)) {
-			$error = true;
-			$message = _s('Field "%1$s" is not IP.', $caption);
-		}
-	}
-	elseif ($type == T_ZBX_IP_RANGE) {
-		if (!validate_ip_range($var)) {
-			$error = true;
-			$message = _s('Field "%1$s" is not IP range.', $caption);
-		}
-	}
-	elseif ($type == T_ZBX_INT_RANGE) {
-		if (!is_int_range($var)) {
-			$error = true;
-			$message = _s('Field "%1$s" is not integer list or range.', $caption);
-		}
-	}
-	elseif ($type == T_ZBX_INT) {
+	if ($type == T_ZBX_INT) {
 		if (!zbx_is_int($var)) {
 			$error = true;
 			$message = _s('Field "%1$s" is not integer.', $caption);
 		}
 	}
 	elseif ($type == T_ZBX_DBL) {
-		$decimalValidator = new CDecimalValidator(array(
+		$decimalValidator = new CDecimalValidator([
 			'maxPrecision' => 16,
 			'maxScale' => 4,
 			'messageInvalid' => _('Value "%2$s" of "%1$s" has incorrect decimal format.'),
@@ -335,7 +159,7 @@ function check_type(&$field, $flags, &$var, $type, $caption = null) {
 				'Value "%2$s" of "%1$s" has too many digits after the decimal point: '.
 				'it cannot have more than %3$s digits.'
 			)
-		));
+		]);
 		$decimalValidator->setObjectName($caption);
 
 		if (!$decimalValidator->validate($var)) {
@@ -344,14 +168,14 @@ function check_type(&$field, $flags, &$var, $type, $caption = null) {
 		}
 	}
 	elseif ($type == T_ZBX_DBL_BIG) {
-		$decimalValidator = new CDecimalValidator(array(
+		$decimalValidator = new CDecimalValidator([
 			'maxScale' => 4,
 			'messageInvalid' => _('Value "%2$s" of "%1$s" has incorrect decimal format.'),
 			'messageScale' => _(
 				'Value "%2$s" of "%1$s" has too many digits after the decimal point: '.
 				'it cannot have more than %3$s digits.'
 			)
-		));
+		]);
 		$decimalValidator->setObjectName($caption);
 
 		if (!$decimalValidator->validate($var)) {
@@ -360,9 +184,9 @@ function check_type(&$field, $flags, &$var, $type, $caption = null) {
 		}
 	}
 	elseif ($type == T_ZBX_DBL_STR) {
-		$decimalStringValidator = new CDecimalStringValidator(array(
+		$decimalStringValidator = new CDecimalStringValidator([
 			'messageInvalid' => _('Value "%2$s" of "%1$s" has incorrect decimal format.')
-		));
+		]);
 		$decimalStringValidator->setObjectName($caption);
 
 		if (!$decimalStringValidator->validate($var)) {
@@ -380,12 +204,45 @@ function check_type(&$field, $flags, &$var, $type, $caption = null) {
 		$colorValidator = new CColorValidator();
 
 		if (!$colorValidator->validate($var)) {
-			$var = 'FFFFFF';
-
 			$error = true;
 			$message = _s('Colour "%1$s" is not correct: expecting hexadecimal colour code (6 symbols).', $caption);
 		}
 	}
+	elseif ($type == T_ZBX_TP) {
+		$time_period_parser = new CTimePeriodsParser(['usermacros' => true]);
+
+		if ($time_period_parser->parse($var) != CParser::PARSE_SUCCESS) {
+			$error = true;
+			$message = _s('Field "%1$s" is not correct: %2$s', $caption, _('a time period is expected'));
+		}
+	}
+	elseif ($type == T_ZBX_TU) {
+		$simple_interval_parser = new CSimpleIntervalParser([
+			'usermacros' => ($flags & P_ALLOW_USER_MACRO),
+			'lldmacros' => ($flags & P_ALLOW_LLD_MACRO)
+		]);
+
+		if ($simple_interval_parser->parse($var) != CParser::PARSE_SUCCESS) {
+			$error = true;
+			$message = _s('Field "%1$s" is not correct: %2$s', $caption, _('a time unit is expected'));
+		}
+	}
+	elseif ($type == T_ZBX_RANGE_TIME) {
+		$range_time_parser = new CRangeTimeParser();
+
+		if (!is_string($var) || $range_time_parser->parse($var) != CParser::PARSE_SUCCESS) {
+			$error = true;
+			$message = _s('Field "%1$s" is not correct: %2$s', $caption, _('a time range is expected'));
+		}
+	}
+	elseif ($type == T_ZBX_ABS_TIME) {
+		$absolute_time_parser = new CAbsoluteTimeParser();
+
+		if (!is_string($var) || $absolute_time_parser->parse($var) != CParser::PARSE_SUCCESS) {
+			$error = true;
+			$message = _s('Field "%1$s" is not correct: %2$s', $caption, _('an explicit time is expected'));
+		}
+	}
 
 	if ($error) {
 		if ($flags & P_SYS) {
@@ -462,14 +319,18 @@ function check_field(&$fields, &$field, $checks) {
 		}
 		elseif ($flags & P_ACT) {
 			if (!isset($_REQUEST['sid'])
-					|| (isset($_COOKIE['zbx_sessionid'])
-							&& $_REQUEST['sid'] != substr($_COOKIE['zbx_sessionid'], 16, 16))) {
+					|| (array_key_exists(ZBX_SESSION_NAME, $_COOKIE)
+							&& $_REQUEST['sid'] != substr($_COOKIE[ZBX_SESSION_NAME], 16, 16))) {
 				info(_('Operation cannot be performed due to unauthorized request.'));
 				return ZBX_VALID_ERROR;
 			}
 		}
 	}
 
+	if ($flags & P_CRLF) {
+		$_REQUEST[$field] = CRLFtoLF($_REQUEST[$field]);
+	}
+
 	if (!($flags & P_NO_TRIM)) {
 		check_trim($_REQUEST[$field]);
 	}
@@ -490,9 +351,12 @@ function check_field(&$fields, &$field, $checks) {
 			info(_s('Incorrect value "%1$s" for "%2$s" field: must be between %3$s and %4$s.',
 				$_REQUEST[$field], $caption, $result[1], $result[2]));
 		}
-		else {
+		elseif (is_scalar($_REQUEST[$field])) {
 			info(_s('Incorrect value "%1$s" for "%2$s" field.', $_REQUEST[$field], $caption));
 		}
+		else {
+			info(_s('Incorrect value for "%1$s" field.', $caption));
+		}
 
 		return ($flags & P_SYS) ? ZBX_VALID_ERROR : ZBX_VALID_WARNING;
 	}
@@ -510,7 +374,7 @@ function invalid_url($msg = null) {
 
 	// backup messages before including page_header.php
 	$temp = $ZBX_MESSAGES;
-	$ZBX_MESSAGES = null;
+	$ZBX_MESSAGES = [];
 
 	require_once dirname(__FILE__).'/page_header.php';
 
@@ -522,15 +386,22 @@ function invalid_url($msg = null) {
 	require_once dirname(__FILE__).'/page_footer.php';
 }
 
-function check_fields(&$fields, $show_messages = true) {
+/**
+ * Validate request fields and return result flags.
+ *
+ * @param array $fields field schema together with validation rules
+ *
+ * @return integer appropriate result flags ZBX_VALID_OK | ZBX_VALID_ERROR | ZBX_VALID_WARNING
+ */
+function check_fields_raw(&$fields) {
 	// VAR	TYPE	OPTIONAL	FLAGS	VALIDATION	EXCEPTION
-	$system_fields = array(
-		'sid' =>			array(T_ZBX_STR, O_OPT, P_SYS, HEX(),		null),
-		'triggers_hash' =>	array(T_ZBX_STR, O_OPT, P_SYS, NOT_EMPTY,	null),
-		'print' =>			array(T_ZBX_INT, O_OPT, P_SYS, IN('1'),		null),
-		'page' =>			array(T_ZBX_INT, O_OPT, P_SYS, null,		null), // paging
-		'ddreset' =>		array(T_ZBX_INT, O_OPT, P_SYS, null,		null)
-	);
+	$system_fields = [
+		'sid' =>			[T_ZBX_STR, O_OPT, P_SYS, HEX(),		null],
+		'triggers_hash' =>	[T_ZBX_STR, O_OPT, P_SYS, NOT_EMPTY,	null],
+		'print' =>			[T_ZBX_INT, O_OPT, P_SYS, IN('1'),		null],
+		'page' =>			[T_ZBX_INT, O_OPT, P_SYS, null,		null], // paging
+		'ddreset' =>		[T_ZBX_INT, O_OPT, P_SYS, null,		null]
+	];
 	$fields = zbx_array_merge($system_fields, $fields);
 
 	$err = ZBX_VALID_OK;
@@ -547,17 +418,68 @@ function check_fields(&$fields, $show_messages = true) {
 
 	$fields = null;
 
-	if ($err&ZBX_VALID_ERROR) {
+	return $err;
+}
+
+/**
+ * Validate request fields and return true on success, false on error.
+ *
+ * @param array $fields field schema together with validation rules
+ * @param bool $show_messages do show messages on error
+ *
+ * @return bool true on success, false on error.
+ */
+function check_fields(&$fields, $show_messages = true) {
+	$err = check_fields_raw($fields);
+
+	if ($err & ZBX_VALID_ERROR) {
 		invalid_url();
 	}
 
 	if ($show_messages && $err != ZBX_VALID_OK) {
-		show_messages(($err == ZBX_VALID_OK), null, _('Page received incorrect data'));
+		show_messages(false, null, _('Page received incorrect data'));
 	}
 
 	return ($err == ZBX_VALID_OK);
 }
 
+/**
+ * Validate "from" and "to" parameters for allowed period.
+ *
+ * @param string|null from
+ * @param string|null to
+ */
+function validateTimeSelectorPeriod($from, $to) {
+	if ($from === null || $to === null) {
+		return;
+	}
+
+	$ts = [];
+	$range_time_parser = new CRangeTimeParser();
+
+	foreach (['from' => $from, 'to' => $to] as $field => $value) {
+		$range_time_parser->parse($value);
+		$ts[$field] = $range_time_parser->getDateTime($field === 'from')->getTimestamp();
+	}
+
+	$period = $ts['to'] - $ts['from'] + 1;
+
+	if ($period < ZBX_MIN_PERIOD) {
+		error(_n('Minimum time period to display is %1$s minute.',
+			'Minimum time period to display is %1$s minutes.', (int) ZBX_MIN_PERIOD / SEC_PER_MIN
+		));
+
+		invalid_url();
+	}
+	elseif ($period > ZBX_MAX_PERIOD) {
+		error(_n('Maximum time period to display is %1$s day.',
+			'Maximum time period to display is %1$s days.', (int) ZBX_MAX_PERIOD / SEC_PER_DAY
+		));
+
+		invalid_url();
+	}
+}
+
 function validatePortNumberOrMacro($port) {
 	return (validatePortNumber($port) || validateUserMacro($port));
 }
@@ -583,7 +505,7 @@ function validateNumber($value, $min = null, $max = null) {
 }
 
 function validateUserMacro($value) {
-	return preg_match('/^'.ZBX_PREG_EXPRESSION_USER_MACROS.'$/', $value);
+	return ((new CUserMacroParser())->parse($value) == CParser::PARSE_SUCCESS);
 }
 
 /**
@@ -629,3 +551,46 @@ function validateDateTime($year, $month, $day, $hours, $minutes, $seconds = null
 function validateDateInterval($year, $month, $day) {
 	return !($year < 1970 || $year > 2038 || ($year == 2038 && (($month > 1) || ($month == 1 && $day > 18))));
 }
+
+/**
+ * Validate a configuration value. Use simple interval parser to parse the string, convert to seconds and check
+ * if the value is in between given min and max values. In some cases it's possible to enter 0, or even 0s or 0d.
+ * If the value is incorrect, set an error.
+ *
+ * @param string $value                  Value to parse and validate.
+ * @param int    $min                    Lowed bound.
+ * @param int    $max                    Upper bound.
+ * @param bool   $allow_zero             Set to "true" to allow value to be zero.
+ * @param string $error
+ * @param array  $options
+ * @param bool   $options['usermacros']
+ * @param bool   $options['lldmacros']
+ *
+ * @return bool
+ */
+function validateTimeUnit($value, $min, $max, $allow_zero, &$error, array $options = []) {
+	$simple_interval_parser = new CSimpleIntervalParser($options);
+
+	if ($simple_interval_parser->parse($value) == CParser::PARSE_SUCCESS) {
+		if ($value[0] !== '{') {
+			$value = timeUnitToSeconds($value);
+
+			if ($allow_zero && $value == 0) {
+				return true;
+			}
+
+			if ($value < $min || $value > $max) {
+				$error = _s('value must be one of %1$s', $allow_zero ? '0, '.$min.'-'.$max : $min.'-'.$max);
+
+				return false;
+			}
+		}
+	}
+	else {
+		$error = _('a time unit is expected');
+
+		return false;
+	}
+
+	return true;
+}
