@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2016 Zabbix SIA
+** Copyright (C) 2001-2019 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -23,52 +23,67 @@ require_once dirname(__FILE__).'/js/administration.general.housekeeper.edit.js.p
 
 $widget = (new CWidget())
 	->setTitle(_('Housekeeping'))
-	->setControls((new CForm())
-		->cleanItems()
-		->addItem((new CList())->addItem(makeAdministrationGeneralMenu('adm.housekeeper.php')))
+	->setControls((new CTag('nav', true,
+		(new CForm())
+			->cleanItems()
+			->addItem((new CList())
+				->addItem(makeAdministrationGeneralMenu('adm.housekeeper.php'))
+			)
+		))
+			->setAttribute('aria-label', _('Content controls'))
 	);
 
 $houseKeeperTab = (new CFormList())
 	->addRow(new CTag('h4', true, _('Events and alerts')))
 	->addRow(
 		new CLabel(_('Enable internal housekeeping'), 'hk_events_mode'),
-		(new CCheckBox('hk_events_mode'))->setChecked($data['hk_events_mode'] == 1)
+		(new CCheckBox('hk_events_mode'))
+			->setChecked($data['hk_events_mode'] == 1)
+			->setAttribute('autofocus', 'autofocus')
 	)
 	->addRow(
-		new CLabel(_('Trigger data storage period (in days)'), 'hk_events_trigger'),
-		(new CNumericBox('hk_events_trigger', $data['hk_events_trigger'], 5))
-			->setWidth(ZBX_TEXTAREA_NUMERIC_STANDARD_WIDTH)
+		(new CLabel(_('Trigger data storage period'), 'hk_events_trigger'))->setAsteriskMark(),
+		(new CTextBox('hk_events_trigger', $data['hk_events_trigger']))
+			->setWidth(ZBX_TEXTAREA_TINY_WIDTH)
 			->setEnabled($data['hk_events_mode'] == 1)
+			->setAriaRequired()
 	)
 	->addRow(
-		new CLabel(_('Internal data storage period (in days)'), 'hk_events_internal'),
-		(new CNumericBox('hk_events_internal', $data['hk_events_internal'], 5))
-			->setWidth(ZBX_TEXTAREA_NUMERIC_STANDARD_WIDTH)
+		(new CLabel(_('Internal data storage period'), 'hk_events_internal'))->setAsteriskMark(),
+		(new CTextBox('hk_events_internal', $data['hk_events_internal']))
+			->setWidth(ZBX_TEXTAREA_TINY_WIDTH)
 			->setEnabled($data['hk_events_mode'] == 1)
+			->setAriaRequired()
 	)
 	->addRow(
-		new CLabel(_('Network discovery data storage period (in days)'), 'hk_events_discovery'),
-		(new CNumericBox('hk_events_discovery', $data['hk_events_discovery'], 5))
-			->setWidth(ZBX_TEXTAREA_NUMERIC_STANDARD_WIDTH)
+		(new CLabel(_('Network discovery data storage period'), 'hk_events_discovery'))
+			->setAsteriskMark(),
+		(new CTextBox('hk_events_discovery', $data['hk_events_discovery']))
+			->setWidth(ZBX_TEXTAREA_TINY_WIDTH)
 			->setEnabled($data['hk_events_mode'] == 1)
+			->setAriaRequired()
 	)
 	->addRow(
-		new CLabel(_('Auto-registration data storage period (in days)'), 'hk_events_autoreg'),
-		(new CNumericBox('hk_events_autoreg', $data['hk_events_autoreg'], 5))
-			->setWidth(ZBX_TEXTAREA_NUMERIC_STANDARD_WIDTH)
+		(new CLabel(_('Auto-registration data storage period'), 'hk_events_autoreg'))
+			->setAsteriskMark(),
+		(new CTextBox('hk_events_autoreg', $data['hk_events_autoreg']))
+			->setWidth(ZBX_TEXTAREA_TINY_WIDTH)
 			->setEnabled($data['hk_events_mode'] == 1)
+			->setAriaRequired()
 	)
 	->addRow(null)
-	->addRow(new CTag('h4', true, _('IT services')))
+	->addRow(new CTag('h4', true, _('Services')))
 	->addRow(
 		new CLabel(_('Enable internal housekeeping'), 'hk_services_mode'),
 		(new CCheckBox('hk_services_mode'))->setChecked($data['hk_services_mode'] == 1)
 	)
 	->addRow(
-		new CLabel(_('Data storage period (in days)'), 'hk_services'),
-		(new CNumericBox('hk_services', $data['hk_services'], 5))
-			->setWidth(ZBX_TEXTAREA_NUMERIC_STANDARD_WIDTH)
+		(new CLabel(_('Data storage period'), 'hk_services'))
+			->setAsteriskMark(),
+		(new CTextBox('hk_services', $data['hk_services']))
+			->setWidth(ZBX_TEXTAREA_TINY_WIDTH)
 			->setEnabled($data['hk_services_mode'] == 1)
+			->setAriaRequired()
 	)
 	->addRow(null)
 	->addRow(new CTag('h4', true, _('Audit')))
@@ -77,10 +92,12 @@ $houseKeeperTab = (new CFormList())
 		(new CCheckBox('hk_audit_mode'))->setChecked($data['hk_audit_mode'] == 1)
 	)
 	->addRow(
-		new CLabel(_('Data storage period (in days)'), 'hk_audit'),
-		(new CNumericBox('hk_audit', $data['hk_audit'], 5))
-			->setWidth(ZBX_TEXTAREA_NUMERIC_STANDARD_WIDTH)
+		(new CLabel(_('Data storage period'), 'hk_audit'))
+			->setAsteriskMark(),
+		(new CTextBox('hk_audit', $data['hk_audit']))
+			->setWidth(ZBX_TEXTAREA_TINY_WIDTH)
 			->setEnabled($data['hk_audit_mode'] == 1)
+			->setAriaRequired()
 	)
 	->addRow(null)
 	->addRow(new CTag('h4', true, _('User sessions')))
@@ -89,10 +106,12 @@ $houseKeeperTab = (new CFormList())
 		(new CCheckBox('hk_sessions_mode'))->setChecked($data['hk_sessions_mode'] == 1)
 	)
 	->addRow(
-		new CLabel(_('Data storage period (in days)'), 'hk_sessions'),
-		(new CNumericBox('hk_sessions', $data['hk_sessions'], 5))
-			->setWidth(ZBX_TEXTAREA_NUMERIC_STANDARD_WIDTH)
+		(new CLabel(_('Data storage period'), 'hk_sessions'))
+			->setAsteriskMark(),
+		(new CTextBox('hk_sessions', $data['hk_sessions']))
+			->setWidth(ZBX_TEXTAREA_TINY_WIDTH)
 			->setEnabled($data['hk_sessions_mode'] == 1)
+			->setAriaRequired()
 	)
 	->addRow(null)
 	->addRow(new CTag('h4', true, _('History')))
@@ -105,10 +124,12 @@ $houseKeeperTab = (new CFormList())
 		(new CCheckBox('hk_history_global'))->setChecked($data['hk_history_global'] == 1)
 	)
 	->addRow(
-		new CLabel(_('Data storage period (in days)'), 'hk_history'),
-		(new CNumericBox('hk_history', $data['hk_history'], 5))
-			->setWidth(ZBX_TEXTAREA_NUMERIC_STANDARD_WIDTH)
+		(new CLabel(_('Data storage period'), 'hk_history'))
+			->setAsteriskMark(),
+		(new CTextBox('hk_history', $data['hk_history']))
+			->setWidth(ZBX_TEXTAREA_TINY_WIDTH)
 			->setEnabled($data['hk_history_global'] == 1)
+			->setAriaRequired()
 	)
 	->addRow(null)
 	->addRow(new CTag('h4', true, _('Trends')))
@@ -121,10 +142,12 @@ $houseKeeperTab = (new CFormList())
 		(new CCheckBox('hk_trends_global'))->setChecked($data['hk_trends_global'] == 1)
 	)
 	->addRow(
-		new CLabel(_('Data storage period (in days)'), 'hk_trends'),
-		(new CNumericBox('hk_trends', $data['hk_trends'], 5))
-			->setWidth(ZBX_TEXTAREA_NUMERIC_STANDARD_WIDTH)
+		(new CLabel(_('Data storage period'), 'hk_trends'))
+			->setAsteriskMark(),
+		(new CTextBox('hk_trends', $data['hk_trends']))
+			->setWidth(ZBX_TEXTAREA_TINY_WIDTH)
 			->setEnabled($data['hk_trends_global'] == 1)
+			->setAriaRequired()
 	);
 
 $houseKeeperView = (new CTabView())
@@ -134,6 +157,9 @@ $houseKeeperView = (new CTabView())
 		[new CButton('resetDefaults', _('Reset defaults'))]
 	));
 
-$widget->addItem((new CForm())->addItem($houseKeeperView));
+$widget->addItem((new CForm())
+	->setAttribute('aria-labeledby', ZBX_STYLE_PAGE_TITLE)
+	->addItem($houseKeeperView)
+);
 
 return $widget;
