@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2019 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -70,7 +70,14 @@ $form->addItem([
 	$table,
 	$data['paging'],
 	new CActionButtonList('action', 'valuemapids', [
-		'valuemap.export' => ['name' => _('Export')],
+		'valuemap.export' => ['name' => _('Export'), 'redirect' =>
+			(new CUrl('zabbix.php'))
+				->setArgument('action', 'export.valuemaps.xml')
+				->setArgument('backurl', (new CUrl('adm.valuemapping.php'))
+					->setArgument('page', getPageNumber())
+					->getUrl())
+				->getUrl()
+		],
 		'valuemap.delete' => ['name' => _('Delete'), 'confirm' => _('Delete selected value maps?')]
 	])
 ]);
