@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2019 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -21,23 +21,30 @@
 
 $widget = (new CWidget())
 	->setTitle(_('Working time'))
-	->setControls((new CForm())
-		->cleanItems()
-		->addItem((new CList())->addItem(makeAdministrationGeneralMenu('adm.workingtime.php')))
+	->setControls((new CTag('nav', true,
+		(new CForm())
+			->cleanItems()
+			->addItem((new CList())
+				->addItem(makeAdministrationGeneralMenu('adm.workingtime.php'))
+			)
+		))
+			->setAttribute('aria-label', _('Content controls'))
 	);
 
 $workingTimeView = (new CTabView())
 	->addTab('workingTime', _('Working time'),
 		(new CFormList())
-			->addRow(_('Working time'),
+			->addRow((new CLabel(_('Working time'), 'work_period'))->setAsteriskMark(),
 				(new CTextBox('work_period', $data['work_period']))
 					->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
+					->setAriaRequired()
 					->setAttribute('autofocus', 'autofocus')
 			)
 	)
 	->setFooter(makeFormFooter(new CSubmit('update', _('Update'))));
 
 $workingTimeForm = (new CForm())
+	->setAttribute('aria-labeledby', ZBX_STYLE_PAGE_TITLE)
 	->addItem($workingTimeView);
 
 $widget->addItem($workingTimeForm);
