@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2016 Zabbix SIA
+** Copyright (C) 2001-2019 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -26,6 +26,7 @@ $hostWidget = (new CWidget())->setTitle(_('Hosts'));
 // create form
 $hostView = (new CForm())
 	->setName('hostForm')
+	->setAttribute('aria-labeledby', ZBX_STYLE_PAGE_TITLE)
 	->addVar('action', 'host.massupdate')
 	->addVar('tls_accept', $data['tls_accept'])
 	->setAttribute('id', 'hostForm');
@@ -37,127 +38,128 @@ foreach ($data['hosts'] as $hostid) {
 $hostFormList = new CFormList('hostFormList');
 
 // replace host groups
-$hostGroupsToReplace = null;
-if (isset($_REQUEST['groups'])) {
-	$getHostGroups = API::HostGroup()->get([
-		'groupids' => $_REQUEST['groups'],
+$hostgroups_to_replace = isset($_REQUEST['groups'])
+	? CArrayHelper::renameObjectsKeys(API::HostGroup()->get([
 		'output' => ['groupid', 'name'],
+		'groupids' => $_REQUEST['groups'],
 		'editable' => true
-	]);
-	foreach ($getHostGroups as $getHostGroup) {
-		$hostGroupsToReplace[] = [
-			'id' => $getHostGroup['groupid'],
-			'name' => $getHostGroup['name']
-		];
-	}
-}
+	]), ['groupid' => 'id'])
+	: [];
 
 $replaceGroups = (new CDiv(
 	(new CMultiSelect([
 		'name' => 'groups[]',
-		'objectName' => 'hostGroup',
-		'objectOptions' => ['editable' => true],
-		'data' => $hostGroupsToReplace,
+		'object_name' => 'hostGroup',
+		'data' => $hostgroups_to_replace,
 		'popup' => [
-			'parameters' => 'srctbl=host_groups&dstfrm='.$hostView->getName().'&dstfld1=groups_&srcfld1=groupid'.
-				'&writeonly=1&multiselect=1'
+			'parameters' => [
+				'srctbl' => 'host_groups',
+				'srcfld1' => 'groupid',
+				'dstfrm' => $hostView->getName(),
+				'dstfld1' => 'groups_',
+				'editable' => true
+			]
 		]
 	]))->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
 ))->setId('replaceGroups');
 
 $hostFormList->addRow(
-	[
-		_('Replace host groups'),
-		SPACE,
-		(new CVisibilityBox('visible[groups]', 'replaceGroups', _('Original')))
-			->setChecked(isset($data['visible']['groups']))
-	],
+	(new CVisibilityBox('visible[groups]', 'replaceGroups', _('Original')))
+		->setLabel(_('Replace host groups'))
+		->setChecked(isset($data['visible']['groups']))
+		->setAttribute('autofocus', 'autofocus'),
 	$replaceGroups
 );
 
 // add new or existing host groups
-$hostGroupsToAdd = null;
+$hostgroups_to_add = [];
 if (isset($_REQUEST['new_groups'])) {
+	$groupids = [];
+
 	foreach ($_REQUEST['new_groups'] as $newHostGroup) {
 		if (is_array($newHostGroup) && isset($newHostGroup['new'])) {
-			$hostGroupsToAdd[] = [
+			$hostgroups_to_add[] = [
 				'id' => $newHostGroup['new'],
 				'name' => $newHostGroup['new'].' ('._x('new', 'new element in multiselect').')',
 				'isNew' => true
 			];
 		}
 		else {
-			$hostGroupIds[] = $newHostGroup;
+			$groupids[] = $newHostGroup;
 		}
 	}
 
-	if (isset($hostGroupIds)) {
-		$getHostGroups = API::HostGroup()->get([
-			'groupids' => $hostGroupIds,
-			'output' => ['groupid', 'name']
-		]);
-		foreach ($getHostGroups as $getHostGroup) {
-			$hostGroupsToAdd[] = [
-				'id' => $getHostGroup['groupid'],
-				'name' => $getHostGroup['name']
-			];
-		}
-	}
+	$hostgroups_to_add = array_merge($hostgroups_to_add, $groupids
+		? CArrayHelper::renameObjectsKeys(API::HostGroup()->get([
+			'output' => ['groupid', 'name'],
+			'groupids' => $groupids
+		]), ['groupid' => 'id'])
+		: []);
 }
-if (CWebUser::getType() == USER_TYPE_SUPER_ADMIN) {
-	$hostFormList->addRow(
-		[
-			_('Add new or existing host groups'),
-			SPACE,
-			(new CVisibilityBox('visible[new_groups]', 'newGroups', _('Original')))
-				->setChecked(isset($data['visible']['new_groups']))
-		],
-		(new CDiv(
-			(new CMultiSelect([
-				'name' => 'new_groups[]',
-				'objectName' => 'hostGroup',
-				'objectOptions' => ['editable' => true],
-				'data' => $hostGroupsToAdd,
-				'addNew' => true,
-				'popup' => [
-					'parameters' => 'srctbl=host_groups&dstfrm='.$hostView->getName().'&dstfld1=new_groups_'.
-						'&srcfld1=groupid&writeonly=1&multiselect=1'
+
+$hostFormList->addRow(
+	(new CVisibilityBox('visible[new_groups]', 'newGroups', _('Original')))
+		->setLabel((CWebUser::getType() == USER_TYPE_SUPER_ADMIN)
+			? _('Add new or existing host groups')
+			: _('New host group')
+		)
+		->setChecked(isset($data['visible']['new_groups'])),
+	(new CDiv(
+		(new CMultiSelect([
+			'name' => 'new_groups[]',
+			'object_name' => 'hostGroup',
+			'add_new' => (CWebUser::getType() == USER_TYPE_SUPER_ADMIN),
+			'data' => $hostgroups_to_add,
+			'popup' => [
+				'parameters' => [
+					'srctbl' => 'host_groups',
+					'srcfld1' => 'groupid',
+					'dstfrm' => $hostView->getName(),
+					'dstfld1' => 'new_groups_',
+					'editable' => true
 				]
-			]))->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
-		))->setId('newGroups')
-	);
-}
-else {
-	$hostFormList->addRow(
-		[
-			_('New host group'),
-			SPACE,
-			(new CVisibilityBox('visible[new_groups]', 'newGroups', _('Original')))
-				->setChecked(isset($data['visible']['new_groups']))
-		],
-		(new CDiv(
-			(new CMultiSelect([
-				'name' => 'new_groups[]',
-				'objectName' => 'hostGroup',
-				'objectOptions' => ['editable' => true],
-				'data' => $hostGroupsToAdd,
-				'popup' => [
-					'parameters' => 'srctbl=host_groups&dstfrm='.$hostView->getName().'&dstfld1=new_groups_'.
-						'&srcfld1=groupid&writeonly=1&multiselect=1'
+			]
+		]))->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
+	))->setId('newGroups')
+);
+
+// Get list of host groups to remove if unsuccessful submit.
+$host_groups_to_remove = getRequest('remove_groups')
+	? CArrayHelper::renameObjectsKeys(API::HostGroup()->get([
+		'output' => ['groupid', 'name'],
+		'groupids' => getRequest('remove_groups'),
+		'editable' => true
+	]), ['groupid' => 'id'])
+	: [];
+
+// Remove host groups control.
+$hostFormList->addRow(
+	(new CVisibilityBox('visible[remove_groups]', 'remove_groups', _('Original')))
+		->setLabel(_('Remove host groups'))
+		->setChecked(array_key_exists('remove_groups', $data['visible'])),
+	(new CDiv(
+		(new CMultiSelect([
+			'name' => 'remove_groups[]',
+			'object_name' => 'hostGroup',
+			'data' => $host_groups_to_remove,
+			'popup' => [
+				'parameters' => [
+					'srctbl' => 'host_groups',
+					'srcfld1' => 'groupid',
+					'dstfrm' => $hostView->getName(),
+					'dstfld1' => 'remove_groups_',
+					'editable' => true
 				]
-			]))->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
-		))->setId('newGroups')
-	);
-}
+			]
+		]))->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
+	))->setId('remove_groups')
+);
 
 // append description to form list
 $hostFormList->addRow(
-	[
-		_('Description'),
-		SPACE,
-		(new CVisibilityBox('visible[description]', 'description', _('Original')))
-			->setChecked(isset($data['visible']['description']))
-	],
+	(new CVisibilityBox('visible[description]', 'description', _('Original')))
+		->setLabel(_('Description'))
+		->setChecked(isset($data['visible']['description'])),
 	(new CTextArea('description', $data['description']))->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
 );
 
@@ -168,23 +170,17 @@ foreach ($data['proxies'] as $proxie) {
 	$proxyComboBox->addItem($proxie['hostid'], $proxie['host']);
 }
 $hostFormList->addRow(
-	[
-		_('Monitored by proxy'),
-		SPACE,
-		(new CVisibilityBox('visible[proxy_hostid]', 'proxy_hostid', _('Original')))
-			->setChecked(isset($data['visible']['proxy_hostid']))
-	],
+	(new CVisibilityBox('visible[proxy_hostid]', 'proxy_hostid', _('Original')))
+		->setLabel(_('Monitored by proxy'))
+		->setChecked(isset($data['visible']['proxy_hostid'])),
 	$proxyComboBox
 );
 
 // append status to form list
 $hostFormList->addRow(
-	[
-		_('Status'),
-		SPACE,
-		(new CVisibilityBox('visible[status]', 'status', _('Original')))
-			->setChecked(isset($data['visible']['status']))
-	],
+	(new CVisibilityBox('visible[status]', 'status', _('Original')))
+		->setLabel(_('Status'))
+		->setChecked(isset($data['visible']['status'])),
 	new CComboBox('status', $data['status'], null, [
 		HOST_STATUS_MONITORED => _('Enabled'),
 		HOST_STATUS_NOT_MONITORED => _('Disabled')
@@ -198,35 +194,36 @@ $newTemplateTable = (new CTable())
 	->addRow([
 		(new CMultiSelect([
 			'name' => 'templates[]',
-			'objectName' => 'templates',
+			'object_name' => 'templates',
 			'data' => $data['linkedTemplates'],
 			'popup' => [
-				'parameters' => 'srctbl=templates&srcfld1=hostid&srcfld2=host&dstfrm='.$hostView->getName().
-					'&dstfld1=templates_&templated_hosts=1&multiselect=1'
+				'parameters' => [
+					'srctbl' => 'templates',
+					'srcfld1' => 'hostid',
+					'srcfld2' => 'host',
+					'dstfrm' => $hostView->getName(),
+					'dstfld1' => 'templates_'
+				]
 			]
 		]))->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
 	])
 	->addRow([
-		(new CDiv([
-			new CLabel(
-				[(new CCheckBox('mass_replace_tpls'))->setChecked($data['mass_replace_tpls'] == 1), _('Replace')],
-				'mass_replace_tpls'
-			),
-			BR(),
-			new CLabel(
-				[(new CCheckBox('mass_clear_tpls'))->setChecked($data['mass_clear_tpls'] == 1), _('Clear when unlinking')],
-				'mass_clear_tpls'
+		(new CList())
+			->addClass(ZBX_STYLE_LIST_CHECK_RADIO)
+			->addItem((new CCheckBox('mass_replace_tpls'))
+				->setLabel(_('Replace'))
+				->setChecked($data['mass_replace_tpls'] == 1)
+			)
+			->addItem((new CCheckBox('mass_clear_tpls'))
+				->setLabel(_('Clear when unlinking'))
+				->setChecked($data['mass_clear_tpls'] == 1)
 			)
-		]))->addClass('floatleft')
 	]);
 
 $templatesFormList->addRow(
-	[
-		_('Link templates'),
-		SPACE,
-		(new CVisibilityBox('visible[templates]', 'templateDiv', _('Original')))
-			->setChecked(isset($data['visible']['templates']))
-	],
+	(new CVisibilityBox('visible[templates]', 'templateDiv', _('Original')))
+		->setLabel(_('Link templates'))
+		->setChecked(isset($data['visible']['templates'])),
 	(new CDiv($newTemplateTable))
 		->setId('templateDiv')
 		->addClass(ZBX_STYLE_TABLE_FORMS_SEPARATOR)
@@ -237,42 +234,30 @@ $ipmiFormList = new CFormList('ipmiFormList');
 
 // append ipmi to form list
 $ipmiFormList->addRow(
-	[
-		_('IPMI authentication algorithm'),
-		SPACE,
-		(new CVisibilityBox('visible[ipmi_authtype]', 'ipmi_authtype', _('Original')))
-			->setChecked(isset($data['visible']['ipmi_authtype']))
-	],
+	(new CVisibilityBox('visible[ipmi_authtype]', 'ipmi_authtype', _('Original')))
+		->setLabel(_('IPMI authentication algorithm'))
+		->setChecked(isset($data['visible']['ipmi_authtype'])),
 	new CComboBox('ipmi_authtype', $data['ipmi_authtype'], null, ipmiAuthTypes())
 );
 
 $ipmiFormList->addRow(
-	[
-		_('IPMI privilege level'),
-		SPACE,
-		(new CVisibilityBox('visible[ipmi_privilege]', 'ipmi_privilege', _('Original')))
-			->setChecked(isset($data['visible']['ipmi_privilege']))
-	],
+	(new CVisibilityBox('visible[ipmi_privilege]', 'ipmi_privilege', _('Original')))
+		->setLabel(_('IPMI privilege level'))
+		->setChecked(isset($data['visible']['ipmi_privilege'])),
 	new CComboBox('ipmi_privilege', $data['ipmi_privilege'], null, ipmiPrivileges())
 );
 
 $ipmiFormList->addRow(
-	[
-		_('IPMI username'),
-		SPACE,
-		(new CVisibilityBox('visible[ipmi_username]', 'ipmi_username', _('Original')))
-			->setChecked(isset($data['visible']['ipmi_username']))
-	],
+	(new CVisibilityBox('visible[ipmi_username]', 'ipmi_username', _('Original')))
+		->setLabel(_('IPMI username'))
+		->setChecked(isset($data['visible']['ipmi_username'])),
 	(new CTextBox('ipmi_username', $data['ipmi_username']))->setWidth(ZBX_TEXTAREA_SMALL_WIDTH)
 );
 
 $ipmiFormList->addRow(
-	[
-		_('IPMI password'),
-		SPACE,
-		(new CVisibilityBox('visible[ipmi_password]', 'ipmi_password', _('Original')))
-			->setChecked(isset($data['visible']['ipmi_password']))
-	],
+	(new CVisibilityBox('visible[ipmi_password]', 'ipmi_password', _('Original')))
+		->setLabel(_('IPMI password'))
+		->setChecked(isset($data['visible']['ipmi_password'])),
 	(new CTextBox('ipmi_password', $data['ipmi_password']))->setWidth(ZBX_TEXTAREA_SMALL_WIDTH)
 );
 
@@ -280,12 +265,9 @@ $inventoryFormList = new CFormList('inventoryFormList');
 
 // append inventories to form list
 $inventoryFormList->addRow(
-	[
-		_('Inventory mode'),
-		SPACE,
-		(new CVisibilityBox('visible[inventory_mode]', 'inventory_mode_div', _('Original')))
-			->setChecked(isset($data['visible']['inventory_mode']))
-	],
+	(new CVisibilityBox('visible[inventory_mode]', 'inventory_mode_div', _('Original')))
+		->setLabel(_('Inventory mode'))
+		->setChecked(isset($data['visible']['inventory_mode'])),
 	(new CDiv(
 		(new CRadioButtonList('inventory_mode', (int) $data['inventory_mode']))
 			->addValue(_('Disabled'), HOST_INVENTORY_DISABLED)
@@ -324,15 +306,9 @@ foreach ($data['inventories'] as $field => $fieldInfo) {
 	}
 
 	$inventoryFormList->addRow(
-		[
-			$fieldInfo['title'],
-			SPACE,
-			(new CVisibilityBox(
-				'visible['.$field.']',
-				'host_inventory['.$field.']',
-				_('Original')
-			))->setChecked(isset($data['visible'][$field]))
-		],
+		(new CVisibilityBox('visible['.$field.']', $fieldInput->getId(), _('Original')))
+			->setLabel($fieldInfo['title'])
+			->setChecked(isset($data['visible'][$field])),
 		$fieldInput, null, 'formrow-inventory'
 	);
 }
@@ -348,13 +324,13 @@ $encryption_table = (new CTable())
 			->addValue(_('Certificate'), HOST_ENCRYPTION_CERTIFICATE)
 			->setModern(true)
 	])
-	->addRow([_('Connections from host'), [
-		[(new CCheckBox('tls_in_none')), _('No encryption')],
-		BR(),
-		[(new CCheckBox('tls_in_psk')), _('PSK')],
-		BR(),
-		[(new CCheckBox('tls_in_cert')), _('Certificate')]
-	]])
+	->addRow([_('Connections from host'),
+		(new CList())
+			->addClass(ZBX_STYLE_LIST_CHECK_RADIO)
+			->addItem((new CCheckBox('tls_in_none'))->setLabel(_('No encryption')))
+			->addItem((new CCheckBox('tls_in_psk'))->setLabel(_('PSK')))
+			->addItem((new CCheckBox('tls_in_cert'))->setLabel(_('Certificate')))
+	])
 	->addRow([_('PSK identity'),
 		(new CTextBox('tls_psk_identity', $data['tls_psk_identity'], false, 128))->setWidth(ZBX_TEXTAREA_BIG_WIDTH)
 	])
@@ -368,12 +344,10 @@ $encryption_table = (new CTable())
 		(new CTextBox('tls_subject', $data['tls_subject'], false, 1024))->setWidth(ZBX_TEXTAREA_BIG_WIDTH)
 	]);
 
-$encryption_form_list->addRow([
-	_('Connections'),
-	SPACE,
+$encryption_form_list->addRow(
 	(new CVisibilityBox('visible[encryption]', 'encryption_div', _('Original')))
-		->setChecked(isset($data['visible']['encryption']))
-	],
+		->setLabel(_('Connections'))
+		->setChecked(isset($data['visible']['encryption'])),
 	(new CDiv($encryption_table))
 		->setId('encryption_div')
 		->addClass(ZBX_STYLE_TABLE_FORMS_SEPARATOR)
