@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2019 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -21,10 +21,15 @@
 
 $widget = (new CWidget())
 	->setTitle(_('Host prototypes'))
-	->setControls((new CForm('get'))
-		->cleanItems()
-		->addVar('parent_discoveryid', $this->data['parent_discoveryid'])
-		->addItem((new CList())->addItem(new CSubmit('form', _('Create host prototype'))))
+	->setControls(
+		(new CTag('nav', true,
+			(new CList())->addItem(new CRedirectButton(_('Create host prototype'),
+				(new CUrl('host_prototypes.php'))
+					->setArgument('form', 'create')
+					->setArgument('parent_discoveryid', $data['parent_discoveryid'])
+					->getUrl()
+			))
+		))->setAttribute('aria-label', _('Content controls'))
 	)
 	->addItem(
 		get_header_host_table('hosts', $this->data['discovery_rule']['hostid'], $this->data['parent_discoveryid'])
@@ -35,37 +40,31 @@ $itemForm = (new CForm())
 	->setName('hosts')
 	->addVar('parent_discoveryid', $this->data['parent_discoveryid']);
 
+$url = (new CUrl('host_prototypes.php'))
+	->setArgument('parent_discoveryid', $data['parent_discoveryid'])
+	->getUrl();
+
 // create table
 $hostTable = (new CTableInfo())
 	->setHeader([
 		(new CColHeader(
 			(new CCheckBox('all_hosts'))->onClick("checkAll('".$itemForm->getName()."', 'all_hosts', 'group_hostid');")
 		))->addClass(ZBX_STYLE_CELL_WIDTH),
-		make_sorting_header(_('Name'), 'name', $this->data['sort'], $this->data['sortorder']),
+		make_sorting_header(_('Name'), 'name', $data['sort'], $data['sortorder'], $url),
 		_('Templates'),
-		make_sorting_header(_('Create enabled'), 'status', $this->data['sort'], $this->data['sortorder'])
+		make_sorting_header(_('Create enabled'), 'status', $data['sort'], $data['sortorder'], $url)
 	]);
 
 foreach ($this->data['hostPrototypes'] as $hostPrototype) {
 	// name
 	$name = [];
-	if ($hostPrototype['templateid']) {
-		$sourceTemplate = $hostPrototype['sourceTemplate'];
-
-		if (array_key_exists($sourceTemplate['hostid'], $data['writable_templates'])) {
-			$name[] = (new CLink($sourceTemplate['name'],
-				'?parent_discoveryid='.$hostPrototype['sourceDiscoveryRuleId']
-			))
-				->addClass(ZBX_STYLE_LINK_ALT)
-				->addClass(ZBX_STYLE_GREY);
-		}
-		else {
-			$name[] = (new CSpan($sourceTemplate['name']))->addClass(ZBX_STYLE_GREY);
-		}
-
-		$name[] = NAME_DELIMITER;
-	}
-	$name[] = new CLink($hostPrototype['name'], '?form=update&parent_discoveryid='.$this->data['discovery_rule']['itemid'].'&hostid='.$hostPrototype['hostid']);
+	$name[] = makeHostPrototypeTemplatePrefix($hostPrototype['hostid'], $data['parent_templates']);
+	$name[] = new CLink(CHtml::encode($hostPrototype['name']),
+		(new CUrl('host_prototypes.php'))
+			->setArgument('form', 'update')
+			->setArgument('parent_discoveryid', $data['discovery_rule']['itemid'])
+			->setArgument('hostid', $hostPrototype['hostid'])
+	);
 
 	// template list
 	if (empty($hostPrototype['templates'])) {
@@ -144,8 +143,6 @@ foreach ($this->data['hostPrototypes'] as $hostPrototype) {
 	]);
 }
 
-zbx_add_post_js('cookie.prefix = "'.$this->data['discovery_rule']['itemid'].'";');
-
 // append table to form
 $itemForm->addItem([
 	$hostTable,
@@ -153,10 +150,10 @@ $itemForm->addItem([
 	new CActionButtonList('action', 'group_hostid',
 		[
 			'hostprototype.massenable' => ['name' => _('Create enabled'),
-				'confirm' => _('Enable selected host prototypes?')
+				'confirm' => _('Create hosts from selected prototypes as enabled?')
 			],
 			'hostprototype.massdisable' => ['name' => _('Create disabled'),
-				'confirm' => _('Disable selected host prototypes?')
+				'confirm' => _('Create hosts from selected prototypes as disabled?')
 			],
 			'hostprototype.massdelete' => ['name' => _('Delete'),
 				'confirm' => _('Delete selected host prototypes?')
