@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2019 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
 **/
 
 
-$filter = (new CFilter())
+$filter = (new CFilter(new CUrl('httpconf.php')))
 	->setProfile($data['profileIdx'])
 	->setActiveTab($data['active_tab'])
 	->addFilterTab(_('Filter'), [
@@ -101,14 +101,13 @@ $httpTests = $this->data['httpTests'];
 
 foreach ($httpTests as $httpTestId => $httpTest) {
 	$name = [];
-	if (isset($this->data['parentTemplates'][$httpTestId])) {
-		$template = $this->data['parentTemplates'][$httpTestId];
-		$name[] = (new CLink($template['name'], '?groupid=0&hostid='.$template['id']))
-			->addClass(ZBX_STYLE_LINK_ALT)
-			->addClass(ZBX_STYLE_GREY);
-		$name[] = NAME_DELIMITER;
-	}
-	$name[] = new CLink($httpTest['name'], '?form=update'.'&httptestid='.$httpTestId.'&hostid='.$httpTest['hostid']);
+	$name[] = makeHttpTestTemplatePrefix($httpTestId, $data['parent_templates']);
+	$name[] = new CLink(CHtml::encode($httpTest['name']),
+		(new CUrl('httpconf.php'))
+			->setArgument('form', 'update')
+			->setArgument('hostid', $httpTest['hostid'])
+			->setArgument('httptestid', $httpTestId)
+	);
 
 	if ($this->data['showInfoColumn']) {
 		$info_icons = [];
@@ -157,8 +156,6 @@ foreach ($httpTests as $httpTestId => $httpTest) {
 	]);
 }
 
-zbx_add_post_js('cookie.prefix = "'.$this->data['hostid'].'";');
-
 // append table to form
 $httpForm->addItem([
 	$httpTable,
