@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2019 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -19,6 +19,34 @@
 **/
 
 
+$filter = new CFilter(new CUrl('templates.php'));
+$filter->setProfile($data['profileIdx'])
+	->setActiveTab($data['active_tab'])
+	->addFilterTab(_('Filter'), [
+		(new CFormList())->addRow(_('Name'),
+			(new CTextBox('filter_name', $data['filter']['name']))
+				->setWidth(ZBX_TEXTAREA_FILTER_STANDARD_WIDTH)
+				->setAttribute('autofocus', 'autofocus')
+		),
+		(new CFormList())->addRow(
+			(new CLabel(_('Linked templates'), 'filter_templates__ms')),
+			(new CMultiSelect([
+				'name' => 'filter_templates[]',
+				'object_name' => 'templates',
+				'data' => $data['filter']['templates'],
+				'popup' => [
+					'parameters' => [
+						'srctbl' => 'templates',
+						'srcfld1' => 'hostid',
+						'srcfld2' => 'host',
+						'dstfrm' => $filter->getName(),
+						'dstfld1' => 'filter_templates_'
+					]
+				]
+			]))->setWidth(ZBX_TEXTAREA_MEDIUM_WIDTH)
+		)
+	]);
+
 $widget = (new CWidget())
 	->setTitle(_('Templates'))
 	->setControls(new CList([
@@ -45,17 +73,7 @@ $widget = (new CWidget())
 				)
 		))->setAttribute('aria-label', _('Content controls'))
 	]))
-	->addItem((new CFilter())
-		->setProfile($data['profileIdx'])
-		->setActiveTab($data['active_tab'])
-		->addFilterTab(_('Filter'), [
-			(new CFormList())->addRow(_('Name'),
-				(new CTextBox('filter_name', $data['filter']['name']))
-					->setWidth(ZBX_TEXTAREA_FILTER_STANDARD_WIDTH)
-					->setAttribute('autofocus', 'autofocus')
-			)
-		])
-	);
+	->addItem($filter);
 
 $form = (new CForm())->setName('templates');
 
@@ -200,7 +218,15 @@ $form->addItem([
 	$data['paging'],
 	new CActionButtonList('action', 'templates',
 		[
-			'template.export' => ['name' => _('Export')],
+			'template.export' => ['name' => _('Export'), 'redirect' =>
+				(new CUrl('zabbix.php'))
+					->setArgument('action', 'export.templates.xml')
+					->setArgument('backurl', (new CUrl('templates.php'))
+						->setArgument('groupid', $data['pageFilter']->groupid)
+						->setArgument('page', getPageNumber())
+						->getUrl())
+					->getUrl()
+			],
 			'template.massdelete' => ['name' => _('Delete'), 'confirm' => _('Delete selected templates?')],
 			'template.massdeleteclear' => ['name' => _('Delete and clear'),
 				'confirm' => _('Delete and clear selected templates? (Warning: all linked hosts will be cleared!)')
