@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2017 Zabbix SIA
+** Copyright (C) 2001-2019 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -28,8 +28,9 @@ $triggersWidget = (new CWidget())
 // create form
 $triggersForm = (new CForm())
 	->setName('triggersForm')
+	->setAttribute('aria-labeledby', ZBX_STYLE_PAGE_TITLE)
 	->addVar('form', $data['form'])
-	->addVar('parent_discoveryid', $data['parent_discoveryid'])
+	->addItem((new CVar('parent_discoveryid', $data['parent_discoveryid']))->removeId())
 	->addVar('expression_constructor', $data['expression_constructor'])
 	->addVar('recovery_expression_constructor', $data['recovery_expression_constructor'])
 	->addVar('toggle_expression_constructor', '')
@@ -45,11 +46,8 @@ if ($data['limited']) {
 	$triggersForm
 		->addVar('recovery_mode', $data['recovery_mode'])
 		->addVar('type', $data['type'])
-		->addVar('correlation_mode', $data['correlation_mode']);
-
-	if ($data['config']['event_ack_enable']) {
-		$triggersForm->addVar('manual_close', $data['manual_close']);
-	}
+		->addVar('correlation_mode', $data['correlation_mode'])
+		->addVar('manual_close', $data['manual_close']);
 }
 
 // create form list
@@ -57,11 +55,13 @@ $triggersFormList = new CFormList('triggersFormList');
 if (!empty($data['templates'])) {
 	$triggersFormList->addRow(_('Parent triggers'), $data['templates']);
 }
-$triggersFormList->addRow(_('Name'),
+$triggersFormList->addRow(
+	(new CLabel(_('Name'), 'description'))->setAsteriskMark(),
 	(new CTextBox('description', $data['description'], $data['limited']))
 		->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
+		->setAriaRequired()
 		->setAttribute('autofocus', 'autofocus')
-)
+	)
 	->addRow(_('Severity'), new CSeverity(['name' => 'priority', 'value' => (int) $data['priority']]));
 
 // append expression to form list
@@ -73,14 +73,24 @@ if ($data['recovery_expression_field_readonly']) {
 	$triggersForm->addVar('recovery_expression', $data['recovery_expression']);
 }
 
+$popup_options = [
+	'srctbl' => 'expression',
+	'srcfld1' => 'expression',
+	'dstfrm' => $triggersForm->getName(),
+	'dstfld1' => $data['expression_field_name'],
+	'parent_discoveryid' => $data['parent_discoveryid']
+];
+if ($data['groupid'] && $data['hostid']) {
+	$popup_options['groupid'] = $data['groupid'];
+	$popup_options['hostid'] = $data['hostid'];
+}
 $add_expression_button = (new CButton('insert', ($data['expression_constructor'] == IM_TREE) ? _('Edit') : _('Add')))
 	->addClass(ZBX_STYLE_BTN_GREY)
-	->onClick(
-		'return PopUp("popup_trexpr.php?dstfrm='.$triggersForm->getName().
-			'&dstfld1='.$data['expression_field_name'].'&srctbl='.$data['expression_field_name'].
-			url_param('parent_discoveryid').'&srcfld1='.$data['expression_field_name'].
-			'&expression="+encodeURIComponent(jQuery(\'[name="'.$data['expression_field_name'].'"]\').val()));'
-	);
+	->onClick('return PopUp("popup.triggerexpr",jQuery.extend('.
+		CJs::encodeJson($popup_options).
+			',{expression: jQuery(\'[name="'.$data['expression_field_name'].'"]\').val()}), null, this);'
+	)
+	->removeId();
 if ($data['limited']) {
 	$add_expression_button->setAttribute('disabled', 'disabled');
 }
@@ -89,7 +99,9 @@ $expression_row = [
 		$data['expression_field_name'],
 		$data['expression_field_value'],
 		['readonly' => $data['expression_field_readonly']]
-	))->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH),
+	))
+		->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
+		->setAriaRequired(),
 	(new CDiv())->addClass(ZBX_STYLE_FORM_INPUT_MARGIN),
 	$add_expression_button
 ];
@@ -146,13 +158,16 @@ elseif ($data['expression_constructor'] != IM_FORCED) {
 	$expression_row[] = [BR(), $input_method_toggle];
 }
 
-$triggersFormList->addRow(_('Expression'), $expression_row, 'expression_row');
+$triggersFormList->addRow(
+	(new CLabel(_('Expression'), $data['expression_field_name']))->setAsteriskMark(),
+	$expression_row,
+	'expression_row'
+);
 
 // Append expression table to form list.
 if ($data['expression_constructor'] == IM_TREE) {
 	$expressionTable = (new CTable())
 		->setAttribute('style', 'width: 100%;')
-		->setId('exp_list')
 		->setHeader([
 			$data['limited'] ? null : _('Target'),
 			_('Expression'),
@@ -184,10 +199,8 @@ if ($data['expression_constructor'] == IM_TREE) {
 			if ($data['limited']) {
 				// make all links inside inactive
 				foreach ($e['list'] as &$obj) {
-					if (gettype($obj) === 'object' && get_class($obj) === 'CSpan'
-							&& $obj->getAttribute('class') == ZBX_STYLE_LINK_ACTION) {
-						$obj->removeAttribute('class');
-						$obj->onClick(null);
+					if ($obj instanceof CLinkAction && $obj->getAttribute('class') == ZBX_STYLE_LINK_ACTION) {
+						$obj = new CSpan($obj->items);
 					}
 				}
 				unset($obj);
@@ -224,11 +237,10 @@ if ($data['expression_constructor'] == IM_TREE) {
 	}
 
 	$testButton = (new CButton('test_expression', _('Test')))
-		->onClick('openWinCentered("tr_testexpr.php?expression="+'.
-			'encodeURIComponent(this.form.elements["expression"].value),'.
-			'"ExpressionTest", 950, 650, "titlebar=no, resizable=yes, scrollbars=yes"); return false;'
-		)
-		->addClass(ZBX_STYLE_BTN_LINK);
+		->onClick('return PopUp("popup.testtriggerexpr",{expression: this.form.elements["expression"].value}, null,'.
+					'this);')
+		->addClass(ZBX_STYLE_BTN_LINK)
+		->removeId();
 	if (!$allowed_testing) {
 		$testButton->setAttribute('disabled', 'disabled');
 	}
@@ -268,11 +280,15 @@ $add_recovery_expression_button = (new CButton('insert',
 		($data['recovery_expression_constructor'] == IM_TREE) ? _('Edit') : _('Add'))
 	)
 	->addClass(ZBX_STYLE_BTN_GREY)
-	->onClick(
-		'return PopUp("popup_trexpr.php?dstfrm='.$triggersForm->getName().
-			'&dstfld1='.$data['recovery_expression_field_name'].url_param('parent_discoveryid').
-			'&srctbl='.$data['recovery_expression_field_name'].'&srcfld1='.$data['recovery_expression_field_name'].
-			'&expression="+encodeURIComponent(jQuery(\'[name="'.$data['recovery_expression_field_name'].'"]\').val()));'
+	->onClick('return PopUp("popup.triggerexpr",jQuery.extend('.
+		CJs::encodeJson([
+			'srctbl' => $data['recovery_expression_field_name'],
+			'srcfld1' => $data['recovery_expression_field_name'],
+			'dstfrm' => $triggersForm->getName(),
+			'dstfld1' => $data['recovery_expression_field_name'],
+			'parent_discoveryid' => $data['parent_discoveryid']
+		]).
+			',{expression: jQuery(\'[name="'.$data['recovery_expression_field_name'].'"]\').val()}), null, this);'
 	);
 
 if ($data['limited']) {
@@ -284,7 +300,9 @@ $recovery_expression_row = [
 		$data['recovery_expression_field_name'],
 		$data['recovery_expression_field_value'],
 		['readonly' => $data['recovery_expression_field_readonly']]
-	))->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH),
+	))
+		->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
+		->setAriaRequired(),
 	(new CDiv())->addClass(ZBX_STYLE_FORM_INPUT_MARGIN),
 	$add_recovery_expression_button
 ];
@@ -333,7 +351,10 @@ elseif ($data['recovery_expression_constructor'] != IM_FORCED) {
 	$recovery_expression_row[] = [BR(), $input_method_toggle];
 }
 
-$triggersFormList->addRow(_('Recovery expression'), $recovery_expression_row, null,
+$triggersFormList->addRow(
+	(new CLabel(_('Recovery expression'), $data['recovery_expression_field_name']))->setAsteriskMark(),
+	$recovery_expression_row,
+	null,
 	'recovery_expression_constructor_row'
 );
 
@@ -341,7 +362,6 @@ $triggersFormList->addRow(_('Recovery expression'), $recovery_expression_row, nu
 if ($data['recovery_expression_constructor'] == IM_TREE) {
 	$recovery_expression_table = (new CTable())
 		->setAttribute('style', 'width: 100%;')
-		->setId('exp_list')
 		->setHeader([
 			$data['limited'] ? null : _('Target'),
 			_('Expression'),
@@ -374,10 +394,8 @@ if ($data['recovery_expression_constructor'] == IM_TREE) {
 			if ($data['limited']) {
 				// make all links inside inactive
 				foreach ($e['list'] as &$obj) {
-					if (gettype($obj) === 'object' && get_class($obj) === 'CSpan'
-							&& $obj->getAttribute('class') == ZBX_STYLE_LINK_ACTION) {
-						$obj->removeAttribute('class');
-						$obj->onClick(null);
+					if ($obj instanceof CLinkAction && $obj->getAttribute('class') == ZBX_STYLE_LINK_ACTION) {
+						$obj = new CSpan($obj->items);
 					}
 				}
 				unset($obj);
@@ -414,10 +432,10 @@ if ($data['recovery_expression_constructor'] == IM_TREE) {
 	}
 
 	$testButton = (new CButton('test_expression', _('Test')))
-		->onClick('openWinCentered("tr_testexpr.php?expression="'.
-			'+encodeURIComponent(this.form.elements["recovery_expression"].value),'.
-		'"ExpressionTest", 950, 650, "titlebar=no, resizable=yes, scrollbars=yes"); return false;')
-		->addClass(ZBX_STYLE_BTN_LINK);
+		->onClick('return PopUp("popup.testtriggerexpr",'.
+			'{expression: this.form.elements["recovery_expression"].value}, null, this);')
+		->addClass(ZBX_STYLE_BTN_LINK)
+		->removeId();
 	if (!$allowed_testing) {
 		$testButton->setAttribute('disabled', 'disabled');
 	}
@@ -461,9 +479,11 @@ $triggersFormList
 			->setEnabled(!$data['limited']),
 		'correlation_mode_row'
 	)
-	->addRow(_('Tag for matching'),
+	->addRow(
+		(new CLabel(_('Tag for matching'), 'correlation_tag'))->setAsteriskMark(),
 		(new CTextBox('correlation_tag', $data['correlation_tag'], $data['limited']))
-			->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH),
+			->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
+			->setAriaRequired(),
 		'correlation_tag_row'
 	);
 
@@ -497,13 +517,11 @@ $triggersFormList->addRow(_('Tags'),
 		->setAttribute('style', 'min-width: '.ZBX_TEXTAREA_STANDARD_WIDTH.'px;')
 );
 
-if ($data['config']['event_ack_enable']) {
-	$triggersFormList->addRow(_('Allow manual close'),
-		(new CCheckBox('manual_close'))
-			->setChecked($data['manual_close'] == ZBX_TRIGGER_MANUAL_CLOSE_ALLOWED)
-			->setEnabled(!$data['limited'])
-	);
-}
+$triggersFormList->addRow(_('Allow manual close'),
+	(new CCheckBox('manual_close'))
+		->setChecked($data['manual_close'] == ZBX_TRIGGER_MANUAL_CLOSE_ALLOWED)
+		->setEnabled(!$data['limited'])
+);
 
 // append status to form list
 if (empty($data['triggerid']) && empty($data['form_refresh'])) {
@@ -558,6 +576,7 @@ foreach ($data['db_dependencies'] as $dependency) {
 			(new CButton('remove', _('Remove')))
 				->onClick('javascript: removeDependency("'.$dependency['triggerid'].'");')
 				->addClass(ZBX_STYLE_BTN_LINK)
+				->removeId()
 		))->addClass(ZBX_STYLE_NOWRAP)
 	]);
 
@@ -570,12 +589,30 @@ $dependenciesFormList->addRow(_('Dependencies'),
 		$dependenciesTable,
 		new CHorList([
 			(new CButton('add_dep_trigger', _('Add')))
-				->onClick('return PopUp("popup.php?srctbl=triggers&srcfld1=triggerid&reference=deptrigger'.
-					'&multiselect=1&with_triggers=1&normal_only=1&noempty=1");')
+				->onClick('return PopUp("popup.generic",'.
+					CJs::encodeJson([
+						'srctbl' => 'triggers',
+						'srcfld1' => 'triggerid',
+						'reference' => 'deptrigger',
+						'multiselect' => '1',
+						'with_triggers' => '1',
+						'normal_only' => '1',
+						'noempty' => '1',
+						'hostid' => $data['hostid'],
+						'groupid' => $data['groupid']
+					]).', null, this);'
+				)
 				->addClass(ZBX_STYLE_BTN_LINK),
 			(new CButton('add_dep_trigger_prototype', _('Add prototype')))
-				->onClick('return PopUp("popup.php?srctbl=trigger_prototypes&srcfld1=triggerid&reference=deptrigger'.
-					url_param('parent_discoveryid').'&multiselect=1");')
+				->onClick('return PopUp("popup.generic",'.
+					CJs::encodeJson([
+						'srctbl' => 'trigger_prototypes',
+						'srcfld1' => 'triggerid',
+						'reference' => 'deptrigger',
+						'multiselect' => '1',
+						'parent_discoveryid' => $data['parent_discoveryid']
+					]).', null, this);'
+				)
 				->addClass(ZBX_STYLE_BTN_LINK)
 		])
 	]))
