@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2019 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -108,7 +108,7 @@ $filter_tags_table->addRow(
 
 $filter_column2 = (new CFormList())->addRow(_('Tags'), $filter_tags_table);
 
-$filter = (new CFilter())
+$filter = (new CFilter(new CUrl('triggers.php')))
 	->setProfile($data['profileIdx'])
 	->setActiveTab($data['active_tab'])
 	->addFilterTab(_('Filter'), [$filter_column1, $filter_column2]);
@@ -183,33 +183,12 @@ foreach ($data['triggers'] as $tnum => $trigger) {
 
 	// description
 	$description = [];
+	$description[] = makeTriggerTemplatePrefix($trigger['triggerid'], $data['parent_templates'],
+		ZBX_FLAG_DISCOVERY_NORMAL
+	);
 
 	$trigger['hosts'] = zbx_toHash($trigger['hosts'], 'hostid');
 
-	if ($trigger['templateid'] > 0) {
-		if (!isset($data['realHosts'][$triggerid])) {
-			$description[] = (new CSpan(_('Host')))->addClass(ZBX_STYLE_GREY);
-			$description[] = NAME_DELIMITER;
-		}
-		else {
-			$real_hosts = $data['realHosts'][$triggerid];
-			$real_host = reset($real_hosts);
-
-			if (array_key_exists($real_host['hostid'], $data['writable_templates'])) {
-				$description[] = (new CLink(CHtml::encode($real_host['name']),
-					'triggers.php?hostid='.$real_host['hostid']
-				))
-					->addClass(ZBX_STYLE_LINK_ALT)
-					->addClass(ZBX_STYLE_GREY);
-			}
-			else {
-				$description[] = (new CSpan(CHtml::encode($real_host['name'])))->addClass(ZBX_STYLE_GREY);
-			}
-
-			$description[] = NAME_DELIMITER;
-		}
-	}
-
 	if ($trigger['discoveryRule']) {
 		$description[] = (new CLink(
 			CHtml::encode($trigger['discoveryRule']['name']),
@@ -311,8 +290,6 @@ foreach ($data['triggers'] as $tnum => $trigger) {
 	]);
 }
 
-zbx_add_post_js('cookie.prefix = "'.$data['hostid'].'";');
-
 // append table to form
 $triggers_form->addItem([
 	$triggers_table,
