@@ -1,7 +1,7 @@
 <script type="text/x-jquery-tmpl" id="delayFlexRow">
 	<tr class="form_row">
 		<td>
-			<ul class="radio-segmented" id="delay_flex_#{rowNum}_type">
+			<ul class="<?= CRadioButtonList::ZBX_STYLE_CLASS ?>" id="delay_flex_#{rowNum}_type">
 				<li>
 					<input type="radio" id="delay_flex_#{rowNum}_type_0" name="delay_flex[#{rowNum}][type]" value="0" checked="checked">
 					<label for="delay_flex_#{rowNum}_type_0"><?= _('Flexible') ?></label>
@@ -12,8 +12,8 @@
 			</ul>
 		</td>
 		<td>
-			<input type="text" id="delay_flex_#{rowNum}_delay" name="delay_flex[#{rowNum}][delay]" maxlength="5" onchange="validateNumericBox(this, true, false);" placeholder="50" style="text-align: right;">
-			<input type="text" id="delay_flex_#{rowNum}_schedule" name="delay_flex[#{rowNum}][schedule]" maxlength="255" placeholder="wd1-5h9-18" style="display: none;">
+			<input type="text" id="delay_flex_#{rowNum}_delay" name="delay_flex[#{rowNum}][delay]" maxlength="255" placeholder="<?= ZBX_ITEM_FLEXIBLE_DELAY_DEFAULT ?>">
+			<input type="text" id="delay_flex_#{rowNum}_schedule" name="delay_flex[#{rowNum}][schedule]" maxlength="255" placeholder="<?= ZBX_ITEM_SCHEDULING_DEFAULT ?>" style="display: none;">
 		</td>
 		<td>
 			<input type="text" id="delay_flex_#{rowNum}_period" name="delay_flex[#{rowNum}][period]" maxlength="255" placeholder="<?= ZBX_DEFAULT_INTERVAL ?>">
@@ -23,12 +23,45 @@
 		</td>
 	</tr>
 </script>
+<?php if (!$data['is_discovery_rule']) : ?>
+	<script type="text/x-jquery-tmpl" id="preprocessing_steps_row">
+	<?php
+		$preproc_types_cbbox = new CComboBox('preprocessing[#{rowNum}][type]', '');
+
+		foreach (get_preprocessing_types() as $group) {
+			$cb_group = new COptGroup($group['label']);
+
+			foreach ($group['types'] as $type => $label) {
+				$cb_group->addItem(new CComboItem($type, $label));
+			}
+
+			$preproc_types_cbbox->addItem($cb_group);
+		}
+
+		echo (new CRow([
+			$readonly
+				? null
+				: (new CCol(
+					(new CDiv())->addClass(ZBX_STYLE_DRAG_ICON)
+				))->addClass(ZBX_STYLE_TD_DRAG_ICON),
+				$preproc_types_cbbox,
+				(new CTextBox('preprocessing[#{rowNum}][params][0]', ''))->setAttribute('placeholder', _('pattern')),
+				(new CTextBox('preprocessing[#{rowNum}][params][1]', ''))->setAttribute('placeholder', _('output')),
+				(new CButton('preprocessing[#{rowNum}][remove]', _('Remove')))
+					->addClass(ZBX_STYLE_BTN_LINK)
+					->addClass('element-table-remove')
+		]))
+			->addClass('sortable')
+			->toString()
+	?>
+	</script>
+<?php endif ?>
 <script type="text/javascript">
 	jQuery(function($) {
 		$('#delayFlexTable').on('click', 'input[type="radio"]', function() {
 			var rowNum = $(this).attr('id').split('_')[2];
 
-			if ($(this).val() == <?= ITEM_DELAY_FLEX_TYPE_FLEXIBLE; ?>) {
+			if ($(this).val() == <?= ITEM_DELAY_FLEXIBLE; ?>) {
 				$('#delay_flex_' + rowNum + '_schedule').hide();
 				$('#delay_flex_' + rowNum + '_delay').show();
 				$('#delay_flex_' + rowNum + '_period').show();
@@ -43,6 +76,108 @@
 		$('#delayFlexTable').dynamicRows({
 			template: '#delayFlexRow'
 		});
+
+		<?php if (!$data['is_discovery_rule']) : ?>
+			var preproc_row_tpl = new Template($('#preprocessing_steps_row').html()),
+				$preprocessing = $('#preprocessing');
+
+			$preprocessing.sortable({
+				disabled: $preprocessing.find('div.<?= ZBX_STYLE_DRAG_ICON ?>').hasClass('<?= ZBX_STYLE_DISABLED ?>'),
+				items: 'tr.sortable',
+				axis: 'y',
+				containment: 'parent',
+				cursor: IE ? 'move' : 'grabbing',
+				handle: 'div.<?= ZBX_STYLE_DRAG_ICON ?>',
+				tolerance: 'pointer',
+				opacity: 0.6,
+				helper: function(e, ui) {
+					ui.children().each(function() {
+						var td = $(this);
+
+						td.width(td.width());
+					});
+
+					return ui;
+				},
+				start: function(e, ui) {
+					$(ui.placeholder).height($(ui.helper).height());
+				}
+			});
+
+			$preprocessing
+				.on('click', '.element-table-add', function() {
+					var row = $(this).parent().parent();
+					row.before(preproc_row_tpl.evaluate({rowNum: $preprocessing.find('tr.sortable').length}));
+
+					var sortable_count = $preprocessing.find('tr.sortable').length;
+
+					if (sortable_count == 1) {
+						$preprocessing
+							.sortable('disable')
+							.find('div.<?= ZBX_STYLE_DRAG_ICON ?>').addClass('<?= ZBX_STYLE_DISABLED ?>');
+					}
+					else if (sortable_count > 1) {
+						$preprocessing
+							.sortable('enable')
+							.find('div.<?= ZBX_STYLE_DRAG_ICON ?>').removeClass('<?= ZBX_STYLE_DISABLED ?>');
+					}
+				})
+				.on('click', '.element-table-remove', function() {
+					var row = $(this).parent().parent();
+					row.remove();
+
+					if ($preprocessing.find('tr.sortable').length < 2) {
+						$preprocessing
+							.sortable('disable')
+							.find('div.<?= ZBX_STYLE_DRAG_ICON ?>').addClass('<?= ZBX_STYLE_DISABLED ?>');
+					}
+				})
+				.on('change', 'select[name*="type"]', function() {
+					var inputs = $(this).parent().parent().find('[name*="params"]');
+
+					switch ($(this).val()) {
+						case '<?= ZBX_PREPROC_MULTIPLIER ?>':
+							$(inputs[0])
+								.show()
+								.attr('placeholder', <?= CJs::encodeJson(_('number')) ?>);
+							$(inputs[1]).hide();
+							break;
+
+						case '<?= ZBX_PREPROC_RTRIM ?>':
+						case '<?= ZBX_PREPROC_LTRIM ?>':
+						case '<?= ZBX_PREPROC_TRIM ?>':
+							$(inputs[0])
+								.show()
+								.attr('placeholder', <?= CJs::encodeJson(_('list of characters')) ?>);
+							$(inputs[1]).hide();
+							break;
+
+						case '<?= ZBX_PREPROC_XPATH ?>':
+						case '<?= ZBX_PREPROC_JSONPATH ?>':
+							$(inputs[0])
+								.show()
+								.attr('placeholder', <?= CJs::encodeJson(_('path')) ?>);
+							$(inputs[1]).hide();
+							break;
+
+						case '<?= ZBX_PREPROC_REGSUB ?>':
+							$(inputs[0])
+								.show()
+								.attr('placeholder', <?= CJs::encodeJson(_('pattern')) ?>);
+							$(inputs[1]).show();
+							break;
+
+						case '<?= ZBX_PREPROC_BOOL2DEC ?>':
+						case '<?= ZBX_PREPROC_OCT2DEC ?>':
+						case '<?= ZBX_PREPROC_HEX2DEC ?>':
+						case '<?= ZBX_PREPROC_DELTA_VALUE ?>':
+						case '<?= ZBX_PREPROC_DELTA_SPEED ?>':
+							$(inputs[0]).hide();
+							$(inputs[1]).hide();
+							break;
+					}
+				});
+		<?php endif ?>
 	});
 </script>
 <?php
@@ -52,22 +187,24 @@
  */
 $this->data['typeVisibility'] = [];
 $i = 0;
+
 foreach ($this->data['delay_flex'] as $delayFlex) {
 	if (!isset($delayFlex['delay']) && !isset($delayFlex['period'])) {
 		continue;
 	}
+
 	foreach ($this->data['types'] as $type => $label) {
-		if ($type == ITEM_TYPE_TRAPPER || $type == ITEM_TYPE_ZABBIX_ACTIVE || $type == ITEM_TYPE_SNMPTRAP) {
+		if ($type == ITEM_TYPE_TRAPPER || $type == ITEM_TYPE_ZABBIX_ACTIVE || $type == ITEM_TYPE_SNMPTRAP
+				|| $type == ITEM_TYPE_DEPENDENT) {
 			continue;
 		}
 		zbx_subarray_push($this->data['typeVisibility'], $type, 'delay_flex['.$i.'][delay]');
 		zbx_subarray_push($this->data['typeVisibility'], $type, 'delay_flex['.$i.'][period]');
 	}
+
 	$i++;
-	if ($i == 7) {
-		break;
-	}
 }
+
 if (!empty($this->data['interfaces'])) {
 	zbx_subarray_push($this->data['typeVisibility'], ITEM_TYPE_ZABBIX, 'interface_row');
 	zbx_subarray_push($this->data['typeVisibility'], ITEM_TYPE_ZABBIX, 'interfaceid');
@@ -91,6 +228,8 @@ if (!empty($this->data['interfaces'])) {
 	zbx_subarray_push($this->data['typeVisibility'], ITEM_TYPE_JMX, 'interfaceid');
 	zbx_subarray_push($this->data['typeVisibility'], ITEM_TYPE_SNMPTRAP, 'interface_row');
 	zbx_subarray_push($this->data['typeVisibility'], ITEM_TYPE_SNMPTRAP, 'interfaceid');
+	zbx_subarray_push($this->data['typeVisibility'], ITEM_TYPE_HTTPAGENT, 'interface_row');
+	zbx_subarray_push($this->data['typeVisibility'], ITEM_TYPE_HTTPAGENT, 'interfaceid');
 }
 zbx_subarray_push($this->data['typeVisibility'], ITEM_TYPE_SIMPLE, 'row_username');
 zbx_subarray_push($this->data['typeVisibility'], ITEM_TYPE_SIMPLE, 'username');
@@ -130,6 +269,8 @@ zbx_subarray_push($this->data['typeVisibility'], ITEM_TYPE_DB_MONITOR, 'username
 zbx_subarray_push($this->data['typeVisibility'], ITEM_TYPE_DB_MONITOR, 'row_username');
 zbx_subarray_push($this->data['typeVisibility'], ITEM_TYPE_JMX, 'username');
 zbx_subarray_push($this->data['typeVisibility'], ITEM_TYPE_JMX, 'row_username');
+zbx_subarray_push($this->data['typeVisibility'], ITEM_TYPE_JMX, 'jmx_endpoint');
+zbx_subarray_push($this->data['typeVisibility'], ITEM_TYPE_JMX, 'row_jmx_endpoint');
 zbx_subarray_push($this->data['typeVisibility'], ITEM_TYPE_SSH, 'password');
 zbx_subarray_push($this->data['typeVisibility'], ITEM_TYPE_SSH, 'row_password');
 zbx_subarray_push($this->data['typeVisibility'], ITEM_TYPE_TELNET, 'password');
@@ -152,6 +293,19 @@ zbx_subarray_push($this->data['typeVisibility'], ITEM_TYPE_CALCULATED, 'params_c
 zbx_subarray_push($this->data['typeVisibility'], ITEM_TYPE_CALCULATED, 'row_params');
 zbx_subarray_push($this->data['typeVisibility'], ITEM_TYPE_TRAPPER, 'trapper_hosts');
 zbx_subarray_push($this->data['typeVisibility'], ITEM_TYPE_TRAPPER, 'row_trapper_hosts');
+zbx_subarray_push($this->data['typeVisibility'], ITEM_TYPE_DEPENDENT, 'row_master_item');
+$ui_rows = [
+	ITEM_TYPE_HTTPAGENT => [
+		'url_row', 'query_fields_row', 'request_method_row', 'timeout_row', 'post_type_row', 'posts_row', 'headers_row',
+		'status_codes_row', 'follow_redirects_row', 'retrieve_mode_row', 'output_format_row', 'allow_traps_row',
+		'request_method', 'http_proxy_row', 'http_authtype_row', 'http_authtype', 'verify_peer_row', 'verify_host_row',
+		'ssl_key_file_row', 'ssl_cert_file_row', 'ssl_key_password_row', 'trapper_hosts', 'allow_traps'
+	]
+];
+foreach ($ui_rows[ITEM_TYPE_HTTPAGENT] as $row) {
+	zbx_subarray_push($this->data['typeVisibility'], ITEM_TYPE_HTTPAGENT, $row);
+}
+
 foreach ($this->data['types'] as $type => $label) {
 	switch ($type) {
 		case ITEM_TYPE_DB_MONITOR:
@@ -172,17 +326,13 @@ foreach ($this->data['types'] as $type => $label) {
 				['id' => 'key', 'defaultValue' => ZBX_DEFAULT_KEY_TELNET]
 			);
 			break;
-		case ITEM_TYPE_JMX:
-			zbx_subarray_push($this->data['typeVisibility'], $type,
-				['id' => 'key', 'defaultValue' => ZBX_DEFAULT_KEY_JMX]
-			);
-			break;
 		default:
 			zbx_subarray_push($this->data['typeVisibility'], $type, ['id' => 'key', 'defaultValue' => '']);
 	}
 }
 foreach ($this->data['types'] as $type => $label) {
-	if ($type == ITEM_TYPE_TRAPPER || $type == ITEM_TYPE_ZABBIX_ACTIVE || $type == ITEM_TYPE_SNMPTRAP) {
+	if ($type == ITEM_TYPE_TRAPPER || $type == ITEM_TYPE_ZABBIX_ACTIVE || $type == ITEM_TYPE_SNMPTRAP
+			|| $type == ITEM_TYPE_DEPENDENT) {
 		continue;
 	}
 	zbx_subarray_push($this->data['typeVisibility'], $type, 'row_flex_intervals');
@@ -192,7 +342,7 @@ foreach ($this->data['types'] as $type => $label) {
 	zbx_subarray_push($this->data['typeVisibility'], $type, 'add_delay_flex');
 }
 foreach ($this->data['types'] as $type => $label) {
-	if ($type == ITEM_TYPE_TRAPPER || $type == ITEM_TYPE_SNMPTRAP) {
+	if ($type == ITEM_TYPE_TRAPPER || $type == ITEM_TYPE_SNMPTRAP || $type == ITEM_TYPE_DEPENDENT) {
 		continue;
 	}
 	zbx_subarray_push($this->data['typeVisibility'], $type, 'delay');
@@ -203,9 +353,6 @@ foreach ($this->data['types'] as $type => $label) {
 foreach ([ITEM_TYPE_CALCULATED, ITEM_TYPE_AGGREGATE] as $type) {
 	// set to disable character, log and text items in value type
 	zbx_subarray_push($this->data['typeDisable'], $type, [ITEM_VALUE_TYPE_STR, ITEM_VALUE_TYPE_LOG, ITEM_VALUE_TYPE_TEXT], 'value_type');
-
-	// disable octal, hexadecimal and boolean items in data_type; Necessary for Numeric (unsigned) value type only
-	zbx_subarray_push($this->data['typeDisable'], $type, [ITEM_DATA_TYPE_OCTAL, ITEM_DATA_TYPE_HEXADECIMAL, ITEM_DATA_TYPE_BOOLEAN], 'data_type');
 }
 
 $this->data['securityLevelVisibility'] = [];
@@ -229,7 +376,6 @@ zbx_subarray_push($this->data['authTypeVisibility'], ITEM_AUTHTYPE_PUBLICKEY, 'p
 zbx_subarray_push($this->data['authTypeVisibility'], ITEM_AUTHTYPE_PUBLICKEY, 'row_privatekey');
 
 ?>
-
 <script type="text/javascript">
 	function setAuthTypeLabel() {
 		if (jQuery('#authtype').val() == <?php echo CJs::encodeJson(ITEM_AUTHTYPE_PUBLICKEY); ?>
@@ -251,12 +397,25 @@ zbx_subarray_push($this->data['authTypeVisibility'], ITEM_AUTHTYPE_PUBLICKEY, 'r
 			var typeSwitcher = new CViewSwitcher('type', 'change',
 				<?php echo zbx_jsvalue($this->data['typeVisibility'], true); ?>,
 				<?php echo zbx_jsvalue($this->data['typeDisable'], true); ?>);
-		<?php }
+		<?php } ?>
+		if (jQuery('#http_authtype').length) {
+			new CViewSwitcher('http_authtype', 'change', <?= zbx_jsvalue([
+				HTTPTEST_AUTH_BASIC => ['http_username_row', 'http_password_row'],
+				HTTPTEST_AUTH_NTLM => ['http_username_row', 'http_password_row']
+			], true) ?>);
+		}
+		<?php
 		if (!empty($this->data['securityLevelVisibility'])) { ?>
 			var securityLevelSwitcher = new CViewSwitcher('snmpv3_securitylevel', 'change',
 				<?php echo zbx_jsvalue($this->data['securityLevelVisibility'], true); ?>);
 		<?php } ?>
 
+		if (jQuery('#allow_traps').length) {
+			new CViewSwitcher('allow_traps', 'change', <?= zbx_jsvalue([
+				HTTPCHECK_ALLOW_TRAPS_ON => ['row_trapper_hosts']
+			], true) ?>);
+		}
+
 		jQuery('#type')
 			.change(function() {
 				// update the interface select with each item type change
@@ -270,5 +429,195 @@ zbx_subarray_push($this->data['authTypeVisibility'], ITEM_AUTHTYPE_PUBLICKEY, 'r
 		jQuery('#authtype').bind('change', function() {
 			setAuthTypeLabel();
 		});
+
+		var $ = jQuery,
+			editableTable = function (elm, tmpl, tmpl_defaults) {
+			var table,
+				row_template,
+				row_default_values,
+				insert_point,
+				row_index = 0,
+				table_row_class = 'editable_table_row';
+
+			table = $(elm);
+			insert_point = table.find('tbody tr[data-insert-point]');
+			row_template = new Template($(tmpl).html());
+			row_default_values = tmpl_defaults;
+
+			table.sortable({
+				disabled: true,
+				items: 'tbody tr.sortable',
+				axis: 'y',
+				containment: 'parent',
+				cursor: IE ? 'move' : 'grabbing',
+				handle: 'div.<?= ZBX_STYLE_DRAG_ICON ?>',
+				tolerance: 'pointer',
+				opacity: 0.6,
+				helper: function(e, ui) {
+					ui.children('td').each(function() {
+						$(this).width($(this).width());
+					});
+
+					return ui;
+				},
+				start: function(e, ui) {
+					// Fix placeholder not to change height while object is being dragged.
+					$(ui.placeholder).height($(ui.helper).height());
+				}
+			});
+
+			table.on('click', '[data-row-action]', function (e) {
+				e.preventDefault();
+
+				switch ($(e.currentTarget).data('row-action')) {
+					case 'remove_row' :
+						removeRow($(e.currentTarget).closest('.' + table_row_class));
+						break;
+
+					case 'add_row' :
+						var row_data = $(e.currentTarget).data('values'),
+							new_row = addRow(row_data || {});
+
+						if (!row_data) {
+							new_row.find('[type="text"]').val('');
+						}
+						break;
+				}
+			});
+
+			/**
+			 * Enable or disable table rows sorting according to rows count. At least 2 rows should exists to be able
+			 * sort rows using drag and drop.
+			 */
+			function setSortableState() {
+				var allow_sort = table.find('.' + table_row_class).length < 2;
+				table.sortable('option', 'disabled', allow_sort);
+			}
+
+			/**
+			 * Add table row. Returns new added row DOM node.
+			 *
+			 * @param {object}  Object with data for added row.
+			 *
+			 * @return {object}
+			 */
+			function addRow(values) {
+				row_index += 1;
+				values.index = row_index;
+
+				var new_row = $(row_template.evaluate(values))
+					.addClass(table_row_class)
+					.addClass('sortable')
+					.data('values', values)
+					.insertBefore(insert_point);
+
+				setSortableState();
+				return new_row;
+			}
+
+			/**
+			 * Add multiple rows to table.
+			 *
+			 * @param {array} rows_values  Array of objects for every added row.
+			 */
+			function addRows(rows_values) {
+				$.each(rows_values, function(index, values) {
+					addRow(values);
+				});
+			}
+
+			/**
+			 * Remove table row.
+			 *
+			 * @param {object} row_node Table row DOM node to be removed.
+			 */
+			function removeRow(row_node) {
+				row_node.remove();
+				setSortableState();
+			}
+
+			return {
+				addRow: function(values) {
+					return addRow(values);
+				},
+				addRows: function(rows_values) {
+					addRows(rows_values);
+					return table;
+				},
+				removeRow: function(row_node) {
+					removeRow(row_node);
+				},
+				getTableRows: function() {
+					return table.find('.' + table_row_class);
+				}
+			};
+		};
+
+		$('[data-sortable-pairs-table]').each(function() {
+			var t = $(this),
+				table = t.find('table'),
+				data = JSON.parse(t.find('[type="text/json"]').text()),
+				template = t.find('[type="text/x-jquery-tmpl"]'),
+				container = new editableTable(table, template);
+
+			container.addRows(data);
+
+			if (t.data('sortable-pairs-table') != 1) {
+				table.sortable('option', 'disabled', true);
+			}
+
+			t.data('editableTable', container);
+		});
+
+		$('[data-action="parse_url"]').click(function() {
+			var url_node = $(this).siblings('[name="url"]'),
+				table = $('#query_fields_pairs').data('editableTable'),
+				url = parseUrlString(url_node.val())
+
+			if (typeof url === 'object') {
+				if (url.pairs.length > 0) {
+					table.addRows(url.pairs);
+					table.getTableRows().map(function() {
+						var empty = $(this).find('input[type="text"]').map(function() {
+							return $(this).val() == '' ? this : null;
+						});
+
+						return empty.length == 2 ? this : null;
+					}).map(function() {
+						table.removeRow(this);
+					});
+				}
+
+				url_node.val(url.url);
+			}
+			else {
+				overlayDialogue({
+					'title': <?= CJs::encodeJson(_('Error')); ?>,
+					'content': $('<span>').html(<?=
+						CJs::encodeJson(_('Failed to parse URL.').'<br><br>'._('URL is not properly encoded.'));
+					?>),
+					'buttons': [
+						{
+							title: <?= CJs::encodeJson(_('Ok')); ?>,
+							class: 'btn-alt',
+							focused: true,
+							action: function() {}
+						}
+					]
+				});
+			}
+		});
+
+		$('#request_method').change(function() {
+			if ($(this).val() == <?= HTTPCHECK_REQUEST_HEAD ?>) {
+				$(':radio', '#retrieve_mode')
+					.filter('[value=<?= HTTPTEST_STEP_RETRIEVE_MODE_HEADERS ?>]').click()
+					.end()
+					.prop('disabled', true);
+			}
+			else {
+				$(':radio', '#retrieve_mode').prop('disabled', false);
+			}
+		});
 	});
 </script>
