@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2019 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -18,43 +18,50 @@
 ** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 **/
 
+
 $widget = (new CWidget())->setTitle(_('Screens'));
+$form = (new CForm('get'))->cleanItems();
 
-$controls = new CList();
-
-if (!$data['templateid']) {
-	$controls->addItem(
-		new CComboBox('config', 'screens.php', 'redirect(this.options[this.selectedIndex].value);', [
-			'screens.php' => _('Screens'),
-			'slides.php' => _('Slide shows')
-		])
-	);
-}
-
-$controls->addItem(new CSubmit('form', _('Create screen')));
-
-$createForm = (new CForm('get'))->cleanItems();
+$content_control = (new CList())->addItem(new CSubmit('form', _('Create screen')));
 
 if ($data['templateid']) {
-	$createForm->addVar('templateid', $data['templateid']);
+	$form->addItem((new CVar('templateid', $data['templateid']))->removeId());
 	$widget->addItem(get_header_host_table('screens', $data['templateid']));
 }
 else {
-	$controls->addItem((new CButton('form', _('Import')))->onClick('redirect("screen.import.php?rules_preset=screen")'));
+	$form->addItem((new CList())
+		->addItem(
+			(new CComboBox('config', 'screens.php', 'redirect(this.options[this.selectedIndex].value);', [
+				'screens.php' => _('Screens'),
+				'slides.php' => _('Slide shows')
+			]))->removeId()
+		)
+	);
+	$content_control->addItem(
+		(new CButton('form', _('Import')))
+			->onClick('redirect("screen.import.php?rules_preset=screen")')
+			->removeId()
+	);
 }
 
-$createForm->addItem($controls);
-$widget->setControls($createForm);
+$form->addItem($content_control);
+$widget->setControls((new CTag('nav', true, $form))
+	->setAttribute('aria-label', _('Content controls'))
+);
 
 // filter
 if (!$data['templateid']) {
 	$widget->addItem(
-		(new CFilter('web.screenconf.filter.state'))
-			->addColumn((new CFormList())->addRow(_('Name'),
-				(new CTextBox('filter_name', $data['filter']['name']))
-					->setWidth(ZBX_TEXTAREA_FILTER_STANDARD_WIDTH)
-					->setAttribute('autofocus', 'autofocus')
-			))
+		(new CFilter(new CUrl('screenconf.php')))
+			->setProfile($data['profileIdx'])
+			->setActiveTab($data['active_tab'])
+			->addFilterTab(_('Filter'), [
+				(new CFormList())->addRow(_('Name'),
+					(new CTextBox('filter_name', $data['filter']['name']))
+						->setWidth(ZBX_TEXTAREA_FILTER_STANDARD_WIDTH)
+						->setAttribute('autofocus', 'autofocus')
+				)
+			])
 	);
 }
 
@@ -106,13 +113,24 @@ foreach ($data['screens'] as $screen) {
 $buttons = [];
 
 if (!$data['templateid']) {
-	$buttons['screen.export'] = ['name' => _('Export')];
+	$buttons['screen.export'] = ['name' => _('Export'), 'redirect' =>
+		(new CUrl('zabbix.php'))
+			->setArgument('action', 'export.screens.xml')
+			->setArgument('backurl', (new CUrl('screenconf.php'))
+				->setArgument('page', getPageNumber())
+				->getUrl())
+			->getUrl()
+	];
 }
 
 $buttons['screen.massdelete'] = ['name' => _('Delete'), 'confirm' => _('Delete selected screens?')];
 
 // append table to form
-$screenForm->addItem([$screenTable, $data['paging'], new CActionButtonList('action', 'screens', $buttons)]);
+$screenForm->addItem([
+	$screenTable,
+	$data['paging'],
+	new CActionButtonList('action', 'screens', $buttons, $data['templateid'] ? $data['templateid'] : null)
+]);
 
 // append form to widget
 $widget->addItem($screenForm);
