@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2019 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -41,6 +41,7 @@ if (typeof (zbx_widget_navtree_trigger) !== typeof (Function)) {
 	$.widget('zbx.sortable_tree', $.extend({}, $.ui.sortable.prototype, {
 		options: {
 			// jQuery UI sortable options:
+			cursor: IE ? 'move' : 'grabbing',
 			placeholder: 'placeholder',
 			forcePlaceholderSize: true,
 			toleranceElement: '> div',
@@ -415,7 +416,7 @@ if (typeof (zbx_widget_navtree_trigger) !== typeof (Function)) {
 			$('.tree-list', item).not(':empty').each(function(i, item) {
 				levels = 0;
 
-				while ($('.tree-list', item).size()) {
+				while ($('.tree-list', item).length) {
 					item = $('.tree-list', item).not(':empty');
 					levels++;
 				}
@@ -604,7 +605,7 @@ jQuery(function($) {
 						map_name: '',
 						map_mapid: 0,
 						depth: depth || 1,
-						map_id: id
+						map_id: +id
 					};
 
 				if (id) {
@@ -623,7 +624,7 @@ jQuery(function($) {
 					data: ajax_data,
 					dataType: 'json',
 					success: function(resp) {
-						var id = ajax_data['map_id'];
+						var id = +ajax_data['map_id'];
 
 						overlayDialogue({
 							'title': t('Edit tree element'),
@@ -638,7 +639,7 @@ jQuery(function($) {
 											ajax_data = {
 												add_submaps: $('[name="add_submaps"]', form).is(':checked') ? 1 : 0,
 												map_name: $('[name="map.name.' + id + '"]', form).val(),
-												map_mapid: +$('[name="linked_map_id"]', form).val(),
+												map_mapid: $('[name="linked_map_id"]', form).val(),
 												depth: depth || 1,
 												mapid: id
 											};
@@ -675,12 +676,13 @@ jQuery(function($) {
 														root = $('.tree-item[data-id=' + parent + ']>ul.tree-list',
 																$obj
 															).get(0),
-														id = +resp['map_id'];
+															id = +resp['map_id'];
+
 														new_item = {
 															name: resp['map_name'],
-															mapid: +resp['map_mapid'],
-															id: +resp['map_id'],
-															parent: parent
+															mapid: resp['map_mapid'],
+															id: id,
+															parent: +parent
 														};
 
 														root.appendChild(createTreeItem($obj, new_item, 1, true, true));
@@ -703,7 +705,7 @@ jQuery(function($) {
 																			submap_itemid = getNextId($obj),
 																			new_item = {
 																				name: submap_item['name'],
-																				mapid: +submap_item['sysmapid'],
+																				mapid: submap_item['sysmapid'],
 																				id: submap_itemid,
 																				parent: +itemid
 																			};
@@ -711,15 +713,13 @@ jQuery(function($) {
 																		root.appendChild(createTreeItem($obj, new_item,
 																			1, true, true
 																		));
-																		add_child_levels($obj, +submapid,
-																			submap_itemid
-																		);
+																		add_child_levels($obj, submapid, submap_itemid);
 																	}
 																});
 															}
 														};
 
-														add_child_levels($obj, +resp['map_mapid'], id);
+														add_child_levels($obj, resp['map_mapid'], id);
 
 														$(root).closest('.tree-item')
 															.addClass('is-parent opened')
@@ -805,7 +805,7 @@ jQuery(function($) {
 					item_clases += ' inaccessible';
 				}
 
-				if (!isEditMode && typeof item.mapid === 'number' && item.mapid > 0 && item.item_active === true) {
+				if (!isEditMode && item.mapid > 0 && item.item_active === true) {
 					var	link = document.createElement('A');
 
 					link.setAttribute('data-mapid', item.mapid);
@@ -816,7 +816,7 @@ jQuery(function($) {
 							step_in_path = $(this).closest('.tree-item'),
 							widget = getWidgetData($obj);
 
-						if ($('.dashbrd-grid-widget-container').dashboardGrid('widgetDataShare', widget,
+						if ($('.dashbrd-grid-container').dashboardGrid('widgetDataShare', widget,
 								'selected_mapid', data_to_share)
 						) {
 							$('.selected', $obj).removeClass('selected');
@@ -887,7 +887,7 @@ jQuery(function($) {
 					btn1.setAttribute('class', 'add-child-btn');
 					btn1.setAttribute('title', t('Add child element'));
 					btn1.addEventListener('click', function(event) {
-						var parentId = $(this).data('id'),
+						var parentId = +$(this).data('id'),
 							widget_data = $obj.data('widgetData'),
 							depth = $(this).closest('.tree-list').attr('data-depth'),
 							branch = $('.tree-item[data-id=' + parentId + ']>ul', $obj);
@@ -909,7 +909,7 @@ jQuery(function($) {
 					btn2.setAttribute('class', 'import-items-btn');
 					btn2.setAttribute('title', t('Add multiple maps'));
 					btn2.addEventListener('click', function(event) {
-						var id = $(this).data('id');
+						var id = +$(this).data('id');
 
 						if (typeof addPopupValues === 'function') {
 							old_addPopupValues = addPopupValues;
@@ -922,7 +922,7 @@ jQuery(function($) {
 							$.each(data.values, function() {
 								new_item = {
 									name: this['name'],
-									mapid: +this['sysmapid'],
+									mapid: this['sysmapid'],
 									id: getNextId($obj),
 									parent: id
 								};
@@ -960,7 +960,7 @@ jQuery(function($) {
 						btn3.setAttribute('class', 'edit-item-btn');
 						btn3.setAttribute('title', t('Edit'));
 						btn3.addEventListener('click', function(event) {
-							var id = $(this).data('id'),
+							var id = +$(this).data('id'),
 								parent = +$('input[name="map.parent.' + id + '"]', $obj).val(),
 								depth = +$(this).closest('[data-depth]').attr('data-depth');
 
@@ -1064,14 +1064,14 @@ jQuery(function($) {
 					parent_fld.setAttribute('type', 'hidden');
 					parent_fld.setAttribute('name', 'map.parent.' + item.id);
 					parent_fld.setAttribute('id', prefix + 'map.parent.' + item.id);
-					parent_fld.value = item.parent || 0;
+					parent_fld.value = +item.parent || 0;
 					li_item.appendChild(parent_fld);
 
 					var mapid_fld = document.createElement('INPUT');
 					mapid_fld.setAttribute('type', 'hidden');
 					mapid_fld.setAttribute('name', 'mapid.' + item.id);
 					mapid_fld.setAttribute('id', prefix + 'mapid.' + item.id);
-					mapid_fld.value = typeof item.mapid === 'number' ? item.mapid : 0;
+					mapid_fld.value = item.mapid;
 					li_item.appendChild(mapid_fld);
 				}
 
@@ -1094,7 +1094,7 @@ jQuery(function($) {
 
 				// Set [data-depth] for list and each sublist.
 				$('.tree-list', $obj).not('.root').each(function() {
-					tree_list_depth = $(this).parents('.tree-list').not('.root').size() + 1;
+					tree_list_depth = $(this).parents('.tree-list').not('.root').length + 1;
 					$(this).attr('data-depth', tree_list_depth);
 				}).promise().done(function() {
 					// Show/hide 'add new items' buttons.
@@ -1129,7 +1129,7 @@ jQuery(function($) {
 
 			var getWidgetData = function($obj) {
 				var widget_data = $obj.data('widgetData'),
-					response = $(".dashbrd-grid-widget-container")
+					response = $(".dashbrd-grid-container")
 						.dashboardGrid('getWidgetsBy', 'uniqueid', widget_data['uniqueid']);
 
 				if (response.length) {
@@ -1146,11 +1146,11 @@ jQuery(function($) {
 			 * @returns {boolean}
 			 */
 			var isEditMode = function() {
-				return $(".dashbrd-grid-widget-container").dashboardGrid('isEditMode');
+				return $(".dashbrd-grid-container").dashboardGrid('isEditMode');
 			};
 
 			/*
-			 * Grouping a seperate widget fields into objects of items. Each items consists of its name, parent, id etc.
+			 * Grouping a separate widget fields into objects of items. Each items consists of its name, parent, id etc.
 			 *
 			 * @returns {Array} - an array of item objects.
 			 */
@@ -1174,7 +1174,7 @@ jQuery(function($) {
 							item.parent = +widget_data['fields']['map.parent.' + item.id];
 						}
 						if (typeof widget_data['fields']['mapid.' + item.id] !== 'undefined') {
-							item.mapid = +widget_data['fields']['mapid.' + item.id];
+							item.mapid = widget_data['fields']['mapid.' + item.id];
 						}
 						if (typeof widget_data['fields']['map.order.' + item.id] !== 'undefined') {
 							item.order = +widget_data['fields']['map.order.' + item.id];
@@ -1189,8 +1189,7 @@ jQuery(function($) {
 
 			// Create multi-level array that represents real child-parent dependencies in tree.
 			var buildTree = function($obj, rows, parent_id) {
-				var parent_id = (typeof parent_id === 'number') ? parent_id : 0,
-					widget_data = $obj.data('widgetData'),
+				var widget_data = $obj.data('widgetData'),
 					rows = rows || [],
 					tree = [];
 
@@ -1257,7 +1256,7 @@ jQuery(function($) {
 				var item = $('[data-id=' + id + ']', $obj),
 					widget_data = $obj.data('widgetData'),
 					prefix = widget_data['uniqueid'] + '_',
-					parent = $('#' + prefix + 'map.parent.' + id, item).val();
+					parent = +$('#' + prefix + 'map.parent.' + id, item).val();
 
 				if ($('#' + prefix + 'children-of-' + parent + '>.tree-item', $obj).length == 1) {
 					$('#' + prefix + 'tree-item-' + parent).removeClass('is-parent');
@@ -1287,7 +1286,7 @@ jQuery(function($) {
 					var id = +field.getAttribute('name').substr(9);
 
 					if (id) {
-						var parent = document.getElementById(prefix + 'map.parent.' + id).value,
+						var parent = +document.getElementById(prefix + 'map.parent.' + id).value,
 							mapid = document.getElementById(prefix + 'mapid.' + id).value,
 							sibl = document.getElementById(prefix + 'children-of-' + parent).childNodes,
 							order = 0;
@@ -1301,7 +1300,7 @@ jQuery(function($) {
 						widget_fields['map.order.' + id] = order + 1;
 
 						if (mapid) {
-							widget_fields['mapid.' + id] = +mapid;
+							widget_fields['mapid.' + id] = mapid;
 						}
 					}
 				});
@@ -1355,7 +1354,7 @@ jQuery(function($) {
 				 * If 'send_data' is set to be 'false', use an unexisting 'data_name', just to check if widget has
 				 * linked widgets, but avoid real data sharing.
 				 */
-				if (item_id && $('.dashbrd-grid-widget-container').dashboardGrid('widgetDataShare', widget,
+				if (item_id && $('.dashbrd-grid-container').dashboardGrid('widgetDataShare', widget,
 						send_data ? 'selected_mapid' : '', {mapid: $(selected_item).data('mapid')})) {
 					$('.selected', $obj).removeClass('selected');
 
@@ -1434,7 +1433,7 @@ jQuery(function($) {
 							triggers = ['onEditStart', 'beforeDashboardSave','beforeConfigLoad', 'onDashboardReady'];
 
 						$.each(triggers, function(index, trigger) {
-							$(".dashbrd-grid-widget-container").dashboardGrid("addAction", trigger,
+							$(".dashbrd-grid-container").dashboardGrid("addAction", trigger,
 								'zbx_widget_navtree_trigger', options.uniqueid, {
 									'parameters': [trigger],
 									'grid': {'widget': 1},
@@ -1448,7 +1447,7 @@ jQuery(function($) {
 							switchToEditMode($this);
 						}
 						else {
-							$('.dashbrd-grid-widget-container').dashboardGrid('registerDataExchange', {
+							$('.dashbrd-grid-container').dashboardGrid('registerDataExchange', {
 								uniqueid: widget_data['uniqueid'],
 								data_name: 'current_sysmapid',
 								callback: function(widget, data) {
