@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2019 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -189,10 +189,10 @@ function validateNumericBox(obj, allowempty, allownegative) {
 /**
  * Validates and formats input element containing a part of date.
  *
- * @param object {obj}			input element value of which is being validated
- * @param int {min}				minimal allowed value (inclusive)
- * @param int {max}				maximum allowed value (inclusive)
- * @param int {paddingSize}		number of zeroes used for padding
+ * @param object {obj}       Input element value of which is being validated.
+ * @param int {min}          Minimal allowed value (inclusive).
+ * @param int {max}          Maximum allowed value (inclusive).
+ * @param int {paddingSize}  Number of zeros used for padding.
  */
 function validateDatePartBox(obj, min, max, paddingSize) {
 	if (obj != null) {
@@ -200,11 +200,11 @@ function validateDatePartBox(obj, min, max, paddingSize) {
 		max = max ? max : 59;
 		paddingSize = paddingSize ? paddingSize : 2;
 
-		var paddingZeroes = [];
+		var paddingZeros = [];
 		for (var i = 0; i != paddingSize; i++) {
-			paddingZeroes.push('0');
+			paddingZeros.push('0');
 		}
-		paddingZeroes = paddingZeroes.join('');
+		paddingZeros = paddingZeros.join('');
 
 		var currentValue = obj.value.toString();
 
@@ -212,15 +212,15 @@ function validateDatePartBox(obj, min, max, paddingSize) {
 			var intValue = parseInt(currentValue, 10);
 
 			if (intValue < min || intValue > max) {
-				obj.value = paddingZeroes;
+				obj.value = paddingZeros;
 			}
 			else if (currentValue.length < paddingSize) {
-				var paddedValue = paddingZeroes + obj.value;
+				var paddedValue = paddingZeros + obj.value;
 				obj.value = paddedValue.substring(paddedValue.length - paddingSize);
 			}
 		}
 		else {
-			obj.value = paddingZeroes;
+			obj.value = paddingZeros;
 		}
 	}
 }
@@ -390,13 +390,13 @@ function formatTimestamp(timestamp, isTsDouble, isExtend) {
 		}
 	}
 
-	var str = (years == 0) ? '' : years + locale['S_YEAR_SHORT'] + ' ';
-	str += (months == 0) ? '' : months + locale['S_MONTH_SHORT'] + ' ';
+	var str = (years == 0) ? '' : years + t('S_YEAR_SHORT') + ' ';
+	str += (months == 0) ? '' : months + t('S_MONTH_SHORT') + ' ';
 	str += (isExtend && isTsDouble)
-		? days + locale['S_DAY_SHORT'] + ' '
-		: ((days == 0) ? '' : days + locale['S_DAY_SHORT'] + ' ');
-	str += (hours == 0) ? '' : hours + locale['S_HOUR_SHORT'] + ' ';
-	str += (minutes == 0) ? '' : minutes + locale['S_MINUTE_SHORT'] + ' ';
+		? days + t('S_DAY_SHORT') + ' '
+		: ((days == 0) ? '' : days + t('S_DAY_SHORT') + ' ');
+	str += (hours == 0) ? '' : hours + t('S_HOUR_SHORT') + ' ';
+	str += (minutes == 0) ? '' : minutes + t('S_MINUTE_SHORT') + ' ';
 
 	return str;
 }
@@ -471,6 +471,23 @@ function stripslashes(str) {
 	});
 }
 
+/**
+ * Function to remove preloader and moves focus to IU element that was clicked to open it.
+ *
+ * @param string   id			Preloader identifier.
+ * @param {object} xhr			(optional) XHR request that must be aborted.
+ */
+function overlayPreloaderDestroy(id, xhr) {
+	if (typeof id !== 'undefined') {
+		if (typeof xhr !== 'undefined') {
+			xhr.abort();
+		}
+
+		jQuery('#' + id).remove();
+		removeFromOverlaysStack(id);
+	}
+}
+
 /**
  * Function to close overlay dialogue and moves focus to IU element that was clicked to open it.
  *
@@ -491,6 +508,7 @@ function overlayDialogueDestroy(dialogueid, xhr) {
 		}
 
 		removeFromOverlaysStack(dialogueid);
+		jQuery.publish('overlay.close', {dialogueid: dialogueid});
 	}
 }
 
@@ -625,7 +643,7 @@ function overlayDialogue(params, trigger_elmnt, xhr) {
 		}
 
 		if ('enabled' in obj && obj.enabled === false) {
-			button.attr('disabled', 'disabled');
+			button.prop('disabled', true);
 		}
 
 		if ('focused' in obj && obj.focused === true) {
@@ -643,7 +661,7 @@ function overlayDialogue(params, trigger_elmnt, xhr) {
 		.append(
 			jQuery('<button>', {
 				class: 'overlay-close-btn',
-				title: t('Close')
+				title: t('S_CLOSE')
 			})
 				.click(function(e) {
 					jQuery('.overlay-bg[data-dialogueid="'+params.dialogueid+'"]').trigger('remove');
