@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2019 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -25,6 +25,10 @@ jQuery(function($) {
 		$layout_mode_btn.on('click', function(e) {
 			e.stopPropagation();
 			updateUserProfile('web.layout.mode', $layout_mode_btn.data('layout-mode'), []).always(function(){
+				var url = new Curl('', false);
+				url.unsetArgument('fullscreen');
+				url.unsetArgument('kiosk');
+				history.replaceState(history.state, '', url.getUrl());
 				location.reload();
 			});
 		});
