@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2019 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -53,12 +53,13 @@ if (!is_array($data) || !isset($data['method'])
 }
 
 $result = [];
+
 switch ($data['method']) {
-	case 'host.get':
+	case 'search':
 		$result = API::Host()->get([
-			'startSearch' => true,
-			'search' => $data['params']['search'],
-			'output' => ['hostid', 'host', 'name'],
+			'output' => ['hostid', 'name'],
+			'search' => ['name' => $data['params']['search'], 'host' => $data['params']['search']],
+			'searchByAny' => true,
 			'sortfield' => 'name',
 			'limit' => 15
 		]);
@@ -85,93 +86,75 @@ switch ($data['method']) {
 	case 'message.get':
 		$msgsettings = getMessageSettings();
 
-		// if no severity is selected, show nothing
 		if (empty($msgsettings['triggers.severities'])) {
 			break;
 		}
 
-		// timeout
 		$timeout = time() - timeUnitToSeconds($msgsettings['timeout']);
 		$lastMsgTime = 0;
 		if (isset($data['params']['messageLast']['events'])) {
 			$lastMsgTime = $data['params']['messageLast']['events']['time'];
 		}
 
-		$options = [
-			'monitored' => true,
-			'lastChangeSince' => max([$lastMsgTime, $msgsettings['last.clock'], $timeout]),
-			'value' => [TRIGGER_VALUE_TRUE, TRIGGER_VALUE_FALSE],
-			'priority' => array_keys($msgsettings['triggers.severities']),
-			'triggerLimit' => 15
-		];
-		if (!$msgsettings['triggers.recovery']) {
-			$options['value'] = [TRIGGER_VALUE_TRUE];
-		}
-		if (!$msgsettings['show_suppressed']) {
-			$options['suppressed'] = $msgsettings['show_suppressed'];
-		}
-		$events = getLastEvents($options);
+		$problems = getLastProblems([
+			'time_from'       => max([$lastMsgTime, $msgsettings['last.clock'], $timeout]),
+			'show_recovered'  => $msgsettings['triggers.recovery'],
+			'show_suppressed' => $msgsettings['show_suppressed'],
+			'severities'      => array_keys($msgsettings['triggers.severities']),
+			'limit'           => 15
+		]);
 
-		$sortClock = [];
-		$sortEvent = [];
+		$used_triggers = [];
+		foreach ($problems as $problem) {
+			if (array_key_exists($problem['triggerid'], $used_triggers)) {
+				continue;
+			}
+			$used_triggers[$problem['triggerid']] = true;
 
-		$usedTriggers = [];
-		foreach ($events as $number => $event) {
-			if (count($usedTriggers) < 15) {
-				if (!isset($usedTriggers[$event['objectid']])) {
-					$trigger = $event['trigger'];
-					$host = $event['host'];
+			$url_problems = (new CUrl('zabbix.php'))
+				->setArgument('action', 'problem.view')
+				->setArgument('filter_hostids[]', $problem['host']['hostid'])
+				->setArgument('filter_set', '1')
+				->getUrl();
+			$url_events = (new CUrl('zabbix.php'))
+				->setArgument('action', 'problem.view')
+				->setArgument('filter_triggerids[]', $problem['objectid'])
+				->setArgument('filter_set', '1')
+				->getUrl();
+			$url_tr_events = 'tr_events.php?eventid='.$problem['eventid'].'&triggerid='.$problem['objectid'];
 
-					if ($event['value'] == TRIGGER_VALUE_FALSE) {
-						$priority = 0;
-						$title = _('Resolved');
-						$sound = $msgsettings['sounds.recovery'];
-					}
-					else {
-						$priority = $trigger['priority'];
-						$title = _('Problem on');
-						$sound = $msgsettings['sounds.'.$trigger['priority']];
-					}
-
-					$url_problems = (new CUrl('zabbix.php'))
-						->setArgument('action', 'problem.view')
-						->setArgument('filter_hostids[]', $host['hostid'])
-						->setArgument('filter_set', '1')
-						->getUrl();
-					$url_events = (new CUrl('zabbix.php'))
-						->setArgument('action', 'problem.view')
-						->setArgument('filter_triggerids[]', $event['objectid'])
-						->setArgument('filter_set', '1')
-						->getUrl();
-					$url_tr_events = 'tr_events.php?eventid='.$event['eventid'].'&triggerid='.$event['objectid'];
-
-					$result[$number] = [
-						'type' => 3,
-						'caption' => 'events',
-						'sourceid' => $event['eventid'],
-						'time' => $event['clock'],
-						'priority' => $priority,
-						'sound' => $sound,
-						'severity_style' => getSeverityStyle($trigger['priority'], $event['value'] == TRIGGER_VALUE_TRUE),
-						'title' => $title.' [url='.$url_problems.']'.CHtml::encode($host['name']).'[/url]',
-						'body' => [
-							'[url='.$url_events.']'.CHtml::encode($trigger['description']).'[/url]',
-							'[url='.$url_tr_events.']'.
-								zbx_date2str(DATE_TIME_FORMAT_SECONDS, $event['clock']).'[/url]',
-						],
-						'timeout' => $msgsettings['timeout']
-					];
-
-					$sortClock[$number] = $event['clock'];
-					$sortEvent[$number] = $event['eventid'];
-					$usedTriggers[$event['objectid']] = true;
-				}
+			if ($problem['resolved']) {
+				$severity = 0;
+				$title = _('Resolved');
+				$sound = $msgsettings['sounds.recovery'];
 			}
 			else {
-				break;
+				$severity = $problem['severity'];
+				$title = _('Problem on');
+				$sound = $msgsettings['sounds.'.$problem['severity']];
 			}
+
+			$result[] = [
+				'type' => 3,
+				'caption' => 'events',
+				'sourceid' => $problem['eventid'],
+				'time' => $problem['clock'],
+				'priority' => $severity,
+				'sound' => $sound,
+				'severity_style' => getSeverityStyle($severity, !$problem['resolved']),
+				'title' => $title.' [url='.$url_problems.']'.CHtml::encode($problem['host']['name']).'[/url]',
+				'body' => [
+					'[url='.$url_events.']'.CHtml::encode($problem['description']).'[/url]',
+					'[url='.$url_tr_events.']'.
+					zbx_date2str(DATE_TIME_FORMAT_SECONDS, $problem['clock']).'[/url]',
+				],
+				'timeout' => $msgsettings['timeout']
+			];
 		}
-		array_multisort($sortClock, SORT_ASC, $sortEvent, SORT_ASC, $result);
+
+		CArrayHelper::sort($result, ['time', 'priority']);
+		$result = array_values($result);
+
 		break;
 
 	case 'message.closeAll':
@@ -253,15 +236,23 @@ switch ($data['method']) {
 
 		switch ($data['objectName']) {
 			case 'hostGroup':
-				$hostGroups = API::HostGroup()->get([
+				$options = [
 					'editable' => array_key_exists('editable', $data) ? $data['editable'] : false,
 					'output' => ['groupid', 'name'],
 					'search' => array_key_exists('search', $data) ? ['name' => $data['search']] : null,
 					'filter' => array_key_exists('filter', $data) ? $data['filter'] : null,
-					'limit' => array_key_exists('limit', $data) ? $data['limit'] : null
-				]);
+					'limit' => array_key_exists('limit', $data) ? $data['limit'] : null,
+					'real_hosts' => array_key_exists('real_hosts', $data) ? $data['real_hosts'] : null
+				];
+				$hostGroups = API::HostGroup()->get($options);
 
 				if ($hostGroups) {
+					if (array_key_exists('enrich_parent_groups', $data)) {
+						$hostGroups = enrichParentGroups($hostGroups, [
+							'real_hosts' => null
+						] + $options);
+					}
+
 					CArrayHelper::sort($hostGroups, [
 						['field' => 'name', 'order' => ZBX_SORT_UP]
 					]);
@@ -495,6 +486,27 @@ switch ($data['method']) {
 				}
 				break;
 
+			case 'drules':
+				$drules = API::DRule()->get([
+					'output' => ['druleid', 'name'],
+					'search' => array_key_exists('search', $data) ? ['name' => $data['search']] : null,
+					'filter' => ['status' => DRULE_STATUS_ACTIVE],
+					'limit' => $config['search_limit']
+				]);
+
+				if ($drules) {
+					CArrayHelper::sort($drules, [
+						['field' => 'name', 'order' => ZBX_SORT_UP]
+					]);
+
+					if (array_key_exists('limit', $data)) {
+						$applications = array_slice($drules, 0, $data['limit']);
+					}
+
+					$result = CArrayHelper::renameObjectsKeys($drules, ['druleid' => 'id']);
+				}
+				break;
+
 		}
 		break;
 
