@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2016 Zabbix SIA
+** Copyright (C) 2001-2019 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -25,7 +25,7 @@ require_once dirname(__FILE__).'/include/html.inc.php';
 
 $page['title'] = _('Search');
 $page['file'] = 'search.php';
-$page['type'] = detect_page_type(PAGE_TYPE_HTML);
+$page['type'] = detect_page_type();
 
 require_once dirname(__FILE__).'/include/page_header.php';
 
@@ -113,7 +113,7 @@ $params = [
 		'dns' => $search,
 		'ip' => $search
 	],
-	'countOutput' => 1,
+	'countOutput' => true,
 	'searchByAny' => true
 ];
 
@@ -126,7 +126,6 @@ $table = (new CTableInfo())
 		_('IP'),
 		_('DNS'),
 		_('Latest data'),
-		_('Triggers'),
 		_('Problems'),
 		_('Graphs'),
 		_('Screens'),
@@ -137,7 +136,8 @@ $table = (new CTableInfo())
 		_('Graphs'),
 		_('Discovery'),
 		_('Web')
-	]);
+	])
+	->removeId();
 
 foreach ($hosts as $hnum => $host) {
 	$hostid = $host['hostid'];
@@ -230,7 +230,6 @@ foreach ($hosts as $hnum => $host) {
 		$hostip,
 		$hostdns,
 		new CLink(_('Latest data'), 'latest.php?filter_set=1&hostids[]='.$hostid),
-		new CLink(_('Triggers'), 'tr_status.php?'.$link),
 		new CLink(_('Problems'),
 			(new CUrl('zabbix.php'))
 				->setArgument('action', 'problem.view')
@@ -251,8 +250,9 @@ foreach ($hosts as $hnum => $host) {
 
 $widgets = [];
 $widgets[] = (new CCollapsibleUiWidget(WIDGET_SEARCH_HOSTS, $table))
+	->addClass(ZBX_STYLE_DASHBRD_WIDGET_FLUID)
 	->setExpanded((bool) CProfile::get('web.search.hats.'.WIDGET_SEARCH_HOSTS.'.state', true))
-	->setHeader(_('Hosts'), [], false, 'search.php')
+	->setHeader(_('Hosts'), [], 'search.php')
 	->setFooter(new CList([_s('Displaying %1$s of %2$s found', $viewCount, $overalCount)]));
 
 // Find Host groups
@@ -278,7 +278,7 @@ $rw_hostGroups = zbx_toHash($rw_hostGroups, 'groupid');
 
 $params = [
 	'search' => ['name' => $search],
-	'countOutput' => 1
+	'countOutput' => true
 ];
 $overalCount = API::HostGroup()->get($params);
 $viewCount = count($hostGroups);
@@ -287,7 +287,6 @@ $table = (new CTableInfo())
 	->setHeader([
 		_('Host group'),
 		_('Latest data'),
-		_('Triggers'),
 		_('Problems'),
 		_('Graphs'),
 		_('Web'),
@@ -337,7 +336,6 @@ foreach ($hostGroups as $hnum => $group) {
 	$table->addRow([
 		$hgroup_link,
 		new CLink(_('Latest data'), 'latest.php?filter_set=1&groupids[]='.$hostgroupid),
-		new CLink(_('Triggers'), 'tr_status.php?'.$link),
 		new CLink(_('Problems'),
 			(new CUrl('zabbix.php'))
 				->setArgument('action', 'problem.view')
@@ -352,8 +350,9 @@ foreach ($hostGroups as $hnum => $group) {
 }
 
 $widgets[] = (new CCollapsibleUiWidget(WIDGET_SEARCH_HOSTGROUP, $table))
+	->addClass(ZBX_STYLE_DASHBRD_WIDGET_FLUID)
 	->setExpanded((bool) CProfile::get('web.search.hats.'.WIDGET_SEARCH_HOSTGROUP.'.state', true))
-	->setHeader(_('Host groups'), [], false, 'search.php')
+	->setHeader(_('Host groups'), [], 'search.php')
 	->setFooter(new CList([_s('Displaying %1$s of %2$s found', $viewCount, $overalCount)]));
 
 // FIND Templates
@@ -396,7 +395,7 @@ if ($admin) {
 			'host' => $search,
 			'name' => $search
 		],
-		'countOutput' => 1,
+		'countOutput' => true,
 		'searchByAny' => true
 	];
 
@@ -493,8 +492,9 @@ if ($admin) {
 	}
 
 	$widgets[] = (new CCollapsibleUiWidget(WIDGET_SEARCH_TEMPLATES, $table))
+		->addClass(ZBX_STYLE_DASHBRD_WIDGET_FLUID)
 		->setExpanded((bool) CProfile::get('web.search.hats.'.WIDGET_SEARCH_TEMPLATES.'.state', true))
-		->setHeader(_('Templates'), [], false, 'search.php')
+		->setHeader(_('Templates'), [], 'search.php')
 		->setFooter(new CList([_s('Displaying %1$s of %2$s found', $viewCount, $overalCount)]));
 }
 
