@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2019 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -20,13 +20,13 @@
 
 
 require_once dirname(__FILE__).'/include/config.inc.php';
-require_once dirname(__FILE__).'/include/hostgroups.inc.php';
 require_once dirname(__FILE__).'/include/hosts.inc.php';
 require_once dirname(__FILE__).'/include/triggers.inc.php';
 require_once dirname(__FILE__).'/include/forms.inc.php';
 
 $page['title'] = _('Configuration of triggers');
 $page['file'] = 'triggers.php';
+$page['scripts'] = ['multiselect.js'];
 
 require_once dirname(__FILE__).'/include/page_header.php';
 
@@ -35,7 +35,12 @@ $fields = [
 	'groupid' =>								[T_ZBX_INT, O_OPT, P_SYS,	DB_ID,			null],
 	'hostid' =>									[T_ZBX_INT, O_OPT, P_SYS,	DB_ID,			null],
 	'triggerid' =>								[T_ZBX_INT, O_OPT, P_SYS,	DB_ID,			'(isset({form}) && ({form} == "update"))'],
-	'copy_type' =>								[T_ZBX_INT, O_OPT, P_SYS,	IN([COPY_TYPE_TO_HOST, COPY_TYPE_TO_TEMPLATE, COPY_TYPE_TO_HOST_GROUP]), 'isset({copy})'],
+	'copy_type' =>								[T_ZBX_INT, O_OPT, P_SYS,
+													IN([COPY_TYPE_TO_HOST_GROUP, COPY_TYPE_TO_HOST,
+														COPY_TYPE_TO_TEMPLATE
+													]),
+													'isset({copy})'
+												],
 	'copy_mode' =>								[T_ZBX_INT, O_OPT, P_SYS,	IN('0'),		null],
 	'type' =>									[T_ZBX_INT, O_OPT, null,	IN('0,1'),		null],
 	'description' =>							[T_ZBX_STR, O_OPT, null,	NOT_EMPTY,		'isset({add}) || isset({update})', _('Name')],
@@ -57,8 +62,7 @@ $fields = [
 	'dependencies' =>							[T_ZBX_INT, O_OPT, null,	DB_ID,			null],
 	'new_dependency' =>							[T_ZBX_INT, O_OPT, null,	DB_ID.'{}>0',	'isset({add_dependency})'],
 	'g_triggerid' =>							[T_ZBX_INT, O_OPT, null,	DB_ID,			null],
-	'copy_targetid' =>							[T_ZBX_INT, O_OPT, null,	DB_ID,			null],
-	'copy_groupid' =>							[T_ZBX_INT, O_OPT, P_SYS,	DB_ID,			'isset({copy}) && (isset({copy_type}) && {copy_type} == 0)'],
+	'copy_targetids' =>							[T_ZBX_INT, O_OPT, null,	DB_ID,			null],
 	'visible' =>								[T_ZBX_STR, O_OPT, null,	null,			null],
 	'tags' =>									[T_ZBX_STR, O_OPT, null,	null,			null],
 	'manual_close' =>							[T_ZBX_INT, O_OPT, null,
@@ -155,14 +159,14 @@ $triggerIds = zbx_toArray($triggerIds);
 if ($triggerIds) {
 	$triggerIds = array_unique($triggerIds);
 
-	$count = API::Trigger()->get([
-		'countOutput' => true,
+	$triggers = API::Trigger()->get([
+		'output' => [],
 		'triggerids' => $triggerIds,
 		'editable' => true
 	]);
 
-	if ($count != count($triggerIds)) {
-		access_deny();
+	if (count($triggers) != count($triggerIds)) {
+		uncheckTableRows(getRequest('hostid'), zbx_objectValues($triggers, 'triggerid'));
 	}
 }
 
@@ -375,10 +379,7 @@ elseif (hasRequest('add') || hasRequest('update')) {
 	}
 }
 elseif (isset($_REQUEST['delete']) && isset($_REQUEST['triggerid'])) {
-	DBstart();
-
 	$result = API::Trigger()->delete([getRequest('triggerid')]);
-	$result = DBend($result);
 
 	if ($result) {
 		unset($_REQUEST['form'], $_REQUEST['triggerid']);
@@ -498,15 +499,15 @@ elseif (hasRequest('action') && str_in_array(getRequest('action'), ['trigger.mas
 }
 elseif (hasRequest('action') && getRequest('action') === 'trigger.masscopyto' && hasRequest('copy')
 		&& hasRequest('g_triggerid')) {
-	if (hasRequest('copy_targetid') && getRequest('copy_targetid') > 0 && hasRequest('copy_type')) {
+	if (getRequest('copy_targetids', []) && hasRequest('copy_type')) {
 		// hosts or templates
 		if (getRequest('copy_type') == COPY_TYPE_TO_HOST || getRequest('copy_type') == COPY_TYPE_TO_TEMPLATE) {
-			$hosts_ids = getRequest('copy_targetid');
+			$hosts_ids = getRequest('copy_targetids');
 		}
 		// host groups
 		else {
 			$hosts_ids = [];
-			$group_ids = getRequest('copy_targetid');
+			$group_ids = getRequest('copy_targetids');
 
 			$db_hosts = DBselect(
 				'SELECT DISTINCT h.hostid'.
@@ -598,9 +599,11 @@ elseif (isset($_REQUEST['form'])) {
 	$triggersView->render();
 	$triggersView->show();
 }
-elseif (hasRequest('action') && getRequest('action') == 'trigger.masscopyto' && hasRequest('g_triggerid')) {
+elseif (hasRequest('action') && getRequest('action') === 'trigger.masscopyto' && hasRequest('g_triggerid')) {
 	$data = getCopyElementsFormData('g_triggerid', _('Triggers'));
 	$data['action'] = 'trigger.masscopyto';
+
+	// render view
 	$triggersView = new CView('configuration.copy.elements', $data);
 	$triggersView->render();
 	$triggersView->show();
@@ -845,28 +848,9 @@ else {
 
 	$data['dependencyTriggers'] = $dependencyTriggers;
 
-	// get real hosts
-	$data['realHosts'] = getParentHostsByTriggers($data['triggers']);
+	$data['parent_templates'] = getTriggerParentTemplates($data['triggers'], ZBX_FLAG_DISCOVERY_NORMAL);
 
-	// Select writable template IDs.
-	$hostids = [];
-
-	foreach ($data['realHosts'] as $realHost) {
-		$hostids = array_merge($hostids, zbx_objectValues($realHost, 'hostid'));
-	}
-
-	$data['writable_templates'] = [];
-
-	if ($hostids) {
-		$data['writable_templates'] = API::Template()->get([
-			'output' => ['templateid'],
-			'templateids' => $hostids,
-			'editable' => true,
-			'preservekeys' => true
-		]);
-	}
-
-	// do not show 'Info' column, if it is a template
+	// Do not show 'Info' column, if it is a template.
 	if ($data['hostid']) {
 		$data['showInfoColumn'] = (bool) API::Host()->get([
 			'output' => [],
